/* Generated by Cython 0.26.1 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "include_dirs": [
            "/usr/lib/python2.7/dist-packages/numpy/core/include"
        ], 
        "language": "c++", 
        "name": "lpu.smt.trans_models.records", 
        "sources": [
            "lpu/smt/trans_models/records.pyx"
        ]
    }, 
    "module_name": "lpu.smt.trans_models.records"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_26_1"
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x03030000 || (PY_MAJOR_VERSION == 2 && PY_VERSION_HEX >= 0x02070000)
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #ifdef __cplusplus
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough) || (defined(__GNUC__) && defined(__attribute__))
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(U other) { return *ptr == other; }
    template<typename U> bool operator !=(U other) { return *ptr != other; }
  private:
    T *ptr;
};

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__lpu__smt__trans_models__records
#define __PYX_HAVE_API__lpu__smt__trans_models__records
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "lpu/smt/trans_models/records.pyx",
  "stringsource",
};

/*--- Type declarations ---*/
struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence;
struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record;
struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord;
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_round;
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts;
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str;
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine;
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str;

/* "lpu/smt/trans_models/records.pyx":42
 *         return CoOccurrence(self.trg, self.src, self.cooc)
 * 
 *     cdef void round(self, digits=6):             # <<<<<<<<<<<<<<
 *         self.src  = round(self.src, digits)
 *         self.trg  = round(self.trg, digits)
 */
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_round {
  int __pyx_n;
  PyObject *digits;
};

/* "lpu/smt/trans_models/records.pyx":47
 *         self.cooc = round(self.cooc, digits)
 * 
 *     cpdef void setCounts(self, object src=None, object trg=None, object cooc=None):             # <<<<<<<<<<<<<<
 *         if src:
 *             self.src = src
 */
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts {
  int __pyx_n;
  PyObject *src;
  PyObject *trg;
  PyObject *cooc;
};

/* "lpu/smt/trans_models/records.pyx":55
 *             self.cooc = cooc
 * 
 *     cpdef str to_str(self, float margin = 0):             # <<<<<<<<<<<<<<
 *         cdef str name = self.__class__.__name__
 *         cdef str mod  = self.__class__.__module__
 */
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str {
  int __pyx_n;
  float margin;
};

/* "lpu/smt/trans_models/records.pyx":328
 * #    cpdef loadLine(self, str line, str delim = '|||'):
 *     #def loadLine(self, line, delim = '|||'):
 *     cpdef loadLine(self, str line, str delim = '|||'):             # <<<<<<<<<<<<<<
 *         cdef list fields
 *         cdef list listCounts
 */
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine {
  int __pyx_n;
  PyObject *delim;
};

/* "lpu/smt/trans_models/records.pyx":404
 * #        return d
 * 
 *     cpdef to_str(self, s = ' ||| '):             # <<<<<<<<<<<<<<
 *       strFeatures = getStrTravatarFeatures(self.features)
 *       self.counts.round()
 */
struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str {
  int __pyx_n;
  PyObject *s;
};

/* "lpu/smt/trans_models/records.pyx":20
 * MATCH=['tree', 'tag', 'string', 'ftree']
 * 
 * cdef class CoOccurrence:             # <<<<<<<<<<<<<<
 *     cdef public float src, trg, cooc
 *     def __cinit__(self, float src=0.0, float trg=0.0, float cooc=0):
 */
struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence {
  PyObject_HEAD
  struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_vtab;
  float src;
  float trg;
  float cooc;
};


/* "lpu/smt/trans_models/records.pyx":66
 *         return self.to_str(0)
 * 
 * cdef class Record(object):             # <<<<<<<<<<<<<<
 *     cdef public str src, trg
 *     #cdef public long src, trg
 */
struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record {
  PyObject_HEAD
  struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record *__pyx_vtab;
  PyObject *src;
  PyObject *trg;
  PyObject *features;
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *counts;
  PyObject *aligns;
  PyObject *data;
};


/* "lpu/smt/trans_models/records.pyx":262
 * 
 * 
 * cdef class TravatarRecord(Record):             # <<<<<<<<<<<<<<
 *     cdef str delim
 * 
 */
struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record __pyx_base;
  PyObject *delim;
};



/* "lpu/smt/trans_models/records.pyx":20
 * MATCH=['tree', 'tag', 'string', 'ftree']
 * 
 * cdef class CoOccurrence:             # <<<<<<<<<<<<<<
 *     cdef public float src, trg, cooc
 *     def __cinit__(self, float src=0.0, float trg=0.0, float cooc=0):
 */

struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence {
  float (*calcEGFP)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch);
  float (*calcFGEP)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch);
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *(*getReversed)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch);
  void (*round)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_round *__pyx_optional_args);
  void (*setCounts)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts *__pyx_optional_args);
  PyObject *(*to_str)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str *__pyx_optional_args);
};
static struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_vtabptr_3lpu_3smt_12trans_models_7records_CoOccurrence;


/* "lpu/smt/trans_models/records.pyx":66
 *         return self.to_str(0)
 * 
 * cdef class Record(object):             # <<<<<<<<<<<<<<
 *     cdef public str src, trg
 *     #cdef public long src, trg
 */

struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record {
  PyObject *(*fixOrder)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch);
  PyObject *(*getAlignMap)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch);
  PyObject *(*getAlignMapRev)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch);
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *(*getReversed)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record *__pyx_vtabptr_3lpu_3smt_12trans_models_7records_Record;


/* "lpu/smt/trans_models/records.pyx":262
 * 
 * 
 * cdef class TravatarRecord(Record):             # <<<<<<<<<<<<<<
 *     cdef str delim
 * 
 */

struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_TravatarRecord {
  struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record __pyx_base;
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *(*getReversed)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, int __pyx_skip_dispatch);
  PyObject *(*loadLine)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine *__pyx_optional_args);
  PyObject *(*syncTags)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, int __pyx_skip_dispatch);
  PyObject *(*to_str)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str *__pyx_optional_args);
};
static struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_vtabptr_3lpu_3smt_12trans_models_7records_TravatarRecord;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_GET();
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* KeywordStringCheck.proto */
static CYTHON_INLINE int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

/* Intern.proto */
static PyObject* __Pyx_Intern(PyObject* s);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o,n,NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* PyObjectCallMethod2.proto */
static PyObject* __Pyx_PyObject_CallMethod2(PyObject* obj, PyObject* method_name, PyObject* arg1, PyObject* arg2);

/* dict_setdefault.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value, int is_safe_type);

/* pyobject_as_double.proto */
static double __Pyx__PyObject_AsDouble(PyObject* obj);
#if CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyObject_AsDouble(obj)\
(likely(PyFloat_CheckExact(obj)) ? PyFloat_AS_DOUBLE(obj) :\
 likely(PyInt_CheckExact(obj)) ?\
 PyFloat_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#else
#define __Pyx_PyObject_AsDouble(obj)\
((likely(PyFloat_CheckExact(obj))) ?\
 PyFloat_AS_DOUBLE(obj) : __Pyx__PyObject_AsDouble(obj))
#endif

/* ArgTypeTest.proto */
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* py_dict_items.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    ((likely((cfunc)->func)) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
             ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
              (PY_VERSION_HEX >= 0x030600B1 && (cfunc)->flag == METH_FASTCALL ?\
                (PY_VERSION_HEX >= 0x030700A0 ?\
                    (*(__Pyx_PyCFunctionFast)(cfunc)->func)(self, &PyTuple_GET_ITEM(__pyx_empty_tuple, 0), 0) :\
                    (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &PyTuple_GET_ITEM(__pyx_empty_tuple, 0), 0, NULL)) :\
              (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
                    (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &PyTuple_GET_ITEM(__pyx_empty_tuple, 0), 0, NULL) :\
                    __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* GetNameInClass.proto */
static PyObject *__Pyx_GetNameInClass(PyObject *nmspace, PyObject *name);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
static int __Pyx_CLineForTraceback(int c_line);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static float __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcEGFP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static float __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcFGEP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_round(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_round *__pyx_optional_args); /* proto*/
static void __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_fixOrder(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMap(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMapRev(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_fixOrder(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed__pyx_wrap_1(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_line, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_syncTags(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str *__pyx_optional_args); /* proto*/

/* Module declarations from 'lpu.smt.trans_models.records' */
static PyTypeObject *__pyx_ptype_3lpu_3smt_12trans_models_7records_CoOccurrence = 0;
static PyTypeObject *__pyx_ptype_3lpu_3smt_12trans_models_7records_Record = 0;
static PyTypeObject *__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord = 0;
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_getStrTravatarFeatures(PyObject *, int __pyx_skip_dispatch); /*proto*/
#define __Pyx_MODULE_NAME "lpu.smt.trans_models.records"
int __pyx_module_is_main_lpu__smt__trans_models__records = 0;

/* Implementation of 'lpu.smt.trans_models.records' */
static PyObject *__pyx_builtin_property;
static PyObject *__pyx_builtin_staticmethod;
static PyObject *__pyx_builtin_round;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_map;
static PyObject *__pyx_builtin_enumerate;
static const char __pyx_k_X[] = ":X";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_e[] = "e";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_p[] = "p";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_t[] = "t";
static const char __pyx_k_w[] = "w";
static const char __pyx_k_x[] = "x";
static const char __pyx_k__3[] = "";
static const char __pyx_k__4[] = " ";
static const char __pyx_k__9[] = "|||";
static const char __pyx_k_co[] = "co";
static const char __pyx_k_x0[] = "x0:";
static const char __pyx_k_X_2[] = "X";
static const char __pyx_k_X_3[] = "[X]";
static const char __pyx_k__10[] = " ||| ";
static const char __pyx_k__11[] = "-";
static const char __pyx_k__20[] = "@";
static const char __pyx_k__30[] = "\"";
static const char __pyx_k__39[] = ":";
static const char __pyx_k__43[] = "=";
static const char __pyx_k__45[] = "(";
static const char __pyx_k_buf[] = "buf";
static const char __pyx_k_d_d[] = "%d-%d";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_egf[] = "egf";
static const char __pyx_k_fge[] = "fge";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_log[] = "log";
static const char __pyx_k_map[] = "map";
static const char __pyx_k_s_2[] = "[%s]";
static const char __pyx_k_s_s[] = "%s=%s";
static const char __pyx_k_src[] = "src";
static const char __pyx_k_tag[] = "tag";
static const char __pyx_k_trg[] = "trg";
static const char __pyx_k_val[] = "val";
static const char __pyx_k_cooc[] = "cooc";
static const char __pyx_k_egfl[] = "egfl";
static const char __pyx_k_egfp[] = "egfp";
static const char __pyx_k_fgel[] = "fgel";
static const char __pyx_k_fgep[] = "fgep";
static const char __pyx_k_find[] = "find";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_line[] = "line";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_math[] = "math";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_rule[] = "rule";
static const char __pyx_k_scfg[] = "scfg";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_sync[] = "sync";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_tree[] = "tree";
static const char __pyx_k_warn[] = "warn";
static const char __pyx_k_MATCH[] = "MATCH";
static const char __pyx_k_align[] = "align";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_delim[] = "delim";
static const char __pyx_k_field[] = "field";
static const char __pyx_k_ftree[] = "ftree";
static const char __pyx_k_hiero[] = "hiero";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_s_s_s[] = "%s %s %s";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_terms[] = "terms";
static const char __pyx_k_aligns[] = "aligns";
static const char __pyx_k_anyNum[] = "anyNum";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_compat[] = "compat";
static const char __pyx_k_counts[] = "counts";
static const char __pyx_k_fields[] = "fields";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_isTree[] = "isTree";
static const char __pyx_k_margin[] = "margin";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_numInt[] = "numInt";
static const char __pyx_k_phrase[] = "phrase";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_scores[] = "scores";
static const char __pyx_k_srcTag[] = "srcTag";
static const char __pyx_k_string[] = "string";
static const char __pyx_k_to_str[] = "to_str";
static const char __pyx_k_changed[] = "changed";
static const char __pyx_k_grammar[] = "grammar";
static const char __pyx_k_isdigit[] = "isdigit";
static const char __pyx_k_logging[] = "logging";
static const char __pyx_k_numbers[] = "numbers";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_reverse[] = "reverse";
static const char __pyx_k_symbols[] = "symbols";
static const char __pyx_k_alignMap[] = "alignMap";
static const char __pyx_k_calcEGFP[] = "calcEGFP";
static const char __pyx_k_calcFGEP[] = "calcFGEP";
static const char __pyx_k_features[] = "features";
static const char __pyx_k_fixOrder[] = "fixOrder";
static const char __pyx_k_getTerms[] = "getTerms";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_loadLine[] = "loadLine";
static const char __pyx_k_numFloat[] = "numFloat";
static const char __pyx_k_property[] = "property";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_srcTerms[] = "srcTerms";
static const char __pyx_k_strField[] = "strField";
static const char __pyx_k_syncTags[] = "syncTags";
static const char __pyx_k_toNumber[] = "toNumber";
static const char __pyx_k_trgTerms[] = "trgTerms";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_getCounts[] = "getCounts";
static const char __pyx_k_getNumber[] = "getNumber";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_newSymbol[] = "newSymbol";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_setCounts[] = "setCounts";
static const char __pyx_k_strAligns[] = "strAligns";
static const char __pyx_k_strCounts[] = "strCounts";
static const char __pyx_k_strKeyVal[] = "strKeyVal";
static const char __pyx_k_getSymbols[] = "getSymbols";
static const char __pyx_k_listCounts[] = "listCounts";
static const char __pyx_k_lpu_common[] = "lpu.common";
static const char __pyx_k_newSymbols[] = "newSymbols";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_setdefault[] = "setdefault";
static const char __pyx_k_trgSymbols[] = "trgSymbols";
static const char __pyx_k_MosesRecord[] = "MosesRecord";
static const char __pyx_k_dicFeatures[] = "dicFeatures";
static const char __pyx_k_getAlignMap[] = "getAlignMap";
static const char __pyx_k_getAlignSet[] = "getAlignSet";
static const char __pyx_k_getReversed[] = "getReversed";
static const char __pyx_k_getSrcTerms[] = "getSrcTerms";
static const char __pyx_k_getTrgTerms[] = "getTrgTerms";
static const char __pyx_k_replaceDict[] = "replaceDict";
static const char __pyx_k_revAlignSet[] = "revAlignSet";
static const char __pyx_k_srcElements[] = "srcElements";
static const char __pyx_k_src_symbols[] = "src_symbols";
static const char __pyx_k_strFeatures[] = "strFeatures";
static const char __pyx_k_trgElements[] = "trgElements";
static const char __pyx_k_staticmethod[] = "staticmethod";
static const char __pyx_k_getSrcSymbols[] = "getSrcSymbols";
static const char __pyx_k_getTrgSymbols[] = "getTrgSymbols";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_getAlignMapRev[] = "getAlignMapRev";
static const char __pyx_k_getRevAlignSet[] = "getRevAlignSet";
static const char __pyx_k_fixOrderStrings[] = "fixOrderStrings";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_srcNonTerminals[] = "srcNonTerminals";
static const char __pyx_k_getMosesFeatures[] = "getMosesFeatures";
static const char __pyx_k_getFlattenSymbols[] = "getFlattenSymbols";
static const char __pyx_k_MosesRecord___init[] = "MosesRecord.__init__";
static const char __pyx_k_MosesRecord_to_str[] = "MosesRecord.to_str";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_getStrMosesFeatures[] = "getStrMosesFeatures";
static const char __pyx_k_getTravatarFeatures[] = "getTravatarFeatures";
static const char __pyx_k_MosesRecord_loadLine[] = "MosesRecord.loadLine";
static const char __pyx_k_s_s_src_s_trg_s_cooc_s[] = "%s.%s(src = %s, trg = %s, cooc = %s)";
static const char __pyx_k_lpu_smt_trans_models_records[] = "lpu.smt.trans_models.records";
static const char __pyx_k_classes_handling_rule_table_reco[] = "classes handling rule table records";
static const char __pyx_k_lpu_smt_trans_models_records_pyx[] = "lpu/smt/trans_models/records.pyx";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static PyObject *__pyx_n_s_MATCH;
static PyObject *__pyx_n_s_MosesRecord;
static PyObject *__pyx_n_s_MosesRecord___init;
static PyObject *__pyx_n_s_MosesRecord_loadLine;
static PyObject *__pyx_n_s_MosesRecord_to_str;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_X;
static PyObject *__pyx_n_s_X_2;
static PyObject *__pyx_kp_s_X_3;
static PyObject *__pyx_kp_s__10;
static PyObject *__pyx_kp_s__11;
static PyObject *__pyx_kp_s__20;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_kp_s__30;
static PyObject *__pyx_kp_s__39;
static PyObject *__pyx_kp_s__4;
static PyObject *__pyx_kp_s__43;
static PyObject *__pyx_kp_s__45;
static PyObject *__pyx_kp_s__9;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_align;
static PyObject *__pyx_n_s_alignMap;
static PyObject *__pyx_n_s_aligns;
static PyObject *__pyx_n_s_anyNum;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_buf;
static PyObject *__pyx_n_s_calcEGFP;
static PyObject *__pyx_n_s_calcFGEP;
static PyObject *__pyx_n_s_changed;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_co;
static PyObject *__pyx_n_s_compat;
static PyObject *__pyx_n_s_cooc;
static PyObject *__pyx_n_s_counts;
static PyObject *__pyx_kp_s_d_d;
static PyObject *__pyx_n_s_delim;
static PyObject *__pyx_n_s_dicFeatures;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_e;
static PyObject *__pyx_n_s_egf;
static PyObject *__pyx_n_s_egfl;
static PyObject *__pyx_n_s_egfp;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_features;
static PyObject *__pyx_n_s_fge;
static PyObject *__pyx_n_s_fgel;
static PyObject *__pyx_n_s_fgep;
static PyObject *__pyx_n_s_field;
static PyObject *__pyx_n_s_fields;
static PyObject *__pyx_n_s_find;
static PyObject *__pyx_n_s_fixOrder;
static PyObject *__pyx_n_s_fixOrderStrings;
static PyObject *__pyx_n_s_ftree;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getAlignMap;
static PyObject *__pyx_n_s_getAlignMapRev;
static PyObject *__pyx_n_s_getAlignSet;
static PyObject *__pyx_n_s_getCounts;
static PyObject *__pyx_n_s_getFlattenSymbols;
static PyObject *__pyx_n_s_getMosesFeatures;
static PyObject *__pyx_n_s_getNumber;
static PyObject *__pyx_n_s_getRevAlignSet;
static PyObject *__pyx_n_s_getReversed;
static PyObject *__pyx_n_s_getSrcSymbols;
static PyObject *__pyx_n_s_getSrcTerms;
static PyObject *__pyx_n_s_getStrMosesFeatures;
static PyObject *__pyx_n_s_getSymbols;
static PyObject *__pyx_n_s_getTerms;
static PyObject *__pyx_n_s_getTravatarFeatures;
static PyObject *__pyx_n_s_getTrgSymbols;
static PyObject *__pyx_n_s_getTrgTerms;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_grammar;
static PyObject *__pyx_n_s_hiero;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_isTree;
static PyObject *__pyx_n_s_isdigit;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_line;
static PyObject *__pyx_n_s_listCounts;
static PyObject *__pyx_n_s_loadLine;
static PyObject *__pyx_n_s_log;
static PyObject *__pyx_n_s_logging;
static PyObject *__pyx_n_s_lpu_common;
static PyObject *__pyx_n_s_lpu_smt_trans_models_records;
static PyObject *__pyx_kp_s_lpu_smt_trans_models_records_pyx;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_map;
static PyObject *__pyx_n_s_margin;
static PyObject *__pyx_n_s_math;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_newSymbol;
static PyObject *__pyx_n_s_newSymbols;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_numFloat;
static PyObject *__pyx_n_s_numInt;
static PyObject *__pyx_n_s_numbers;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_phrase;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_property;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_replaceDict;
static PyObject *__pyx_n_s_revAlignSet;
static PyObject *__pyx_n_s_reverse;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_s_rule;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_kp_s_s_2;
static PyObject *__pyx_kp_s_s_s;
static PyObject *__pyx_kp_s_s_s_s;
static PyObject *__pyx_kp_s_s_s_src_s_trg_s_cooc_s;
static PyObject *__pyx_n_s_scfg;
static PyObject *__pyx_n_s_scores;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_setCounts;
static PyObject *__pyx_n_s_setdefault;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_src;
static PyObject *__pyx_n_s_srcElements;
static PyObject *__pyx_n_s_srcNonTerminals;
static PyObject *__pyx_n_s_srcTag;
static PyObject *__pyx_n_s_srcTerms;
static PyObject *__pyx_n_s_src_symbols;
static PyObject *__pyx_n_s_staticmethod;
static PyObject *__pyx_n_s_strAligns;
static PyObject *__pyx_n_s_strCounts;
static PyObject *__pyx_n_s_strFeatures;
static PyObject *__pyx_n_s_strField;
static PyObject *__pyx_n_s_strKeyVal;
static PyObject *__pyx_n_s_string;
static PyObject *__pyx_n_s_strip;
static PyObject *__pyx_n_s_symbols;
static PyObject *__pyx_n_s_sync;
static PyObject *__pyx_n_s_syncTags;
static PyObject *__pyx_n_s_t;
static PyObject *__pyx_n_s_tag;
static PyObject *__pyx_n_s_terms;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_toNumber;
static PyObject *__pyx_n_s_to_str;
static PyObject *__pyx_n_s_tree;
static PyObject *__pyx_n_s_trg;
static PyObject *__pyx_n_s_trgElements;
static PyObject *__pyx_n_s_trgSymbols;
static PyObject *__pyx_n_s_trgTerms;
static PyObject *__pyx_n_s_val;
static PyObject *__pyx_n_s_w;
static PyObject *__pyx_n_s_warn;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_kp_s_x0;
static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence___cinit__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, float __pyx_v_src, float __pyx_v_trg, float __pyx_v_cooc); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_2calcEGFP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4egfp___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4calcFGEP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4fgep___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_6getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_8setCounts(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_src, PyObject *__pyx_v_trg, PyObject *__pyx_v_cooc); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_10to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, float __pyx_v_margin); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_12__str__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record___cinit__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_2fixOrder(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4getSrcSymbols(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6getSrcTerms(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8getTrgSymbols(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_10getTrgTerms(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_12getAlignMap(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8alignMap___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_14getAlignMapRev(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_11alignMapRev___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_16getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_18getTerms(PyObject *__pyx_v_phrase); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_20getSymbols(PyObject *__pyx_v_phrase); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord_2loadLine(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord_4to_str(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_getAlignMap(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_aligns, PyObject *__pyx_v_reverse); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_2getAlignSet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_strField); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_4getCounts(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_field); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6getNumber(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_anyNum, PyObject *__pyx_v_margin); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_8getRevAlignSet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_aligns); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_10getMosesFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_field); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12getStrMosesFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dicFeatures); /* proto */
static int __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord___cinit__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_2fixOrder(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_4getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_6loadLine(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_8syncTags(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_10to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_12fixOrderStrings(PyObject *__pyx_v_src, PyObject *__pyx_v_trg); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_14getSymbols(PyObject *__pyx_v_rule, PyObject *__pyx_v_hiero); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_16getTerms(PyObject *__pyx_v_symbols); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14getFlattenSymbols(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_symbols, PyObject *__pyx_v_grammar); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_16getStrTravatarFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dict_features); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_18getTravatarFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_field); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_20isTree(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_symbols); /* proto */
static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_22syncTags(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_src_symbols, PyObject *__pyx_v_trgSymbols, PyObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_tp_new_3lpu_3smt_12trans_models_7records_CoOccurrence(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_3lpu_3smt_12trans_models_7records_Record(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_3lpu_3smt_12trans_models_7records_TravatarRecord(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items = {0, &__pyx_n_s_items, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_6;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_slice__21;
static PyObject *__pyx_slice__22;
static PyObject *__pyx_slice__23;
static PyObject *__pyx_slice__24;
static PyObject *__pyx_slice__26;
static PyObject *__pyx_slice__28;
static PyObject *__pyx_slice__31;
static PyObject *__pyx_slice__32;
static PyObject *__pyx_slice__34;
static PyObject *__pyx_slice__37;
static PyObject *__pyx_slice__38;
static PyObject *__pyx_slice__46;
static PyObject *__pyx_slice__47;
static PyObject *__pyx_slice__48;
static PyObject *__pyx_slice__49;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_tuple__71;
static PyObject *__pyx_tuple__73;
static PyObject *__pyx_tuple__75;
static PyObject *__pyx_tuple__77;
static PyObject *__pyx_tuple__79;
static PyObject *__pyx_tuple__81;
static PyObject *__pyx_tuple__83;
static PyObject *__pyx_tuple__85;
static PyObject *__pyx_tuple__87;
static PyObject *__pyx_tuple__89;
static PyObject *__pyx_codeobj__51;
static PyObject *__pyx_codeobj__53;
static PyObject *__pyx_codeobj__55;
static PyObject *__pyx_codeobj__58;
static PyObject *__pyx_codeobj__61;
static PyObject *__pyx_codeobj__64;
static PyObject *__pyx_codeobj__66;
static PyObject *__pyx_codeobj__68;
static PyObject *__pyx_codeobj__70;
static PyObject *__pyx_codeobj__72;
static PyObject *__pyx_codeobj__74;
static PyObject *__pyx_codeobj__76;
static PyObject *__pyx_codeobj__78;
static PyObject *__pyx_codeobj__80;
static PyObject *__pyx_codeobj__82;
static PyObject *__pyx_codeobj__84;
static PyObject *__pyx_codeobj__86;
static PyObject *__pyx_codeobj__88;
static PyObject *__pyx_codeobj__90;

/* "lpu/smt/trans_models/records.pyx":22
 * cdef class CoOccurrence:
 *     cdef public float src, trg, cooc
 *     def __cinit__(self, float src=0.0, float trg=0.0, float cooc=0):             # <<<<<<<<<<<<<<
 *         self.src  = src
 *         self.trg  = trg
 */

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_src;
  float __pyx_v_trg;
  float __pyx_v_cooc;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_src,&__pyx_n_s_trg,&__pyx_n_s_cooc,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_src);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_trg);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_cooc);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 22, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_src = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_src == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 22, __pyx_L3_error)
    } else {
      __pyx_v_src = ((float)0.0);
    }
    if (values[1]) {
      __pyx_v_trg = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_trg == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 22, __pyx_L3_error)
    } else {
      __pyx_v_trg = ((float)0.0);
    }
    if (values[2]) {
      __pyx_v_cooc = __pyx_PyFloat_AsFloat(values[2]); if (unlikely((__pyx_v_cooc == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 22, __pyx_L3_error)
    } else {
      __pyx_v_cooc = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 22, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence___cinit__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self), __pyx_v_src, __pyx_v_trg, __pyx_v_cooc);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence___cinit__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, float __pyx_v_src, float __pyx_v_trg, float __pyx_v_cooc) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "lpu/smt/trans_models/records.pyx":23
 *     cdef public float src, trg, cooc
 *     def __cinit__(self, float src=0.0, float trg=0.0, float cooc=0):
 *         self.src  = src             # <<<<<<<<<<<<<<
 *         self.trg  = trg
 *         self.cooc = cooc
 */
  __pyx_v_self->src = __pyx_v_src;

  /* "lpu/smt/trans_models/records.pyx":24
 *     def __cinit__(self, float src=0.0, float trg=0.0, float cooc=0):
 *         self.src  = src
 *         self.trg  = trg             # <<<<<<<<<<<<<<
 *         self.cooc = cooc
 * 
 */
  __pyx_v_self->trg = __pyx_v_trg;

  /* "lpu/smt/trans_models/records.pyx":25
 *         self.src  = src
 *         self.trg  = trg
 *         self.cooc = cooc             # <<<<<<<<<<<<<<
 * 
 *     # calculate P(e|f) (src -> trg)
 */
  __pyx_v_self->cooc = __pyx_v_cooc;

  /* "lpu/smt/trans_models/records.pyx":22
 * cdef class CoOccurrence:
 *     cdef public float src, trg, cooc
 *     def __cinit__(self, float src=0.0, float trg=0.0, float cooc=0):             # <<<<<<<<<<<<<<
 *         self.src  = src
 *         self.trg  = trg
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":28
 * 
 *     # calculate P(e|f) (src -> trg)
 *     cpdef float calcEGFP(self):             # <<<<<<<<<<<<<<
 *         return self.cooc / self.src
 *     property egfp:
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3calcEGFP(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static float __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcEGFP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch) {
  float __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  float __pyx_t_5;
  __Pyx_RefNannySetupContext("calcEGFP", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_calcEGFP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3calcEGFP)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 28, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":29
 *     # calculate P(e|f) (src -> trg)
 *     cpdef float calcEGFP(self):
 *         return self.cooc / self.src             # <<<<<<<<<<<<<<
 *     property egfp:
 *         def __get__(self): return self.calcEGFP()
 */
  if (unlikely(__pyx_v_self->src == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 29, __pyx_L1_error)
  }
  __pyx_r = (__pyx_v_self->cooc / __pyx_v_self->src);
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":28
 * 
 *     # calculate P(e|f) (src -> trg)
 *     cpdef float calcEGFP(self):             # <<<<<<<<<<<<<<
 *         return self.cooc / self.src
 *     property egfp:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("lpu.smt.trans_models.records.CoOccurrence.calcEGFP", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3calcEGFP(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3calcEGFP(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("calcEGFP (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_2calcEGFP(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_2calcEGFP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("calcEGFP", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcEGFP(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.calcEGFP", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":31
 *         return self.cooc / self.src
 *     property egfp:
 *         def __get__(self): return self.calcEGFP()             # <<<<<<<<<<<<<<
 * 
 *     # calculate P(f|e) (trg -> src)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4egfp_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4egfp_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4egfp___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4egfp___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self->__pyx_vtab)->calcEGFP(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.egfp.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":34
 * 
 *     # calculate P(f|e) (trg -> src)
 *     cpdef float calcFGEP(self):             # <<<<<<<<<<<<<<
 *         return self.cooc / self.trg
 *     property fgep:
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_5calcFGEP(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static float __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcFGEP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch) {
  float __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  float __pyx_t_5;
  __Pyx_RefNannySetupContext("calcFGEP", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_calcFGEP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_5calcFGEP)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 34, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 34, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":35
 *     # calculate P(f|e) (trg -> src)
 *     cpdef float calcFGEP(self):
 *         return self.cooc / self.trg             # <<<<<<<<<<<<<<
 *     property fgep:
 *         def __get__(self): return self.calcFGEP()
 */
  if (unlikely(__pyx_v_self->trg == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 35, __pyx_L1_error)
  }
  __pyx_r = (__pyx_v_self->cooc / __pyx_v_self->trg);
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":34
 * 
 *     # calculate P(f|e) (trg -> src)
 *     cpdef float calcFGEP(self):             # <<<<<<<<<<<<<<
 *         return self.cooc / self.trg
 *     property fgep:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("lpu.smt.trans_models.records.CoOccurrence.calcFGEP", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_5calcFGEP(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_5calcFGEP(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("calcFGEP (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4calcFGEP(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4calcFGEP(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("calcFGEP", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcFGEP(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.calcFGEP", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":37
 *         return self.cooc / self.trg
 *     property fgep:
 *         def __get__(self): return self.calcFGEP()             # <<<<<<<<<<<<<<
 * 
 *     cpdef CoOccurrence getReversed(self):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4fgep_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4fgep_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4fgep___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4fgep___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self->__pyx_vtab)->calcFGEP(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.fgep.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":39
 *         def __get__(self): return self.calcFGEP()
 * 
 *     cpdef CoOccurrence getReversed(self):             # <<<<<<<<<<<<<<
 *         return CoOccurrence(self.trg, self.src, self.cooc)
 * 
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_7getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getReversed", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getReversed); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_7getReversed)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_3lpu_3smt_12trans_models_7records_CoOccurrence))))) __PYX_ERR(0, 39, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":40
 * 
 *     cpdef CoOccurrence getReversed(self):
 *         return CoOccurrence(self.trg, self.src, self.cooc)             # <<<<<<<<<<<<<<
 * 
 *     cdef void round(self, digits=6):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->trg); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->src); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->cooc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_CoOccurrence), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":39
 *         def __get__(self): return self.calcFGEP()
 * 
 *     cpdef CoOccurrence getReversed(self):             # <<<<<<<<<<<<<<
 *         return CoOccurrence(self.trg, self.src, self.cooc)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.getReversed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_7getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_7getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getReversed (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_6getReversed(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_6getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getReversed", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_getReversed(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.getReversed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":42
 *         return CoOccurrence(self.trg, self.src, self.cooc)
 * 
 *     cdef void round(self, digits=6):             # <<<<<<<<<<<<<<
 *         self.src  = round(self.src, digits)
 *         self.trg  = round(self.trg, digits)
 */

static void __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_round(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_round *__pyx_optional_args) {
  PyObject *__pyx_v_digits = ((PyObject *)__pyx_int_6);
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  float __pyx_t_3;
  __Pyx_RefNannySetupContext("round", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_digits = __pyx_optional_args->digits;
    }
  }

  /* "lpu/smt/trans_models/records.pyx":43
 * 
 *     cdef void round(self, digits=6):
 *         self.src  = round(self.src, digits)             # <<<<<<<<<<<<<<
 *         self.trg  = round(self.trg, digits)
 *         self.cooc = round(self.cooc, digits)
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->src); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_digits);
  __Pyx_GIVEREF(__pyx_v_digits);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_digits);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->src = __pyx_t_3;

  /* "lpu/smt/trans_models/records.pyx":44
 *     cdef void round(self, digits=6):
 *         self.src  = round(self.src, digits)
 *         self.trg  = round(self.trg, digits)             # <<<<<<<<<<<<<<
 *         self.cooc = round(self.cooc, digits)
 * 
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->trg); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_digits);
  __Pyx_GIVEREF(__pyx_v_digits);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_digits);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->trg = __pyx_t_3;

  /* "lpu/smt/trans_models/records.pyx":45
 *         self.src  = round(self.src, digits)
 *         self.trg  = round(self.trg, digits)
 *         self.cooc = round(self.cooc, digits)             # <<<<<<<<<<<<<<
 * 
 *     cpdef void setCounts(self, object src=None, object trg=None, object cooc=None):
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->cooc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_digits);
  __Pyx_GIVEREF(__pyx_v_digits);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_digits);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->cooc = __pyx_t_3;

  /* "lpu/smt/trans_models/records.pyx":42
 *         return CoOccurrence(self.trg, self.src, self.cooc)
 * 
 *     cdef void round(self, digits=6):             # <<<<<<<<<<<<<<
 *         self.src  = round(self.src, digits)
 *         self.trg  = round(self.trg, digits)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("lpu.smt.trans_models.records.CoOccurrence.round", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "lpu/smt/trans_models/records.pyx":47
 *         self.cooc = round(self.cooc, digits)
 * 
 *     cpdef void setCounts(self, object src=None, object trg=None, object cooc=None):             # <<<<<<<<<<<<<<
 *         if src:
 *             self.src = src
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_9setCounts(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static void __pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts *__pyx_optional_args) {
  PyObject *__pyx_v_src = ((PyObject *)Py_None);
  PyObject *__pyx_v_trg = ((PyObject *)Py_None);
  PyObject *__pyx_v_cooc = ((PyObject *)Py_None);
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  float __pyx_t_8;
  __Pyx_RefNannySetupContext("setCounts", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_src = __pyx_optional_args->src;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_trg = __pyx_optional_args->trg;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_cooc = __pyx_optional_args->cooc;
        }
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setCounts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_9setCounts)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_src, __pyx_v_trg, __pyx_v_cooc};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_src, __pyx_v_trg, __pyx_v_cooc};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 47, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_src);
        __Pyx_GIVEREF(__pyx_v_src);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_src);
        __Pyx_INCREF(__pyx_v_trg);
        __Pyx_GIVEREF(__pyx_v_trg);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_trg);
        __Pyx_INCREF(__pyx_v_cooc);
        __Pyx_GIVEREF(__pyx_v_cooc);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_cooc);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":48
 * 
 *     cpdef void setCounts(self, object src=None, object trg=None, object cooc=None):
 *         if src:             # <<<<<<<<<<<<<<
 *             self.src = src
 *         if trg:
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_src); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 48, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "lpu/smt/trans_models/records.pyx":49
 *     cpdef void setCounts(self, object src=None, object trg=None, object cooc=None):
 *         if src:
 *             self.src = src             # <<<<<<<<<<<<<<
 *         if trg:
 *             self.trg = trg
 */
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_v_src); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 49, __pyx_L1_error)
    __pyx_v_self->src = __pyx_t_8;

    /* "lpu/smt/trans_models/records.pyx":48
 * 
 *     cpdef void setCounts(self, object src=None, object trg=None, object cooc=None):
 *         if src:             # <<<<<<<<<<<<<<
 *             self.src = src
 *         if trg:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":50
 *         if src:
 *             self.src = src
 *         if trg:             # <<<<<<<<<<<<<<
 *             self.trg = trg
 *         if cooc:
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_trg); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 50, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "lpu/smt/trans_models/records.pyx":51
 *             self.src = src
 *         if trg:
 *             self.trg = trg             # <<<<<<<<<<<<<<
 *         if cooc:
 *             self.cooc = cooc
 */
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_v_trg); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 51, __pyx_L1_error)
    __pyx_v_self->trg = __pyx_t_8;

    /* "lpu/smt/trans_models/records.pyx":50
 *         if src:
 *             self.src = src
 *         if trg:             # <<<<<<<<<<<<<<
 *             self.trg = trg
 *         if cooc:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":52
 *         if trg:
 *             self.trg = trg
 *         if cooc:             # <<<<<<<<<<<<<<
 *             self.cooc = cooc
 * 
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_cooc); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 52, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "lpu/smt/trans_models/records.pyx":53
 *             self.trg = trg
 *         if cooc:
 *             self.cooc = cooc             # <<<<<<<<<<<<<<
 * 
 *     cpdef str to_str(self, float margin = 0):
 */
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_v_cooc); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 53, __pyx_L1_error)
    __pyx_v_self->cooc = __pyx_t_8;

    /* "lpu/smt/trans_models/records.pyx":52
 *         if trg:
 *             self.trg = trg
 *         if cooc:             # <<<<<<<<<<<<<<
 *             self.cooc = cooc
 * 
 */
  }

  /* "lpu/smt/trans_models/records.pyx":47
 *         self.cooc = round(self.cooc, digits)
 * 
 *     cpdef void setCounts(self, object src=None, object trg=None, object cooc=None):             # <<<<<<<<<<<<<<
 *         if src:
 *             self.src = src
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("lpu.smt.trans_models.records.CoOccurrence.setCounts", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_9setCounts(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_9setCounts(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_src = 0;
  PyObject *__pyx_v_trg = 0;
  PyObject *__pyx_v_cooc = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setCounts (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_src,&__pyx_n_s_trg,&__pyx_n_s_cooc,0};
    PyObject* values[3] = {0,0,0};
    values[0] = ((PyObject *)Py_None);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_src);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_trg);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_cooc);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "setCounts") < 0)) __PYX_ERR(0, 47, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_src = values[0];
    __pyx_v_trg = values[1];
    __pyx_v_cooc = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setCounts", 0, 0, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 47, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.setCounts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_8setCounts(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self), __pyx_v_src, __pyx_v_trg, __pyx_v_cooc);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_8setCounts(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_src, PyObject *__pyx_v_trg, PyObject *__pyx_v_cooc) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("setCounts", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1.__pyx_n = 3;
  __pyx_t_1.src = __pyx_v_src;
  __pyx_t_1.trg = __pyx_v_trg;
  __pyx_t_1.cooc = __pyx_v_cooc;
  __pyx_vtabptr_3lpu_3smt_12trans_models_7records_CoOccurrence->setCounts(__pyx_v_self, 1, &__pyx_t_1); 
  __pyx_t_2 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.setCounts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":55
 *             self.cooc = cooc
 * 
 *     cpdef str to_str(self, float margin = 0):             # <<<<<<<<<<<<<<
 *         cdef str name = self.__class__.__name__
 *         cdef str mod  = self.__class__.__module__
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_11to_str(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str *__pyx_optional_args) {
  float __pyx_v_margin = ((float)0.0);
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_mod = 0;
  PyObject *__pyx_v_src = 0;
  PyObject *__pyx_v_trg = 0;
  PyObject *__pyx_v_cooc = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("to_str", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_margin = __pyx_optional_args->margin;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_to_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_11to_str)) {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_margin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 55, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 55, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 55, __pyx_L1_error)
      __pyx_r = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":56
 * 
 *     cpdef str to_str(self, float margin = 0):
 *         cdef str name = self.__class__.__name__             # <<<<<<<<<<<<<<
 *         cdef str mod  = self.__class__.__module__
 *         cdef str src  = str(numbers.toNumber(self.src,margin))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 56, __pyx_L1_error)
  __pyx_v_name = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":57
 *     cpdef str to_str(self, float margin = 0):
 *         cdef str name = self.__class__.__name__
 *         cdef str mod  = self.__class__.__module__             # <<<<<<<<<<<<<<
 *         cdef str src  = str(numbers.toNumber(self.src,margin))
 *         cdef str trg  = str(numbers.toNumber(self.trg,margin))
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_module); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 57, __pyx_L1_error)
  __pyx_v_mod = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":58
 *         cdef str name = self.__class__.__name__
 *         cdef str mod  = self.__class__.__module__
 *         cdef str src  = str(numbers.toNumber(self.src,margin))             # <<<<<<<<<<<<<<
 *         cdef str trg  = str(numbers.toNumber(self.trg,margin))
 *         cdef str cooc = str(numbers.toNumber(self.cooc,margin))
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_numbers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_toNumber); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->src); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_margin); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_2, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_2, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_2 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 58, __pyx_L1_error)
  __pyx_v_src = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":59
 *         cdef str mod  = self.__class__.__module__
 *         cdef str src  = str(numbers.toNumber(self.src,margin))
 *         cdef str trg  = str(numbers.toNumber(self.trg,margin))             # <<<<<<<<<<<<<<
 *         cdef str cooc = str(numbers.toNumber(self.cooc,margin))
 *         return "%s.%s(src = %s, trg = %s, cooc = %s)" % (mod, name, src, trg, cooc)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_numbers); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_toNumber); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->trg); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_margin); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 59, __pyx_L1_error)
  __pyx_v_trg = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":60
 *         cdef str src  = str(numbers.toNumber(self.src,margin))
 *         cdef str trg  = str(numbers.toNumber(self.trg,margin))
 *         cdef str cooc = str(numbers.toNumber(self.cooc,margin))             # <<<<<<<<<<<<<<
 *         return "%s.%s(src = %s, trg = %s, cooc = %s)" % (mod, name, src, trg, cooc)
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_numbers); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_toNumber); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->cooc); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_margin); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_5, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_5, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_2 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 60, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_7, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 60, __pyx_L1_error)
  __pyx_v_cooc = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":61
 *         cdef str trg  = str(numbers.toNumber(self.trg,margin))
 *         cdef str cooc = str(numbers.toNumber(self.cooc,margin))
 *         return "%s.%s(src = %s, trg = %s, cooc = %s)" % (mod, name, src, trg, cooc)             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_mod);
  __Pyx_GIVEREF(__pyx_v_mod);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_mod);
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_src);
  __Pyx_GIVEREF(__pyx_v_src);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_src);
  __Pyx_INCREF(__pyx_v_trg);
  __Pyx_GIVEREF(__pyx_v_trg);
  PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_trg);
  __Pyx_INCREF(__pyx_v_cooc);
  __Pyx_GIVEREF(__pyx_v_cooc);
  PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_v_cooc);
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_s_s_src_s_trg_s_cooc_s, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_3))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":55
 *             self.cooc = cooc
 * 
 *     cpdef str to_str(self, float margin = 0):             # <<<<<<<<<<<<<<
 *         cdef str name = self.__class__.__name__
 *         cdef str mod  = self.__class__.__module__
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_src);
  __Pyx_XDECREF(__pyx_v_trg);
  __Pyx_XDECREF(__pyx_v_cooc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_11to_str(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_11to_str(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_margin;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("to_str (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_margin,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_margin);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "to_str") < 0)) __PYX_ERR(0, 55, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_margin = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_margin == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 55, __pyx_L3_error)
    } else {
      __pyx_v_margin = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("to_str", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 55, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_10to_str(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self), __pyx_v_margin);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_10to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, float __pyx_v_margin) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str __pyx_t_2;
  __Pyx_RefNannySetupContext("to_str", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.margin = __pyx_v_margin;
  __pyx_t_1 = __pyx_vtabptr_3lpu_3smt_12trans_models_7records_CoOccurrence->to_str(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":63
 *         return "%s.%s(src = %s, trg = %s, cooc = %s)" % (mod, name, src, trg, cooc)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.to_str(0)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_13__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_13__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_12__str__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_12__str__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str __pyx_t_2;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "lpu/smt/trans_models/records.pyx":64
 * 
 *     def __str__(self):
 *         return self.to_str(0)             # <<<<<<<<<<<<<<
 * 
 * cdef class Record(object):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.margin = 0.0;
  __pyx_t_1 = ((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self->__pyx_vtab)->to_str(__pyx_v_self, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":63
 *         return "%s.%s(src = %s, trg = %s, cooc = %s)" % (mod, name, src, trg, cooc)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.to_str(0)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":21
 * 
 * cdef class CoOccurrence:
 *     cdef public float src, trg, cooc             # <<<<<<<<<<<<<<
 *     def __cinit__(self, float src=0.0, float trg=0.0, float cooc=0):
 *         self.src  = src
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->src); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.src.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_v_self->src = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.src.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->trg); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.trg.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_v_self->trg = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.trg.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->cooc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.cooc.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_v_self->cooc = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.cooc.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_14__reduce_cython__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_16__setstate_cython__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12CoOccurrence_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.CoOccurrence.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":74
 *     cdef public object data
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *       self.src = ""
 *       self.trg = ""
 */

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record___cinit__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record___cinit__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "lpu/smt/trans_models/records.pyx":75
 * 
 *     def __cinit__(self):
 *       self.src = ""             # <<<<<<<<<<<<<<
 *       self.trg = ""
 *       #self.src = ()
 */
  __Pyx_INCREF(__pyx_kp_s__3);
  __Pyx_GIVEREF(__pyx_kp_s__3);
  __Pyx_GOTREF(__pyx_v_self->src);
  __Pyx_DECREF(__pyx_v_self->src);
  __pyx_v_self->src = __pyx_kp_s__3;

  /* "lpu/smt/trans_models/records.pyx":76
 *     def __cinit__(self):
 *       self.src = ""
 *       self.trg = ""             # <<<<<<<<<<<<<<
 *       #self.src = ()
 *       #self.trg = ()
 */
  __Pyx_INCREF(__pyx_kp_s__3);
  __Pyx_GIVEREF(__pyx_kp_s__3);
  __Pyx_GOTREF(__pyx_v_self->trg);
  __Pyx_DECREF(__pyx_v_self->trg);
  __pyx_v_self->trg = __pyx_kp_s__3;

  /* "lpu/smt/trans_models/records.pyx":81
 *       #self.src = 0
 *       #self.trg = 0
 *       self.features = {}             # <<<<<<<<<<<<<<
 *       self.counts = CoOccurrence()
 * #      self.aligns = []
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->features);
  __Pyx_DECREF(__pyx_v_self->features);
  __pyx_v_self->features = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":82
 *       #self.trg = 0
 *       self.features = {}
 *       self.counts = CoOccurrence()             # <<<<<<<<<<<<<<
 * #      self.aligns = []
 *       self.aligns = set()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_CoOccurrence), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->counts);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->counts));
  __pyx_v_self->counts = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":84
 *       self.counts = CoOccurrence()
 * #      self.aligns = []
 *       self.aligns = set()             # <<<<<<<<<<<<<<
 * 
 *     cpdef fixOrder(self):
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->aligns);
  __Pyx_DECREF(__pyx_v_self->aligns);
  __pyx_v_self->aligns = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":74
 *     cdef public object data
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *       self.src = ""
 *       self.trg = ""
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":86
 *       self.aligns = set()
 * 
 *     cpdef fixOrder(self):             # <<<<<<<<<<<<<<
 *         return False
 * 
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3fixOrder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_fixOrder(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("fixOrder", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_fixOrder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3fixOrder)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":87
 * 
 *     cpdef fixOrder(self):
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     def getSrcSymbols(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":86
 *       self.aligns = set()
 * 
 *     cpdef fixOrder(self):             # <<<<<<<<<<<<<<
 *         return False
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.fixOrder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3fixOrder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3fixOrder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fixOrder (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_2fixOrder(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_2fixOrder(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("fixOrder", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_3lpu_3smt_12trans_models_7records_6Record_fixOrder(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.fixOrder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":89
 *         return False
 * 
 *     def getSrcSymbols(self):             # <<<<<<<<<<<<<<
 *         return self.getSymbols(self.src)
 *     src_symbols = property(getSrcSymbols)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_5getSrcSymbols(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_5getSrcSymbols(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getSrcSymbols (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4getSrcSymbols(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4getSrcSymbols(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getSrcSymbols", 0);

  /* "lpu/smt/trans_models/records.pyx":90
 * 
 *     def getSrcSymbols(self):
 *         return self.getSymbols(self.src)             # <<<<<<<<<<<<<<
 *     src_symbols = property(getSrcSymbols)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->src); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->src};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->src};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self->src);
      __Pyx_GIVEREF(__pyx_v_self->src);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self->src);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":89
 *         return False
 * 
 *     def getSrcSymbols(self):             # <<<<<<<<<<<<<<
 *         return self.getSymbols(self.src)
 *     src_symbols = property(getSrcSymbols)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getSrcSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":93
 *     src_symbols = property(getSrcSymbols)
 * 
 *     def getSrcTerms(self):             # <<<<<<<<<<<<<<
 *         return self.getTerms(self.src)
 *     srcTerms = property(getSrcTerms)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_7getSrcTerms(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_7getSrcTerms(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getSrcTerms (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6getSrcTerms(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6getSrcTerms(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getSrcTerms", 0);

  /* "lpu/smt/trans_models/records.pyx":94
 * 
 *     def getSrcTerms(self):
 *         return self.getTerms(self.src)             # <<<<<<<<<<<<<<
 *     srcTerms = property(getSrcTerms)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getTerms); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->src); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->src};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->src};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self->src);
      __Pyx_GIVEREF(__pyx_v_self->src);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self->src);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":93
 *     src_symbols = property(getSrcSymbols)
 * 
 *     def getSrcTerms(self):             # <<<<<<<<<<<<<<
 *         return self.getTerms(self.src)
 *     srcTerms = property(getSrcTerms)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getSrcTerms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":97
 *     srcTerms = property(getSrcTerms)
 * 
 *     def getTrgSymbols(self):             # <<<<<<<<<<<<<<
 *         return self.getSymbols(self.trg)
 *     trgSymbols = property(getTrgSymbols)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_9getTrgSymbols(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_9getTrgSymbols(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getTrgSymbols (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8getTrgSymbols(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8getTrgSymbols(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getTrgSymbols", 0);

  /* "lpu/smt/trans_models/records.pyx":98
 * 
 *     def getTrgSymbols(self):
 *         return self.getSymbols(self.trg)             # <<<<<<<<<<<<<<
 *     trgSymbols = property(getTrgSymbols)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->trg); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->trg};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->trg};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 98, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self->trg);
      __Pyx_GIVEREF(__pyx_v_self->trg);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self->trg);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":97
 *     srcTerms = property(getSrcTerms)
 * 
 *     def getTrgSymbols(self):             # <<<<<<<<<<<<<<
 *         return self.getSymbols(self.trg)
 *     trgSymbols = property(getTrgSymbols)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getTrgSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":101
 *     trgSymbols = property(getTrgSymbols)
 * 
 *     def getTrgTerms(self):             # <<<<<<<<<<<<<<
 *         return self.getTerms(self.trg)
 *     trgTerms = property(getTrgTerms)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_11getTrgTerms(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_11getTrgTerms(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getTrgTerms (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_10getTrgTerms(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_10getTrgTerms(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getTrgTerms", 0);

  /* "lpu/smt/trans_models/records.pyx":102
 * 
 *     def getTrgTerms(self):
 *         return self.getTerms(self.trg)             # <<<<<<<<<<<<<<
 *     trgTerms = property(getTrgTerms)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getTerms); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->trg); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->trg};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->trg};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 102, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self->trg);
      __Pyx_GIVEREF(__pyx_v_self->trg);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self->trg);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":101
 *     trgSymbols = property(getTrgSymbols)
 * 
 *     def getTrgTerms(self):             # <<<<<<<<<<<<<<
 *         return self.getTerms(self.trg)
 *     trgTerms = property(getTrgTerms)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getTrgTerms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":105
 *     trgTerms = property(getTrgTerms)
 * 
 *     cpdef getAlignMap(self):             # <<<<<<<<<<<<<<
 *         return getAlignMap(self.aligns, reverse = False)
 *     #alignMap = property(getAlignMap)
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_13getAlignMap(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMap(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getAlignMap", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getAlignMap); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_13getAlignMap)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":106
 * 
 *     cpdef getAlignMap(self):
 *         return getAlignMap(self.aligns, reverse = False)             # <<<<<<<<<<<<<<
 *     #alignMap = property(getAlignMap)
 *     property alignMap:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_getAlignMap); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->aligns);
  __Pyx_GIVEREF(__pyx_v_self->aligns);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->aligns);
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_reverse, Py_False) < 0) __PYX_ERR(0, 106, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":105
 *     trgTerms = property(getTrgTerms)
 * 
 *     cpdef getAlignMap(self):             # <<<<<<<<<<<<<<
 *         return getAlignMap(self.aligns, reverse = False)
 *     #alignMap = property(getAlignMap)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getAlignMap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_13getAlignMap(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_13getAlignMap(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getAlignMap (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_12getAlignMap(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_12getAlignMap(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getAlignMap", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMap(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getAlignMap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":109
 *     #alignMap = property(getAlignMap)
 *     property alignMap:
 *         def __get__(self): return self.getAlignMap()             # <<<<<<<<<<<<<<
 * 
 *     cpdef getAlignMapRev(self):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8alignMap_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8alignMap_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8alignMap___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8alignMap___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self->__pyx_vtab)->getAlignMap(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.alignMap.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":111
 *         def __get__(self): return self.getAlignMap()
 * 
 *     cpdef getAlignMapRev(self):             # <<<<<<<<<<<<<<
 *         return getAlignMap(self.aligns, reverse = True)
 *     #alignMapRev = property(getAlignMapRev)
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_15getAlignMapRev(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMapRev(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getAlignMapRev", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getAlignMapRev); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_15getAlignMapRev)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":112
 * 
 *     cpdef getAlignMapRev(self):
 *         return getAlignMap(self.aligns, reverse = True)             # <<<<<<<<<<<<<<
 *     #alignMapRev = property(getAlignMapRev)
 *     property alignMapRev:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_getAlignMap); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->aligns);
  __Pyx_GIVEREF(__pyx_v_self->aligns);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->aligns);
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_reverse, Py_True) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":111
 *         def __get__(self): return self.getAlignMap()
 * 
 *     cpdef getAlignMapRev(self):             # <<<<<<<<<<<<<<
 *         return getAlignMap(self.aligns, reverse = True)
 *     #alignMapRev = property(getAlignMapRev)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getAlignMapRev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_15getAlignMapRev(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_15getAlignMapRev(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getAlignMapRev (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_14getAlignMapRev(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_14getAlignMapRev(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getAlignMapRev", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMapRev(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getAlignMapRev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":115
 *     #alignMapRev = property(getAlignMapRev)
 *     property alignMapRev:
 *         def __get__(self): return self.getAlignMapRev()             # <<<<<<<<<<<<<<
 * 
 *     cpdef Record getReversed(self):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_11alignMapRev_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_11alignMapRev_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_11alignMapRev___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_11alignMapRev___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self->__pyx_vtab)->getAlignMapRev(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.alignMapRev.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":117
 *         def __get__(self): return self.getAlignMapRev()
 * 
 *     cpdef Record getReversed(self):             # <<<<<<<<<<<<<<
 *         cdef Record recRev
 *         cdef dict revFeatures
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_17getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_recRev = 0;
  PyObject *__pyx_v_revFeatures = 0;
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __Pyx_RefNannySetupContext("getReversed", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getReversed); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_17getReversed)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_3lpu_3smt_12trans_models_7records_Record))))) __PYX_ERR(0, 117, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":120
 *         cdef Record recRev
 *         cdef dict revFeatures
 *         recRev = self.__class__()             # <<<<<<<<<<<<<<
 *         recRev.src = self.trg
 *         recRev.trg = self.src
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_3lpu_3smt_12trans_models_7records_Record))))) __PYX_ERR(0, 120, __pyx_L1_error)
  __pyx_v_recRev = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":121
 *         cdef dict revFeatures
 *         recRev = self.__class__()
 *         recRev.src = self.trg             # <<<<<<<<<<<<<<
 *         recRev.trg = self.src
 *         recRev.counts = self.counts.getReversed()
 */
  __pyx_t_1 = __pyx_v_self->trg;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->src);
  __Pyx_DECREF(__pyx_v_recRev->src);
  __pyx_v_recRev->src = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":122
 *         recRev = self.__class__()
 *         recRev.src = self.trg
 *         recRev.trg = self.src             # <<<<<<<<<<<<<<
 *         recRev.counts = self.counts.getReversed()
 * #        debug.log(self.to_str())
 */
  __pyx_t_1 = __pyx_v_self->src;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->trg);
  __Pyx_DECREF(__pyx_v_recRev->trg);
  __pyx_v_recRev->trg = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":123
 *         recRev.src = self.trg
 *         recRev.trg = self.src
 *         recRev.counts = self.counts.getReversed()             # <<<<<<<<<<<<<<
 * #        debug.log(self.to_str())
 * #        debug.log(self.aligns)
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self->counts->__pyx_vtab)->getReversed(__pyx_v_self->counts, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->counts);
  __Pyx_DECREF(((PyObject *)__pyx_v_recRev->counts));
  __pyx_v_recRev->counts = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":127
 * #        debug.log(self.aligns)
 * #        recRev.aligns = getRevAligns(self.aligns)
 *         recRev.aligns = getRevAlignSet(self.aligns)             # <<<<<<<<<<<<<<
 *         revFeatures = {}
 *         if 'egfp' in self.features:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_getRevAlignSet); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->aligns); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->aligns};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->aligns};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self->aligns);
      __Pyx_GIVEREF(__pyx_v_self->aligns);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self->aligns);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PySet_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "set", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->aligns);
  __Pyx_DECREF(__pyx_v_recRev->aligns);
  __pyx_v_recRev->aligns = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":128
 * #        recRev.aligns = getRevAligns(self.aligns)
 *         recRev.aligns = getRevAlignSet(self.aligns)
 *         revFeatures = {}             # <<<<<<<<<<<<<<
 *         if 'egfp' in self.features:
 *             revFeatures[intern('fgep')] = self.features['egfp']
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_revFeatures = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":129
 *         recRev.aligns = getRevAlignSet(self.aligns)
 *         revFeatures = {}
 *         if 'egfp' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('fgep')] = self.features['egfp']
 *         if 'egfl' in self.features:
 */
  if (unlikely(__pyx_v_self->features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 129, __pyx_L1_error)
  }
  __pyx_t_5 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_egfp, __pyx_v_self->features, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 129, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":130
 *         revFeatures = {}
 *         if 'egfp' in self.features:
 *             revFeatures[intern('fgep')] = self.features['egfp']             # <<<<<<<<<<<<<<
 *         if 'egfl' in self.features:
 *             revFeatures[intern('fgel')] = self.features['egfl']
 */
    if (unlikely(__pyx_v_self->features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 130, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->features, __pyx_n_s_egfp); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_Intern(__pyx_n_s_fgep); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 130, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_t_2, __pyx_t_1) < 0)) __PYX_ERR(0, 130, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":129
 *         recRev.aligns = getRevAlignSet(self.aligns)
 *         revFeatures = {}
 *         if 'egfp' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('fgep')] = self.features['egfp']
 *         if 'egfl' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":131
 *         if 'egfp' in self.features:
 *             revFeatures[intern('fgep')] = self.features['egfp']
 *         if 'egfl' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('fgel')] = self.features['egfl']
 *         if 'fgep' in self.features:
 */
  if (unlikely(__pyx_v_self->features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 131, __pyx_L1_error)
  }
  __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_egfl, __pyx_v_self->features, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 131, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":132
 *             revFeatures[intern('fgep')] = self.features['egfp']
 *         if 'egfl' in self.features:
 *             revFeatures[intern('fgel')] = self.features['egfl']             # <<<<<<<<<<<<<<
 *         if 'fgep' in self.features:
 *             revFeatures[intern('egfp')] = self.features['fgep']
 */
    if (unlikely(__pyx_v_self->features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 132, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->features, __pyx_n_s_egfl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_Intern(__pyx_n_s_fgel); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_t_2, __pyx_t_1) < 0)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":131
 *         if 'egfp' in self.features:
 *             revFeatures[intern('fgep')] = self.features['egfp']
 *         if 'egfl' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('fgel')] = self.features['egfl']
 *         if 'fgep' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":133
 *         if 'egfl' in self.features:
 *             revFeatures[intern('fgel')] = self.features['egfl']
 *         if 'fgep' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('egfp')] = self.features['fgep']
 *         if 'fgel' in self.features:
 */
  if (unlikely(__pyx_v_self->features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 133, __pyx_L1_error)
  }
  __pyx_t_5 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_fgep, __pyx_v_self->features, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 133, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":134
 *             revFeatures[intern('fgel')] = self.features['egfl']
 *         if 'fgep' in self.features:
 *             revFeatures[intern('egfp')] = self.features['fgep']             # <<<<<<<<<<<<<<
 *         if 'fgel' in self.features:
 *             revFeatures[intern('egfl')] = self.features['fgel']
 */
    if (unlikely(__pyx_v_self->features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 134, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->features, __pyx_n_s_fgep); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_Intern(__pyx_n_s_egfp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_t_2, __pyx_t_1) < 0)) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":133
 *         if 'egfl' in self.features:
 *             revFeatures[intern('fgel')] = self.features['egfl']
 *         if 'fgep' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('egfp')] = self.features['fgep']
 *         if 'fgel' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":135
 *         if 'fgep' in self.features:
 *             revFeatures[intern('egfp')] = self.features['fgep']
 *         if 'fgel' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('egfl')] = self.features['fgel']
 *         if 'p' in self.features:
 */
  if (unlikely(__pyx_v_self->features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 135, __pyx_L1_error)
  }
  __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_fgel, __pyx_v_self->features, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 135, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":136
 *             revFeatures[intern('egfp')] = self.features['fgep']
 *         if 'fgel' in self.features:
 *             revFeatures[intern('egfl')] = self.features['fgel']             # <<<<<<<<<<<<<<
 *         if 'p' in self.features:
 *             revFeatures[intern('p')] = self.features['p']
 */
    if (unlikely(__pyx_v_self->features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 136, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->features, __pyx_n_s_fgel); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_Intern(__pyx_n_s_egfl); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_t_2, __pyx_t_1) < 0)) __PYX_ERR(0, 136, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":135
 *         if 'fgep' in self.features:
 *             revFeatures[intern('egfp')] = self.features['fgep']
 *         if 'fgel' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('egfl')] = self.features['fgel']
 *         if 'p' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":137
 *         if 'fgel' in self.features:
 *             revFeatures[intern('egfl')] = self.features['fgel']
 *         if 'p' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('p')] = self.features['p']
 *         revFeatures[intern('w')] = len(self.srcTerms)
 */
  if (unlikely(__pyx_v_self->features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 137, __pyx_L1_error)
  }
  __pyx_t_5 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_p, __pyx_v_self->features, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 137, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":138
 *             revFeatures[intern('egfl')] = self.features['fgel']
 *         if 'p' in self.features:
 *             revFeatures[intern('p')] = self.features['p']             # <<<<<<<<<<<<<<
 *         revFeatures[intern('w')] = len(self.srcTerms)
 *         recRev.features = revFeatures
 */
    if (unlikely(__pyx_v_self->features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 138, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->features, __pyx_n_s_p); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_Intern(__pyx_n_s_p); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_t_2, __pyx_t_1) < 0)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":137
 *         if 'fgel' in self.features:
 *             revFeatures[intern('egfl')] = self.features['fgel']
 *         if 'p' in self.features:             # <<<<<<<<<<<<<<
 *             revFeatures[intern('p')] = self.features['p']
 *         revFeatures[intern('w')] = len(self.srcTerms)
 */
  }

  /* "lpu/smt/trans_models/records.pyx":139
 *         if 'p' in self.features:
 *             revFeatures[intern('p')] = self.features['p']
 *         revFeatures[intern('w')] = len(self.srcTerms)             # <<<<<<<<<<<<<<
 *         recRev.features = revFeatures
 *         return recRev
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_srcTerms); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_Intern(__pyx_n_s_w); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_t_2, __pyx_t_1) < 0)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":140
 *             revFeatures[intern('p')] = self.features['p']
 *         revFeatures[intern('w')] = len(self.srcTerms)
 *         recRev.features = revFeatures             # <<<<<<<<<<<<<<
 *         return recRev
 * 
 */
  __Pyx_INCREF(__pyx_v_revFeatures);
  __Pyx_GIVEREF(__pyx_v_revFeatures);
  __Pyx_GOTREF(__pyx_v_recRev->features);
  __Pyx_DECREF(__pyx_v_recRev->features);
  __pyx_v_recRev->features = __pyx_v_revFeatures;

  /* "lpu/smt/trans_models/records.pyx":141
 *         revFeatures[intern('w')] = len(self.srcTerms)
 *         recRev.features = revFeatures
 *         return recRev             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_recRev));
  __pyx_r = __pyx_v_recRev;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":117
 *         def __get__(self): return self.getAlignMapRev()
 * 
 *     cpdef Record getReversed(self):             # <<<<<<<<<<<<<<
 *         cdef Record recRev
 *         cdef dict revFeatures
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getReversed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_recRev);
  __Pyx_XDECREF(__pyx_v_revFeatures);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_17getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_17getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getReversed (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_16getReversed(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_16getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getReversed", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getReversed(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getReversed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":144
 * 
 *     @staticmethod
 *     def getTerms(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_19getTerms(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_6Record_19getTerms = {"getTerms", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_19getTerms, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_19getTerms(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_phrase = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getTerms (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_phrase,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_phrase)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getTerms") < 0)) __PYX_ERR(0, 144, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_phrase = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getTerms", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 144, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getTerms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_18getTerms(__pyx_v_phrase);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_18getTerms(PyObject *__pyx_v_phrase) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getTerms", 0);
  __Pyx_INCREF(__pyx_v_phrase);

  /* "lpu/smt/trans_models/records.pyx":145
 *     @staticmethod
 *     def getTerms(phrase):
 *         if isinstance(phrase, str):             # <<<<<<<<<<<<<<
 *             phrase = phrase.split(' ')
 *         return phrase
 */
  __pyx_t_1 = PyString_Check(__pyx_v_phrase); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "lpu/smt/trans_models/records.pyx":146
 *     def getTerms(phrase):
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')             # <<<<<<<<<<<<<<
 *         return phrase
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_phrase, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_phrase, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":145
 *     @staticmethod
 *     def getTerms(phrase):
 *         if isinstance(phrase, str):             # <<<<<<<<<<<<<<
 *             phrase = phrase.split(' ')
 *         return phrase
 */
  }

  /* "lpu/smt/trans_models/records.pyx":147
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 *         return phrase             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_phrase);
  __pyx_r = __pyx_v_phrase;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":144
 * 
 *     @staticmethod
 *     def getTerms(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getTerms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_phrase);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":150
 * 
 *     @staticmethod
 *     def getSymbols(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_21getSymbols(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_6Record_21getSymbols = {"getSymbols", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_21getSymbols, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_21getSymbols(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_phrase = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getSymbols (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_phrase,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_phrase)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getSymbols") < 0)) __PYX_ERR(0, 150, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_phrase = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getSymbols", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 150, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_20getSymbols(__pyx_v_phrase);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_20getSymbols(PyObject *__pyx_v_phrase) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getSymbols", 0);
  __Pyx_INCREF(__pyx_v_phrase);

  /* "lpu/smt/trans_models/records.pyx":151
 *     @staticmethod
 *     def getSymbols(phrase):
 *         if isinstance(phrase, str):             # <<<<<<<<<<<<<<
 *             phrase = phrase.split(' ')
 *         return phrase
 */
  __pyx_t_1 = PyString_Check(__pyx_v_phrase); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "lpu/smt/trans_models/records.pyx":152
 *     def getSymbols(phrase):
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')             # <<<<<<<<<<<<<<
 *         return phrase
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_phrase, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_phrase, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":151
 *     @staticmethod
 *     def getSymbols(phrase):
 *         if isinstance(phrase, str):             # <<<<<<<<<<<<<<
 *             phrase = phrase.split(' ')
 *         return phrase
 */
  }

  /* "lpu/smt/trans_models/records.pyx":153
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 *         return phrase             # <<<<<<<<<<<<<<
 * 
 * class MosesRecord(Record):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_phrase);
  __pyx_r = __pyx_v_phrase;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":150
 * 
 *     @staticmethod
 *     def getSymbols(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.getSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_phrase);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":67
 * 
 * cdef class Record(object):
 *     cdef public str src, trg             # <<<<<<<<<<<<<<
 *     #cdef public long src, trg
 *     cdef public dict features
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->src);
  __pyx_r = __pyx_v_self->src;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 67, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->src);
  __Pyx_DECREF(__pyx_v_self->src);
  __pyx_v_self->src = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.src.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src_4__del__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3src_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->src);
  __Pyx_DECREF(__pyx_v_self->src);
  __pyx_v_self->src = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->trg);
  __pyx_r = __pyx_v_self->trg;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 67, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->trg);
  __Pyx_DECREF(__pyx_v_self->trg);
  __pyx_v_self->trg = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.trg.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg_4__del__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_3trg_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->trg);
  __Pyx_DECREF(__pyx_v_self->trg);
  __pyx_v_self->trg = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":69
 *     cdef public str src, trg
 *     #cdef public long src, trg
 *     cdef public dict features             # <<<<<<<<<<<<<<
 *     cdef public CoOccurrence counts
 *     cdef public set aligns
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->features);
  __pyx_r = __pyx_v_self->features;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 69, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->features);
  __Pyx_DECREF(__pyx_v_self->features);
  __pyx_v_self->features = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.features.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features_4__del__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_8features_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->features);
  __Pyx_DECREF(__pyx_v_self->features);
  __pyx_v_self->features = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":70
 *     #cdef public long src, trg
 *     cdef public dict features
 *     cdef public CoOccurrence counts             # <<<<<<<<<<<<<<
 *     cdef public set aligns
 *     cdef public object data
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->counts));
  __pyx_r = ((PyObject *)__pyx_v_self->counts);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_3lpu_3smt_12trans_models_7records_CoOccurrence))))) __PYX_ERR(0, 70, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->counts);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->counts));
  __pyx_v_self->counts = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.counts.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts_4__del__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6counts_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->counts);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->counts));
  __pyx_v_self->counts = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":71
 *     cdef public dict features
 *     cdef public CoOccurrence counts
 *     cdef public set aligns             # <<<<<<<<<<<<<<
 *     cdef public object data
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->aligns);
  __pyx_r = __pyx_v_self->aligns;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PySet_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "set", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 71, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->aligns);
  __Pyx_DECREF(__pyx_v_self->aligns);
  __pyx_v_self->aligns = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.aligns.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns_4__del__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_6aligns_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->aligns);
  __Pyx_DECREF(__pyx_v_self->aligns);
  __pyx_v_self->aligns = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":72
 *     cdef public CoOccurrence counts
 *     cdef public set aligns
 *     cdef public object data             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data___get__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data___get__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->data);
  __pyx_r = __pyx_v_self->data;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data_2__set__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data_2__set__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data_4__del__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_4data_4__del__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_22__reduce_cython__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6Record_24__setstate_cython__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6Record_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.Record.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":156
 * 
 * class MosesRecord(Record):
 *     def __init__(self, line = "", delim = '|||'):             # <<<<<<<<<<<<<<
 *         Record.__init__(self)
 *         self.delim = delim
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_11MosesRecord_1__init__ = {"__init__", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_line = 0;
  PyObject *__pyx_v_delim = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_line,&__pyx_n_s_delim,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)((PyObject*)__pyx_kp_s__3));
    values[2] = ((PyObject *)((PyObject*)__pyx_kp_s__9));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_line);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_delim);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 156, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_line = values[1];
    __pyx_v_delim = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 156, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.MosesRecord.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord___init__(__pyx_self, __pyx_v_self, __pyx_v_line, __pyx_v_delim);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "lpu/smt/trans_models/records.pyx":157
 * class MosesRecord(Record):
 *     def __init__(self, line = "", delim = '|||'):
 *         Record.__init__(self)             # <<<<<<<<<<<<<<
 *         self.delim = delim
 *         self.loadLine(line, delim)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self);
      __Pyx_GIVEREF(__pyx_v_self);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":158
 *     def __init__(self, line = "", delim = '|||'):
 *         Record.__init__(self)
 *         self.delim = delim             # <<<<<<<<<<<<<<
 *         self.loadLine(line, delim)
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_delim, __pyx_v_delim) < 0) __PYX_ERR(0, 158, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":159
 *         Record.__init__(self)
 *         self.delim = delim
 *         self.loadLine(line, delim)             # <<<<<<<<<<<<<<
 * 
 *     def loadLine(self, line, delim = '|||'):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_loadLine); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_line, __pyx_v_delim};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_line, __pyx_v_delim};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_line);
    __Pyx_GIVEREF(__pyx_v_line);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_line);
    __Pyx_INCREF(__pyx_v_delim);
    __Pyx_GIVEREF(__pyx_v_delim);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_v_delim);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":156
 * 
 * class MosesRecord(Record):
 *     def __init__(self, line = "", delim = '|||'):             # <<<<<<<<<<<<<<
 *         Record.__init__(self)
 *         self.delim = delim
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.MosesRecord.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":161
 *         self.loadLine(line, delim)
 * 
 *     def loadLine(self, line, delim = '|||'):             # <<<<<<<<<<<<<<
 *         if line:
 *             fields = line.strip().split(delim)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_3loadLine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_11MosesRecord_3loadLine = {"loadLine", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_3loadLine, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_3loadLine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_line = 0;
  PyObject *__pyx_v_delim = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("loadLine (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_line,&__pyx_n_s_delim,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)((PyObject*)__pyx_kp_s__9));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_line)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("loadLine", 0, 2, 3, 1); __PYX_ERR(0, 161, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_delim);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "loadLine") < 0)) __PYX_ERR(0, 161, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_line = values[1];
    __pyx_v_delim = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("loadLine", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 161, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.MosesRecord.loadLine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord_2loadLine(__pyx_self, __pyx_v_self, __pyx_v_line, __pyx_v_delim);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord_2loadLine(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim) {
  PyObject *__pyx_v_fields = NULL;
  PyObject *__pyx_v_listCounts = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("loadLine", 0);

  /* "lpu/smt/trans_models/records.pyx":162
 * 
 *     def loadLine(self, line, delim = '|||'):
 *         if line:             # <<<<<<<<<<<<<<
 *             fields = line.strip().split(delim)
 *             self.src = intern( fields[0].strip() )
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_line); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 162, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "lpu/smt/trans_models/records.pyx":163
 *     def loadLine(self, line, delim = '|||'):
 *         if line:
 *             fields = line.strip().split(delim)             # <<<<<<<<<<<<<<
 *             self.src = intern( fields[0].strip() )
 *             self.trg = intern( fields[1].strip() )
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_line, __pyx_n_s_strip); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_split); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_delim); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_delim};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_delim};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_v_delim);
        __Pyx_GIVEREF(__pyx_v_delim);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_delim);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_fields = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "lpu/smt/trans_models/records.pyx":164
 *         if line:
 *             fields = line.strip().split(delim)
 *             self.src = intern( fields[0].strip() )             # <<<<<<<<<<<<<<
 *             self.trg = intern( fields[1].strip() )
 *             self.features = getMosesFeatures(fields[2])
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_fields, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_strip); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_Intern(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_src, __pyx_t_5) < 0) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "lpu/smt/trans_models/records.pyx":165
 *             fields = line.strip().split(delim)
 *             self.src = intern( fields[0].strip() )
 *             self.trg = intern( fields[1].strip() )             # <<<<<<<<<<<<<<
 *             self.features = getMosesFeatures(fields[2])
 * #            self.aligns = fields[3].strip().split()
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_fields, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_strip); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_2) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 165, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_Intern(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_trg, __pyx_t_4) < 0) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":166
 *             self.src = intern( fields[0].strip() )
 *             self.trg = intern( fields[1].strip() )
 *             self.features = getMosesFeatures(fields[2])             # <<<<<<<<<<<<<<
 * #            self.aligns = fields[3].strip().split()
 *             self.aligns = getAlignSet( fields[3] )
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_getMosesFeatures); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_fields, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_2};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_2};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_features, __pyx_t_4) < 0) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":168
 *             self.features = getMosesFeatures(fields[2])
 * #            self.aligns = fields[3].strip().split()
 *             self.aligns = getAlignSet( fields[3] )             # <<<<<<<<<<<<<<
 *             listCounts = getCounts(fields[4])
 *             self.counts.setCounts(trg = listCounts[0], src = listCounts[1], co = listCounts[2])
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_getAlignSet); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_fields, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_6};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_6};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_aligns, __pyx_t_4) < 0) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":169
 * #            self.aligns = fields[3].strip().split()
 *             self.aligns = getAlignSet( fields[3] )
 *             listCounts = getCounts(fields[4])             # <<<<<<<<<<<<<<
 *             self.counts.setCounts(trg = listCounts[0], src = listCounts[1], co = listCounts[2])
 * 
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_getCounts); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_fields, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 169, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 169, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 169, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 169, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 169, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_listCounts = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":170
 *             self.aligns = getAlignSet( fields[3] )
 *             listCounts = getCounts(fields[4])
 *             self.counts.setCounts(trg = listCounts[0], src = listCounts[1], co = listCounts[2])             # <<<<<<<<<<<<<<
 * 
 * #    def getSrcSymbols(self):
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_counts); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_setCounts); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_listCounts, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_trg, __pyx_t_2) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_listCounts, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_src, __pyx_t_2) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_listCounts, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_co, __pyx_t_2) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "lpu/smt/trans_models/records.pyx":162
 * 
 *     def loadLine(self, line, delim = '|||'):
 *         if line:             # <<<<<<<<<<<<<<
 *             fields = line.strip().split(delim)
 *             self.src = intern( fields[0].strip() )
 */
  }

  /* "lpu/smt/trans_models/records.pyx":161
 *         self.loadLine(line, delim)
 * 
 *     def loadLine(self, line, delim = '|||'):             # <<<<<<<<<<<<<<
 *         if line:
 *             fields = line.strip().split(delim)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.MosesRecord.loadLine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_listCounts);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":188
 * #    trgTerms = property(getTrgTerms)
 * 
 *     def to_str(self, s = ' ||| '):             # <<<<<<<<<<<<<<
 *         strFeatures = getStrMosesFeatures(self.features)
 * #        strAligns = str.join(' ', self.aligns)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_5to_str(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_11MosesRecord_5to_str = {"to_str", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_5to_str, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11MosesRecord_5to_str(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_s = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("to_str (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_s,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)((PyObject*)__pyx_kp_s__10));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_s);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "to_str") < 0)) __PYX_ERR(0, 188, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_s = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("to_str", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 188, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.MosesRecord.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord_4to_str(__pyx_self, __pyx_v_self, __pyx_v_s);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_11MosesRecord_4to_str(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_v_strFeatures = NULL;
  PyObject *__pyx_v_strAligns = NULL;
  PyObject *__pyx_v_strCounts = NULL;
  PyObject *__pyx_v_buf = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("to_str", 0);

  /* "lpu/smt/trans_models/records.pyx":189
 * 
 *     def to_str(self, s = ' ||| '):
 *         strFeatures = getStrMosesFeatures(self.features)             # <<<<<<<<<<<<<<
 * #        strAligns = str.join(' ', self.aligns)
 *         strAligns = str.join(' ', sorted(self.aligns) )
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_getStrMosesFeatures); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_features); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_strFeatures = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":191
 *         strFeatures = getStrMosesFeatures(self.features)
 * #        strAligns = str.join(' ', self.aligns)
 *         strAligns = str.join(' ', sorted(self.aligns) )             # <<<<<<<<<<<<<<
 *         self.counts.round()
 *         strCounts   = "%s %s %s" % (self.counts.trg, self.counts.src, self.counts.cooc)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_aligns); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_6 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 191, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyString_Join(__pyx_kp_s__4, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_strAligns = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "lpu/smt/trans_models/records.pyx":192
 * #        strAligns = str.join(' ', self.aligns)
 *         strAligns = str.join(' ', sorted(self.aligns) )
 *         self.counts.round()             # <<<<<<<<<<<<<<
 *         strCounts   = "%s %s %s" % (self.counts.trg, self.counts.src, self.counts.cooc)
 *         #buf = str.join(s, [self.src, self.trg, strFeatures, strAligns, strCounts]) + "\n"
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_counts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_round); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_1) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 192, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "lpu/smt/trans_models/records.pyx":193
 *         strAligns = str.join(' ', sorted(self.aligns) )
 *         self.counts.round()
 *         strCounts   = "%s %s %s" % (self.counts.trg, self.counts.src, self.counts.cooc)             # <<<<<<<<<<<<<<
 *         #buf = str.join(s, [self.src, self.trg, strFeatures, strAligns, strCounts]) + "\n"
 *         buf = str.join(s, [self.src, self.trg, strFeatures, strAligns, strCounts])
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_counts); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_trg); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_counts); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_src); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_counts); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_cooc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_s_s_s, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_strCounts = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "lpu/smt/trans_models/records.pyx":195
 *         strCounts   = "%s %s %s" % (self.counts.trg, self.counts.src, self.counts.cooc)
 *         #buf = str.join(s, [self.src, self.trg, strFeatures, strAligns, strCounts]) + "\n"
 *         buf = str.join(s, [self.src, self.trg, strFeatures, strAligns, strCounts])             # <<<<<<<<<<<<<<
 * #        buf = str.join(s, [str(self.src), str(self.trg), strFeatures, strAligns, strCounts]) + "\n"
 *         return buf
 */
  if (unlikely(__pyx_v_s == Py_None)) {
    PyErr_Format(PyExc_TypeError, "descriptor '%s' requires a '%s' object but received a 'NoneType'", "join", "str");
    __PYX_ERR(0, 195, __pyx_L1_error)
  }
  if (!(likely(PyString_Check(__pyx_v_s))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_s)->tp_name), 0))) __PYX_ERR(0, 195, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_src); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_trg); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyList_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_v_strFeatures);
  __Pyx_GIVEREF(__pyx_v_strFeatures);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_v_strFeatures);
  __Pyx_INCREF(__pyx_v_strAligns);
  __Pyx_GIVEREF(__pyx_v_strAligns);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_v_strAligns);
  __Pyx_INCREF(__pyx_v_strCounts);
  __Pyx_GIVEREF(__pyx_v_strCounts);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_v_strCounts);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyString_Join(((PyObject*)__pyx_v_s), __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_buf = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "lpu/smt/trans_models/records.pyx":197
 *         buf = str.join(s, [self.src, self.trg, strFeatures, strAligns, strCounts])
 * #        buf = str.join(s, [str(self.src), str(self.trg), strFeatures, strAligns, strCounts]) + "\n"
 *         return buf             # <<<<<<<<<<<<<<
 * 
 * def getAlignMap(aligns, reverse = False):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_buf);
  __pyx_r = __pyx_v_buf;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":188
 * #    trgTerms = property(getTrgTerms)
 * 
 *     def to_str(self, s = ' ||| '):             # <<<<<<<<<<<<<<
 *         strFeatures = getStrMosesFeatures(self.features)
 * #        strAligns = str.join(' ', self.aligns)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.MosesRecord.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_strFeatures);
  __Pyx_XDECREF(__pyx_v_strAligns);
  __Pyx_XDECREF(__pyx_v_strCounts);
  __Pyx_XDECREF(__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":199
 *         return buf
 * 
 * def getAlignMap(aligns, reverse = False):             # <<<<<<<<<<<<<<
 *     alignMap = {}
 *     for align in aligns:
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_1getAlignMap(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_1getAlignMap = {"getAlignMap", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_1getAlignMap, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_1getAlignMap(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_aligns = 0;
  PyObject *__pyx_v_reverse = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getAlignMap (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_aligns,&__pyx_n_s_reverse,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_aligns)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_reverse);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getAlignMap") < 0)) __PYX_ERR(0, 199, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_aligns = values[0];
    __pyx_v_reverse = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getAlignMap", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 199, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getAlignMap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_getAlignMap(__pyx_self, __pyx_v_aligns, __pyx_v_reverse);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_getAlignMap(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_aligns, PyObject *__pyx_v_reverse) {
  PyObject *__pyx_v_alignMap = NULL;
  PyObject *__pyx_v_align = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("getAlignMap", 0);

  /* "lpu/smt/trans_models/records.pyx":200
 * 
 * def getAlignMap(aligns, reverse = False):
 *     alignMap = {}             # <<<<<<<<<<<<<<
 *     for align in aligns:
 *         (s, t) = map(int, align.split('-'))
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_alignMap = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":201
 * def getAlignMap(aligns, reverse = False):
 *     alignMap = {}
 *     for align in aligns:             # <<<<<<<<<<<<<<
 *         (s, t) = map(int, align.split('-'))
 *         if reverse:
 */
  if (likely(PyList_CheckExact(__pyx_v_aligns)) || PyTuple_CheckExact(__pyx_v_aligns)) {
    __pyx_t_1 = __pyx_v_aligns; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_aligns); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 201, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 201, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 201, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 201, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 201, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 201, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_align, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":202
 *     alignMap = {}
 *     for align in aligns:
 *         (s, t) = map(int, align.split('-'))             # <<<<<<<<<<<<<<
 *         if reverse:
 *             alignMap.setdefault(t, []).append(s)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_align, __pyx_n_s_split); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
    __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)(&PyInt_Type)));
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
      PyObject* sequence = __pyx_t_5;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 202, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_4 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 202, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_4);
    __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "lpu/smt/trans_models/records.pyx":203
 *     for align in aligns:
 *         (s, t) = map(int, align.split('-'))
 *         if reverse:             # <<<<<<<<<<<<<<
 *             alignMap.setdefault(t, []).append(s)
 *         else:
 */
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_reverse); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 203, __pyx_L1_error)
    if (__pyx_t_9) {

      /* "lpu/smt/trans_models/records.pyx":204
 *         (s, t) = map(int, align.split('-'))
 *         if reverse:
 *             alignMap.setdefault(t, []).append(s)             # <<<<<<<<<<<<<<
 *         else:
 *             alignMap.setdefault(s, []).append(t)
 */
      __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyDict_SetDefault(__pyx_v_alignMap, __pyx_v_t, __pyx_t_5, -1L); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_10 = __Pyx_PyObject_Append(__pyx_t_6, __pyx_v_s); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "lpu/smt/trans_models/records.pyx":203
 *     for align in aligns:
 *         (s, t) = map(int, align.split('-'))
 *         if reverse:             # <<<<<<<<<<<<<<
 *             alignMap.setdefault(t, []).append(s)
 *         else:
 */
      goto __pyx_L7;
    }

    /* "lpu/smt/trans_models/records.pyx":206
 *             alignMap.setdefault(t, []).append(s)
 *         else:
 *             alignMap.setdefault(s, []).append(t)             # <<<<<<<<<<<<<<
 *     return alignMap
 * 
 */
    /*else*/ {
      __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = __Pyx_PyDict_SetDefault(__pyx_v_alignMap, __pyx_v_s, __pyx_t_6, -1L); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_10 = __Pyx_PyObject_Append(__pyx_t_5, __pyx_v_t); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_L7:;

    /* "lpu/smt/trans_models/records.pyx":201
 * def getAlignMap(aligns, reverse = False):
 *     alignMap = {}
 *     for align in aligns:             # <<<<<<<<<<<<<<
 *         (s, t) = map(int, align.split('-'))
 *         if reverse:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":207
 *         else:
 *             alignMap.setdefault(s, []).append(t)
 *     return alignMap             # <<<<<<<<<<<<<<
 * 
 * def getAlignSet(strField):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_alignMap);
  __pyx_r = __pyx_v_alignMap;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":199
 *         return buf
 * 
 * def getAlignMap(aligns, reverse = False):             # <<<<<<<<<<<<<<
 *     alignMap = {}
 *     for align in aligns:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getAlignMap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_alignMap);
  __Pyx_XDECREF(__pyx_v_align);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":209
 *     return alignMap
 * 
 * def getAlignSet(strField):             # <<<<<<<<<<<<<<
 *     return set(strField.strip().split())
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_3getAlignSet(PyObject *__pyx_self, PyObject *__pyx_v_strField); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_3getAlignSet = {"getAlignSet", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_3getAlignSet, METH_O, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_3getAlignSet(PyObject *__pyx_self, PyObject *__pyx_v_strField) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getAlignSet (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_2getAlignSet(__pyx_self, ((PyObject *)__pyx_v_strField));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_2getAlignSet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_strField) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getAlignSet", 0);

  /* "lpu/smt/trans_models/records.pyx":210
 * 
 * def getAlignSet(strField):
 *     return set(strField.strip().split())             # <<<<<<<<<<<<<<
 * 
 * def getCounts(field):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_strField, __pyx_n_s_strip); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySet_New(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":209
 *     return alignMap
 * 
 * def getAlignSet(strField):             # <<<<<<<<<<<<<<
 *     return set(strField.strip().split())
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getAlignSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":212
 *     return set(strField.strip().split())
 * 
 * def getCounts(field):             # <<<<<<<<<<<<<<
 * #    return map(getNumber, field.split())
 * #    return map(numbers.toNumber, field.split())
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_5getCounts(PyObject *__pyx_self, PyObject *__pyx_v_field); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_5getCounts = {"getCounts", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_5getCounts, METH_O, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_5getCounts(PyObject *__pyx_self, PyObject *__pyx_v_field) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getCounts (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_4getCounts(__pyx_self, ((PyObject *)__pyx_v_field));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_4getCounts(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_field) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getCounts", 0);

  /* "lpu/smt/trans_models/records.pyx":215
 * #    return map(getNumber, field.split())
 * #    return map(numbers.toNumber, field.split())
 *     return list(map(numbers.toNumber, field.split()))             # <<<<<<<<<<<<<<
 * 
 * def getNumber(anyNum, margin = 0):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_numbers); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_toNumber); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_field, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":212
 *     return set(strField.strip().split())
 * 
 * def getCounts(field):             # <<<<<<<<<<<<<<
 * #    return map(getNumber, field.split())
 * #    return map(numbers.toNumber, field.split())
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getCounts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":217
 *     return list(map(numbers.toNumber, field.split()))
 * 
 * def getNumber(anyNum, margin = 0):             # <<<<<<<<<<<<<<
 *     numFloat = float(anyNum)
 *     numInt = int(round(numFloat))
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_7getNumber(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_7getNumber = {"getNumber", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_7getNumber, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_7getNumber(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_anyNum = 0;
  PyObject *__pyx_v_margin = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getNumber (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_anyNum,&__pyx_n_s_margin,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_anyNum)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_margin);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getNumber") < 0)) __PYX_ERR(0, 217, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_anyNum = values[0];
    __pyx_v_margin = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getNumber", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 217, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getNumber", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_6getNumber(__pyx_self, __pyx_v_anyNum, __pyx_v_margin);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_6getNumber(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_anyNum, PyObject *__pyx_v_margin) {
  double __pyx_v_numFloat;
  PyObject *__pyx_v_numInt = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("getNumber", 0);

  /* "lpu/smt/trans_models/records.pyx":218
 * 
 * def getNumber(anyNum, margin = 0):
 *     numFloat = float(anyNum)             # <<<<<<<<<<<<<<
 *     numInt = int(round(numFloat))
 *     if margin > 0:
 */
  __pyx_t_1 = __Pyx_PyObject_AsDouble(__pyx_v_anyNum); if (unlikely(__pyx_t_1 == ((double)-1) && PyErr_Occurred())) __PYX_ERR(0, 218, __pyx_L1_error)
  __pyx_v_numFloat = __pyx_t_1;

  /* "lpu/smt/trans_models/records.pyx":219
 * def getNumber(anyNum, margin = 0):
 *     numFloat = float(anyNum)
 *     numInt = int(round(numFloat))             # <<<<<<<<<<<<<<
 *     if margin > 0:
 *         if abs(numInt - numFloat) < margin:
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_numFloat); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_numInt = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "lpu/smt/trans_models/records.pyx":220
 *     numFloat = float(anyNum)
 *     numInt = int(round(numFloat))
 *     if margin > 0:             # <<<<<<<<<<<<<<
 *         if abs(numInt - numFloat) < margin:
 *             return numInt
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_margin, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "lpu/smt/trans_models/records.pyx":221
 *     numInt = int(round(numFloat))
 *     if margin > 0:
 *         if abs(numInt - numFloat) < margin:             # <<<<<<<<<<<<<<
 *             return numInt
 *         else:
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_numFloat); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyNumber_Subtract(__pyx_v_numInt, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Absolute(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_3, __pyx_v_margin, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_4) {

      /* "lpu/smt/trans_models/records.pyx":222
 *     if margin > 0:
 *         if abs(numInt - numFloat) < margin:
 *             return numInt             # <<<<<<<<<<<<<<
 *         else:
 *             return numFloat
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_numInt);
      __pyx_r = __pyx_v_numInt;
      goto __pyx_L0;

      /* "lpu/smt/trans_models/records.pyx":221
 *     numInt = int(round(numFloat))
 *     if margin > 0:
 *         if abs(numInt - numFloat) < margin:             # <<<<<<<<<<<<<<
 *             return numInt
 *         else:
 */
    }

    /* "lpu/smt/trans_models/records.pyx":224
 *             return numInt
 *         else:
 *             return numFloat             # <<<<<<<<<<<<<<
 *     elif numFloat == numInt:
 *         return numInt
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_2 = PyFloat_FromDouble(__pyx_v_numFloat); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L0;
    }

    /* "lpu/smt/trans_models/records.pyx":220
 *     numFloat = float(anyNum)
 *     numInt = int(round(numFloat))
 *     if margin > 0:             # <<<<<<<<<<<<<<
 *         if abs(numInt - numFloat) < margin:
 *             return numInt
 */
  }

  /* "lpu/smt/trans_models/records.pyx":225
 *         else:
 *             return numFloat
 *     elif numFloat == numInt:             # <<<<<<<<<<<<<<
 *         return numInt
 *     else:
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_numFloat); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_v_numInt, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "lpu/smt/trans_models/records.pyx":226
 *             return numFloat
 *     elif numFloat == numInt:
 *         return numInt             # <<<<<<<<<<<<<<
 *     else:
 *         return numFloat
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_numInt);
    __pyx_r = __pyx_v_numInt;
    goto __pyx_L0;

    /* "lpu/smt/trans_models/records.pyx":225
 *         else:
 *             return numFloat
 *     elif numFloat == numInt:             # <<<<<<<<<<<<<<
 *         return numInt
 *     else:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":228
 *         return numInt
 *     else:
 *         return numFloat             # <<<<<<<<<<<<<<
 * 
 * #def getRevAligns(aligns):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_numFloat); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "lpu/smt/trans_models/records.pyx":217
 *     return list(map(numbers.toNumber, field.split()))
 * 
 * def getNumber(anyNum, margin = 0):             # <<<<<<<<<<<<<<
 *     numFloat = float(anyNum)
 *     numInt = int(round(numFloat))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getNumber", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_numInt);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":231
 * 
 * #def getRevAligns(aligns):
 * def getRevAlignSet(aligns):             # <<<<<<<<<<<<<<
 * #    revAlignList = []
 *     revAlignSet = set()
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_9getRevAlignSet(PyObject *__pyx_self, PyObject *__pyx_v_aligns); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_9getRevAlignSet = {"getRevAlignSet", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_9getRevAlignSet, METH_O, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_9getRevAlignSet(PyObject *__pyx_self, PyObject *__pyx_v_aligns) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getRevAlignSet (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_8getRevAlignSet(__pyx_self, ((PyObject *)__pyx_v_aligns));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_8getRevAlignSet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_aligns) {
  PyObject *__pyx_v_revAlignSet = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("getRevAlignSet", 0);

  /* "lpu/smt/trans_models/records.pyx":233
 * def getRevAlignSet(aligns):
 * #    revAlignList = []
 *     revAlignSet = set()             # <<<<<<<<<<<<<<
 * #    debug.log(aligns)
 *     for a in aligns:
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_revAlignSet = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":235
 *     revAlignSet = set()
 * #    debug.log(aligns)
 *     for a in aligns:             # <<<<<<<<<<<<<<
 *       (s, t) = map(int, a.split('-'))
 * #      revAlignList.append( "%d-%d" % (t, s) )
 */
  if (likely(PyList_CheckExact(__pyx_v_aligns)) || PyTuple_CheckExact(__pyx_v_aligns)) {
    __pyx_t_1 = __pyx_v_aligns; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_aligns); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 235, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 235, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 235, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 235, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_a, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "lpu/smt/trans_models/records.pyx":236
 * #    debug.log(aligns)
 *     for a in aligns:
 *       (s, t) = map(int, a.split('-'))             # <<<<<<<<<<<<<<
 * #      revAlignList.append( "%d-%d" % (t, s) )
 *       revAlignSet.add( "%d-%d" % (t, s) )
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_n_s_split); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
    __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)(&PyInt_Type)));
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
      PyObject* sequence = __pyx_t_5;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 236, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 236, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 236, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 236, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_4 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 236, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 236, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_4);
    __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "lpu/smt/trans_models/records.pyx":238
 *       (s, t) = map(int, a.split('-'))
 * #      revAlignList.append( "%d-%d" % (t, s) )
 *       revAlignSet.add( "%d-%d" % (t, s) )             # <<<<<<<<<<<<<<
 * #    return sorted(revAlignList)
 *     return sorted(revAlignSet)
 */
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_t);
    __Pyx_GIVEREF(__pyx_v_t);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_t);
    __Pyx_INCREF(__pyx_v_s);
    __Pyx_GIVEREF(__pyx_v_s);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_s);
    __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_d_d, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = PySet_Add(__pyx_v_revAlignSet, __pyx_t_6); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "lpu/smt/trans_models/records.pyx":235
 *     revAlignSet = set()
 * #    debug.log(aligns)
 *     for a in aligns:             # <<<<<<<<<<<<<<
 *       (s, t) = map(int, a.split('-'))
 * #      revAlignList.append( "%d-%d" % (t, s) )
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":240
 *       revAlignSet.add( "%d-%d" % (t, s) )
 * #    return sorted(revAlignList)
 *     return sorted(revAlignSet)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PySequence_List(__pyx_v_revAlignSet); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_9 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 240, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":231
 * 
 * #def getRevAligns(aligns):
 * def getRevAlignSet(aligns):             # <<<<<<<<<<<<<<
 * #    revAlignList = []
 *     revAlignSet = set()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getRevAlignSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_revAlignSet);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":243
 * 
 * 
 * def getMosesFeatures(field):             # <<<<<<<<<<<<<<
 *     features = {}
 *     scores = map(getNumber, field.split())
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11getMosesFeatures(PyObject *__pyx_self, PyObject *__pyx_v_field); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_11getMosesFeatures = {"getMosesFeatures", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_11getMosesFeatures, METH_O, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_11getMosesFeatures(PyObject *__pyx_self, PyObject *__pyx_v_field) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getMosesFeatures (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_10getMosesFeatures(__pyx_self, ((PyObject *)__pyx_v_field));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_10getMosesFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_field) {
  PyObject *__pyx_v_features = NULL;
  PyObject *__pyx_v_scores = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("getMosesFeatures", 0);

  /* "lpu/smt/trans_models/records.pyx":244
 * 
 * def getMosesFeatures(field):
 *     features = {}             # <<<<<<<<<<<<<<
 *     scores = map(getNumber, field.split())
 *     features[intern('fgep')] = scores[0]
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_features = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":245
 * def getMosesFeatures(field):
 *     features = {}
 *     scores = map(getNumber, field.split())             # <<<<<<<<<<<<<<
 *     features[intern('fgep')] = scores[0]
 *     features[intern('fgel')] = scores[1]
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_getNumber); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_field, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 245, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 245, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_scores = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":246
 *     features = {}
 *     scores = map(getNumber, field.split())
 *     features[intern('fgep')] = scores[0]             # <<<<<<<<<<<<<<
 *     features[intern('fgel')] = scores[1]
 *     features[intern('egfp')] = scores[2]
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_scores, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Intern(__pyx_n_s_fgep); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_features, __pyx_t_3, __pyx_t_2) < 0)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":247
 *     scores = map(getNumber, field.split())
 *     features[intern('fgep')] = scores[0]
 *     features[intern('fgel')] = scores[1]             # <<<<<<<<<<<<<<
 *     features[intern('egfp')] = scores[2]
 *     features[intern('egfl')] = scores[3]
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_scores, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Intern(__pyx_n_s_fgel); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_features, __pyx_t_3, __pyx_t_2) < 0)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":248
 *     features[intern('fgep')] = scores[0]
 *     features[intern('fgel')] = scores[1]
 *     features[intern('egfp')] = scores[2]             # <<<<<<<<<<<<<<
 *     features[intern('egfl')] = scores[3]
 *     return features
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_scores, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Intern(__pyx_n_s_egfp); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_features, __pyx_t_3, __pyx_t_2) < 0)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":249
 *     features[intern('fgel')] = scores[1]
 *     features[intern('egfp')] = scores[2]
 *     features[intern('egfl')] = scores[3]             # <<<<<<<<<<<<<<
 *     return features
 * 
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_scores, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Intern(__pyx_n_s_egfl); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_features, __pyx_t_3, __pyx_t_2) < 0)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":250
 *     features[intern('egfp')] = scores[2]
 *     features[intern('egfl')] = scores[3]
 *     return features             # <<<<<<<<<<<<<<
 * 
 * def getStrMosesFeatures(dicFeatures):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_features);
  __pyx_r = __pyx_v_features;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":243
 * 
 * 
 * def getMosesFeatures(field):             # <<<<<<<<<<<<<<
 *     features = {}
 *     scores = map(getNumber, field.split())
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getMosesFeatures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_features);
  __Pyx_XDECREF(__pyx_v_scores);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":252
 *     return features
 * 
 * def getStrMosesFeatures(dicFeatures):             # <<<<<<<<<<<<<<
 *     '''convert back the feature dictionary to score string separated by space'''
 *     scores = []
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_13getStrMosesFeatures(PyObject *__pyx_self, PyObject *__pyx_v_dicFeatures); /*proto*/
static char __pyx_doc_3lpu_3smt_12trans_models_7records_12getStrMosesFeatures[] = "convert back the feature dictionary to score string separated by space";
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_13getStrMosesFeatures = {"getStrMosesFeatures", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_13getStrMosesFeatures, METH_O, __pyx_doc_3lpu_3smt_12trans_models_7records_12getStrMosesFeatures};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_13getStrMosesFeatures(PyObject *__pyx_self, PyObject *__pyx_v_dicFeatures) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getStrMosesFeatures (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_12getStrMosesFeatures(__pyx_self, ((PyObject *)__pyx_v_dicFeatures));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_12getStrMosesFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dicFeatures) {
  PyObject *__pyx_v_scores = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("getStrMosesFeatures", 0);

  /* "lpu/smt/trans_models/records.pyx":254
 * def getStrMosesFeatures(dicFeatures):
 *     '''convert back the feature dictionary to score string separated by space'''
 *     scores = []             # <<<<<<<<<<<<<<
 *     scores.append( dicFeatures.get('fgep', 0) )
 *     scores.append( dicFeatures.get('fgel', 0) )
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_scores = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":255
 *     '''convert back the feature dictionary to score string separated by space'''
 *     scores = []
 *     scores.append( dicFeatures.get('fgep', 0) )             # <<<<<<<<<<<<<<
 *     scores.append( dicFeatures.get('fgel', 0) )
 *     scores.append( dicFeatures.get('egfp', 0) )
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dicFeatures, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_scores, __pyx_t_2); if (unlikely(__pyx_t_3 == -1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":256
 *     scores = []
 *     scores.append( dicFeatures.get('fgep', 0) )
 *     scores.append( dicFeatures.get('fgel', 0) )             # <<<<<<<<<<<<<<
 *     scores.append( dicFeatures.get('egfp', 0) )
 *     scores.append( dicFeatures.get('egfl', 0) )
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dicFeatures, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_scores, __pyx_t_1); if (unlikely(__pyx_t_3 == -1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":257
 *     scores.append( dicFeatures.get('fgep', 0) )
 *     scores.append( dicFeatures.get('fgel', 0) )
 *     scores.append( dicFeatures.get('egfp', 0) )             # <<<<<<<<<<<<<<
 *     scores.append( dicFeatures.get('egfl', 0) )
 *     return str.join(' ', map(str,scores))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dicFeatures, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_scores, __pyx_t_2); if (unlikely(__pyx_t_3 == -1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":258
 *     scores.append( dicFeatures.get('fgel', 0) )
 *     scores.append( dicFeatures.get('egfp', 0) )
 *     scores.append( dicFeatures.get('egfl', 0) )             # <<<<<<<<<<<<<<
 *     return str.join(' ', map(str,scores))
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dicFeatures, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_scores, __pyx_t_1); if (unlikely(__pyx_t_3 == -1)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":259
 *     scores.append( dicFeatures.get('egfp', 0) )
 *     scores.append( dicFeatures.get('egfl', 0) )
 *     return str.join(' ', map(str,scores))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&PyString_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyString_Type)));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&PyString_Type)));
  __Pyx_INCREF(__pyx_v_scores);
  __Pyx_GIVEREF(__pyx_v_scores);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_scores);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s__4, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":252
 *     return features
 * 
 * def getStrMosesFeatures(dicFeatures):             # <<<<<<<<<<<<<<
 *     '''convert back the feature dictionary to score string separated by space'''
 *     scores = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getStrMosesFeatures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_scores);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":265
 *     cdef str delim
 * 
 *     def __cinit__(self, object line="", str delim='|||'):             # <<<<<<<<<<<<<<
 *         Record.__init__(self)
 *         self.delim = delim
 */

/* Python wrapper */
static int __pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_line = 0;
  PyObject *__pyx_v_delim = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_line,&__pyx_n_s_delim,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)__pyx_kp_s__3);
    values[1] = ((PyObject*)__pyx_kp_s__9);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_line);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_delim);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 265, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_line = values[0];
    __pyx_v_delim = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 265, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_delim), (&PyString_Type), 1, "delim", 1))) __PYX_ERR(0, 265, __pyx_L1_error)
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord___cinit__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self), __pyx_v_line, __pyx_v_delim);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord___cinit__(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine __pyx_t_5;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "lpu/smt/trans_models/records.pyx":266
 * 
 *     def __cinit__(self, object line="", str delim='|||'):
 *         Record.__init__(self)             # <<<<<<<<<<<<<<
 *         self.delim = delim
 *         self.loadLine(compat.to_str(line), delim)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_self));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":267
 *     def __cinit__(self, object line="", str delim='|||'):
 *         Record.__init__(self)
 *         self.delim = delim             # <<<<<<<<<<<<<<
 *         self.loadLine(compat.to_str(line), delim)
 * 
 */
  __Pyx_INCREF(__pyx_v_delim);
  __Pyx_GIVEREF(__pyx_v_delim);
  __Pyx_GOTREF(__pyx_v_self->delim);
  __Pyx_DECREF(__pyx_v_self->delim);
  __pyx_v_self->delim = __pyx_v_delim;

  /* "lpu/smt/trans_models/records.pyx":268
 *         Record.__init__(self)
 *         self.delim = delim
 *         self.loadLine(compat.to_str(line), delim)             # <<<<<<<<<<<<<<
 * 
 * #    cpdef getSrcSymbols(self):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_compat); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_to_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_line); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_line};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_line};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_v_line);
      __Pyx_GIVEREF(__pyx_v_line);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_line);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 268, __pyx_L1_error)
  __pyx_t_5.__pyx_n = 1;
  __pyx_t_5.delim = __pyx_v_delim;
  __pyx_t_4 = ((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self->__pyx_base.__pyx_vtab)->loadLine(__pyx_v_self, ((PyObject*)__pyx_t_1), 0, &__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "lpu/smt/trans_models/records.pyx":265
 *     cdef str delim
 * 
 *     def __cinit__(self, object line="", str delim='|||'):             # <<<<<<<<<<<<<<
 *         Record.__init__(self)
 *         self.delim = delim
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":294
 * #        def __get__(self): return self.trgTerms()
 * 
 *     cpdef fixOrder(self):             # <<<<<<<<<<<<<<
 *         cdef str src, trg
 *         cdef object changed
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_3fixOrder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_fixOrder(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_src = 0;
  PyObject *__pyx_v_trg = 0;
  PyObject *__pyx_v_changed = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("fixOrder", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_fixOrder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_3fixOrder)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":297
 *         cdef str src, trg
 *         cdef object changed
 *         src, trg = TravatarRecord.fixOrderStrings(self.src, self.trg)             # <<<<<<<<<<<<<<
 *         changed = bool(src != self.src)
 *         self.src = src
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord), __pyx_n_s_fixOrderStrings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self->__pyx_base.src, __pyx_v_self->__pyx_base.trg};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self->__pyx_base.src, __pyx_v_self->__pyx_base.trg};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self->__pyx_base.src);
    __Pyx_GIVEREF(__pyx_v_self->__pyx_base.src);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_v_self->__pyx_base.src);
    __Pyx_INCREF(__pyx_v_self->__pyx_base.trg);
    __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trg);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_v_self->__pyx_base.trg);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 297, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_4);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_6(__pyx_t_3); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_4 = __pyx_t_6(__pyx_t_3); if (unlikely(!__pyx_t_4)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_3), 2) < 0) __PYX_ERR(0, 297, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 297, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 297, __pyx_L1_error)
  if (!(likely(PyString_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(0, 297, __pyx_L1_error)
  __pyx_v_src = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_trg = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "lpu/smt/trans_models/records.pyx":298
 *         cdef object changed
 *         src, trg = TravatarRecord.fixOrderStrings(self.src, self.trg)
 *         changed = bool(src != self.src)             # <<<<<<<<<<<<<<
 *         self.src = src
 *         self.trg = trg
 */
  __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_v_src, __pyx_v_self->__pyx_base.src, Py_NE)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 298, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyBool_FromLong((!(!__pyx_t_7))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_changed = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":299
 *         src, trg = TravatarRecord.fixOrderStrings(self.src, self.trg)
 *         changed = bool(src != self.src)
 *         self.src = src             # <<<<<<<<<<<<<<
 *         self.trg = trg
 *         return changed
 */
  __Pyx_INCREF(__pyx_v_src);
  __Pyx_GIVEREF(__pyx_v_src);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.src);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.src);
  __pyx_v_self->__pyx_base.src = __pyx_v_src;

  /* "lpu/smt/trans_models/records.pyx":300
 *         changed = bool(src != self.src)
 *         self.src = src
 *         self.trg = trg             # <<<<<<<<<<<<<<
 *         return changed
 * 
 */
  __Pyx_INCREF(__pyx_v_trg);
  __Pyx_GIVEREF(__pyx_v_trg);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.trg);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.trg);
  __pyx_v_self->__pyx_base.trg = __pyx_v_trg;

  /* "lpu/smt/trans_models/records.pyx":301
 *         self.src = src
 *         self.trg = trg
 *         return changed             # <<<<<<<<<<<<<<
 * 
 *     cpdef TravatarRecord getReversed(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_changed);
  __pyx_r = __pyx_v_changed;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":294
 * #        def __get__(self): return self.trgTerms()
 * 
 *     cpdef fixOrder(self):             # <<<<<<<<<<<<<<
 *         cdef str src, trg
 *         cdef object changed
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.fixOrder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_src);
  __Pyx_XDECREF(__pyx_v_trg);
  __Pyx_XDECREF(__pyx_v_changed);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_3fixOrder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_3fixOrder(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fixOrder (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_2fixOrder(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_2fixOrder(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("fixOrder", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_fixOrder(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.fixOrder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":303
 *         return changed
 * 
 *     cpdef TravatarRecord getReversed(self):             # <<<<<<<<<<<<<<
 *       cdef TravatarRecord recRev = TravatarRecord()
 *       cdef dict revFeatures
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_5getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_recRev = 0;
  PyObject *__pyx_v_revFeatures = 0;
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __Pyx_RefNannySetupContext("getReversed", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getReversed); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_5getReversed)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 303, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 303, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord))))) __PYX_ERR(0, 303, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":304
 * 
 *     cpdef TravatarRecord getReversed(self):
 *       cdef TravatarRecord recRev = TravatarRecord()             # <<<<<<<<<<<<<<
 *       cdef dict revFeatures
 *       recRev.src = self.trg
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_recRev = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":306
 *       cdef TravatarRecord recRev = TravatarRecord()
 *       cdef dict revFeatures
 *       recRev.src = self.trg             # <<<<<<<<<<<<<<
 *       recRev.trg = self.src
 *       recRev.counts = self.counts.getReversed()
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.trg;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->__pyx_base.src);
  __Pyx_DECREF(__pyx_v_recRev->__pyx_base.src);
  __pyx_v_recRev->__pyx_base.src = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":307
 *       cdef dict revFeatures
 *       recRev.src = self.trg
 *       recRev.trg = self.src             # <<<<<<<<<<<<<<
 *       recRev.counts = self.counts.getReversed()
 *   #    recRev.aligns = record.getRevAligns(self.aligns)
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.src;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->__pyx_base.trg);
  __Pyx_DECREF(__pyx_v_recRev->__pyx_base.trg);
  __pyx_v_recRev->__pyx_base.trg = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":308
 *       recRev.src = self.trg
 *       recRev.trg = self.src
 *       recRev.counts = self.counts.getReversed()             # <<<<<<<<<<<<<<
 *   #    recRev.aligns = record.getRevAligns(self.aligns)
 *       recRev.aligns = getRevAlignSet(self.aligns)
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self->__pyx_base.counts->__pyx_vtab)->getReversed(__pyx_v_self->__pyx_base.counts, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->__pyx_base.counts);
  __Pyx_DECREF(((PyObject *)__pyx_v_recRev->__pyx_base.counts));
  __pyx_v_recRev->__pyx_base.counts = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":310
 *       recRev.counts = self.counts.getReversed()
 *   #    recRev.aligns = record.getRevAligns(self.aligns)
 *       recRev.aligns = getRevAlignSet(self.aligns)             # <<<<<<<<<<<<<<
 *       revFeatures = {}
 *       if 'egfp' in self.features:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_getRevAlignSet); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->__pyx_base.aligns); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->__pyx_base.aligns};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self->__pyx_base.aligns};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self->__pyx_base.aligns);
      __Pyx_GIVEREF(__pyx_v_self->__pyx_base.aligns);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self->__pyx_base.aligns);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PySet_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "set", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_recRev->__pyx_base.aligns);
  __Pyx_DECREF(__pyx_v_recRev->__pyx_base.aligns);
  __pyx_v_recRev->__pyx_base.aligns = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":311
 *   #    recRev.aligns = record.getRevAligns(self.aligns)
 *       recRev.aligns = getRevAlignSet(self.aligns)
 *       revFeatures = {}             # <<<<<<<<<<<<<<
 *       if 'egfp' in self.features:
 *         revFeatures['fgep'] = self.features['egfp']
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_revFeatures = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":312
 *       recRev.aligns = getRevAlignSet(self.aligns)
 *       revFeatures = {}
 *       if 'egfp' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['fgep'] = self.features['egfp']
 *       if 'egfl' in self.features:
 */
  if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 312, __pyx_L1_error)
  }
  __pyx_t_5 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_egfp, __pyx_v_self->__pyx_base.features, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 312, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":313
 *       revFeatures = {}
 *       if 'egfp' in self.features:
 *         revFeatures['fgep'] = self.features['egfp']             # <<<<<<<<<<<<<<
 *       if 'egfl' in self.features:
 *         revFeatures['fgel'] = self.features['egfl']
 */
    if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 313, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.features, __pyx_n_s_egfp); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_n_s_fgep, __pyx_t_1) < 0)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":312
 *       recRev.aligns = getRevAlignSet(self.aligns)
 *       revFeatures = {}
 *       if 'egfp' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['fgep'] = self.features['egfp']
 *       if 'egfl' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":314
 *       if 'egfp' in self.features:
 *         revFeatures['fgep'] = self.features['egfp']
 *       if 'egfl' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['fgel'] = self.features['egfl']
 *       if 'fgep' in self.features:
 */
  if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 314, __pyx_L1_error)
  }
  __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_egfl, __pyx_v_self->__pyx_base.features, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 314, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":315
 *         revFeatures['fgep'] = self.features['egfp']
 *       if 'egfl' in self.features:
 *         revFeatures['fgel'] = self.features['egfl']             # <<<<<<<<<<<<<<
 *       if 'fgep' in self.features:
 *         revFeatures['egfp'] = self.features['fgep']
 */
    if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 315, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.features, __pyx_n_s_egfl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_n_s_fgel, __pyx_t_1) < 0)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":314
 *       if 'egfp' in self.features:
 *         revFeatures['fgep'] = self.features['egfp']
 *       if 'egfl' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['fgel'] = self.features['egfl']
 *       if 'fgep' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":316
 *       if 'egfl' in self.features:
 *         revFeatures['fgel'] = self.features['egfl']
 *       if 'fgep' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['egfp'] = self.features['fgep']
 *       if 'fgel' in self.features:
 */
  if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 316, __pyx_L1_error)
  }
  __pyx_t_5 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_fgep, __pyx_v_self->__pyx_base.features, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 316, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":317
 *         revFeatures['fgel'] = self.features['egfl']
 *       if 'fgep' in self.features:
 *         revFeatures['egfp'] = self.features['fgep']             # <<<<<<<<<<<<<<
 *       if 'fgel' in self.features:
 *         revFeatures['egfl'] = self.features['fgel']
 */
    if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 317, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.features, __pyx_n_s_fgep); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_n_s_egfp, __pyx_t_1) < 0)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":316
 *       if 'egfl' in self.features:
 *         revFeatures['fgel'] = self.features['egfl']
 *       if 'fgep' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['egfp'] = self.features['fgep']
 *       if 'fgel' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":318
 *       if 'fgep' in self.features:
 *         revFeatures['egfp'] = self.features['fgep']
 *       if 'fgel' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['egfl'] = self.features['fgel']
 *       if 'p' in self.features:
 */
  if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 318, __pyx_L1_error)
  }
  __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_fgel, __pyx_v_self->__pyx_base.features, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 318, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":319
 *         revFeatures['egfp'] = self.features['fgep']
 *       if 'fgel' in self.features:
 *         revFeatures['egfl'] = self.features['fgel']             # <<<<<<<<<<<<<<
 *       if 'p' in self.features:
 *         revFeatures['p'] = self.features['p']
 */
    if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 319, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.features, __pyx_n_s_fgel); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_n_s_egfl, __pyx_t_1) < 0)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":318
 *       if 'fgep' in self.features:
 *         revFeatures['egfp'] = self.features['fgep']
 *       if 'fgel' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['egfl'] = self.features['fgel']
 *       if 'p' in self.features:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":320
 *       if 'fgel' in self.features:
 *         revFeatures['egfl'] = self.features['fgel']
 *       if 'p' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['p'] = self.features['p']
 *       revFeatures['w'] = len(self.srcTerms)
 */
  if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 320, __pyx_L1_error)
  }
  __pyx_t_5 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_p, __pyx_v_self->__pyx_base.features, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 320, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":321
 *         revFeatures['egfl'] = self.features['fgel']
 *       if 'p' in self.features:
 *         revFeatures['p'] = self.features['p']             # <<<<<<<<<<<<<<
 *       revFeatures['w'] = len(self.srcTerms)
 *       recRev.features = revFeatures
 */
    if (unlikely(__pyx_v_self->__pyx_base.features == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 321, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__pyx_base.features, __pyx_n_s_p); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_n_s_p, __pyx_t_1) < 0)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":320
 *       if 'fgel' in self.features:
 *         revFeatures['egfl'] = self.features['fgel']
 *       if 'p' in self.features:             # <<<<<<<<<<<<<<
 *         revFeatures['p'] = self.features['p']
 *       revFeatures['w'] = len(self.srcTerms)
 */
  }

  /* "lpu/smt/trans_models/records.pyx":322
 *       if 'p' in self.features:
 *         revFeatures['p'] = self.features['p']
 *       revFeatures['w'] = len(self.srcTerms)             # <<<<<<<<<<<<<<
 *       recRev.features = revFeatures
 *       return recRev
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_srcTerms); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyDict_SetItem(__pyx_v_revFeatures, __pyx_n_s_w, __pyx_t_1) < 0)) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":323
 *         revFeatures['p'] = self.features['p']
 *       revFeatures['w'] = len(self.srcTerms)
 *       recRev.features = revFeatures             # <<<<<<<<<<<<<<
 *       return recRev
 * 
 */
  __Pyx_INCREF(__pyx_v_revFeatures);
  __Pyx_GIVEREF(__pyx_v_revFeatures);
  __Pyx_GOTREF(__pyx_v_recRev->__pyx_base.features);
  __Pyx_DECREF(__pyx_v_recRev->__pyx_base.features);
  __pyx_v_recRev->__pyx_base.features = __pyx_v_revFeatures;

  /* "lpu/smt/trans_models/records.pyx":324
 *       revFeatures['w'] = len(self.srcTerms)
 *       recRev.features = revFeatures
 *       return recRev             # <<<<<<<<<<<<<<
 * 
 * #    cpdef loadLine(self, str line, str delim = '|||'):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_recRev));
  __pyx_r = __pyx_v_recRev;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":303
 *         return changed
 * 
 *     cpdef TravatarRecord getReversed(self):             # <<<<<<<<<<<<<<
 *       cdef TravatarRecord recRev = TravatarRecord()
 *       cdef dict revFeatures
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.getReversed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_recRev);
  __Pyx_XDECREF(__pyx_v_revFeatures);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_5getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_5getReversed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getReversed (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_4getReversed(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_4getReversed(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getReversed", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.getReversed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed__pyx_wrap_1(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch) {
  return __pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed(__pyx_v_self, __pyx_skip_dispatch);
}

/* "lpu/smt/trans_models/records.pyx":328
 * #    cpdef loadLine(self, str line, str delim = '|||'):
 *     #def loadLine(self, line, delim = '|||'):
 *     cpdef loadLine(self, str line, str delim = '|||'):             # <<<<<<<<<<<<<<
 *         cdef list fields
 *         cdef list listCounts
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_7loadLine(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_line, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine *__pyx_optional_args) {
  PyObject *__pyx_v_delim = ((PyObject*)__pyx_kp_s__9);
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_listCounts = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts __pyx_t_10;
  __Pyx_RefNannySetupContext("loadLine", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_delim = __pyx_optional_args->delim;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_loadLine); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_7loadLine)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_line, __pyx_v_delim};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_line, __pyx_v_delim};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 328, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_line);
        __Pyx_GIVEREF(__pyx_v_line);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_line);
        __Pyx_INCREF(__pyx_v_delim);
        __Pyx_GIVEREF(__pyx_v_delim);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_delim);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":331
 *         cdef list fields
 *         cdef list listCounts
 *         if line:             # <<<<<<<<<<<<<<
 *             fields = line.strip().split(delim)
 *             self.src = fields[0].strip()
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_line); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "lpu/smt/trans_models/records.pyx":332
 *         cdef list listCounts
 *         if line:
 *             fields = line.strip().split(delim)             # <<<<<<<<<<<<<<
 *             self.src = fields[0].strip()
 *             self.trg = fields[1].strip()
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_line, __pyx_n_s_strip); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_6) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_delim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_delim};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_delim};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 332, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2); __pyx_t_2 = NULL;
        __Pyx_INCREF(__pyx_v_delim);
        __Pyx_GIVEREF(__pyx_v_delim);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_delim);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 332, __pyx_L1_error)
    __pyx_v_fields = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":333
 *         if line:
 *             fields = line.strip().split(delim)
 *             self.src = fields[0].strip()             # <<<<<<<<<<<<<<
 *             self.trg = fields[1].strip()
 *             #self.src = vocab.phrase2id(fields[0])
 */
    if (unlikely(__pyx_v_fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 333, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_GetItemInt_List(__pyx_v_fields, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_strip); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    if (__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 333, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.src);
    __Pyx_DECREF(__pyx_v_self->__pyx_base.src);
    __pyx_v_self->__pyx_base.src = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":334
 *             fields = line.strip().split(delim)
 *             self.src = fields[0].strip()
 *             self.trg = fields[1].strip()             # <<<<<<<<<<<<<<
 *             #self.src = vocab.phrase2id(fields[0])
 *             #self.trg = vocab.phrase2id(fields[1])
 */
    if (unlikely(__pyx_v_fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 334, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_List(__pyx_v_fields, 1, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_strip); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_6) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.trg);
    __Pyx_DECREF(__pyx_v_self->__pyx_base.trg);
    __pyx_v_self->__pyx_base.trg = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":337
 *             #self.src = vocab.phrase2id(fields[0])
 *             #self.trg = vocab.phrase2id(fields[1])
 *             self.features = getTravatarFeatures(fields[2])             # <<<<<<<<<<<<<<
 *             listCounts = getCounts(fields[3])
 *             if len(listCounts) == 3:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_getTravatarFeatures); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 337, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(__pyx_v_fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 337, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_List(__pyx_v_fields, 2, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 337, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 337, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 337, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.features);
    __Pyx_DECREF(__pyx_v_self->__pyx_base.features);
    __pyx_v_self->__pyx_base.features = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":338
 *             #self.trg = vocab.phrase2id(fields[1])
 *             self.features = getTravatarFeatures(fields[2])
 *             listCounts = getCounts(fields[3])             # <<<<<<<<<<<<<<
 *             if len(listCounts) == 3:
 *                 self.counts.setCounts(cooc = listCounts[0], src = listCounts[1], trg = listCounts[2])
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_getCounts); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(__pyx_v_fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 338, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_fields, 3, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 338, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 338, __pyx_L1_error)
    __pyx_v_listCounts = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":339
 *             self.features = getTravatarFeatures(fields[2])
 *             listCounts = getCounts(fields[3])
 *             if len(listCounts) == 3:             # <<<<<<<<<<<<<<
 *                 self.counts.setCounts(cooc = listCounts[0], src = listCounts[1], trg = listCounts[2])
 *             #self.aligns = fields[4].strip().split()
 */
    if (unlikely(__pyx_v_listCounts == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 339, __pyx_L1_error)
    }
    __pyx_t_8 = PyList_GET_SIZE(__pyx_v_listCounts); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 339, __pyx_L1_error)
    __pyx_t_7 = ((__pyx_t_8 == 3) != 0);
    if (__pyx_t_7) {

      /* "lpu/smt/trans_models/records.pyx":340
 *             listCounts = getCounts(fields[3])
 *             if len(listCounts) == 3:
 *                 self.counts.setCounts(cooc = listCounts[0], src = listCounts[1], trg = listCounts[2])             # <<<<<<<<<<<<<<
 *             #self.aligns = fields[4].strip().split()
 *             self.aligns = getAlignSet( fields[4] )
 */
      if (unlikely(__pyx_v_listCounts == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 340, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_listCounts, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (unlikely(__pyx_v_listCounts == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 340, __pyx_L1_error)
      }
      __pyx_t_3 = __Pyx_GetItemInt_List(__pyx_v_listCounts, 1, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 340, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(__pyx_v_listCounts == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 340, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_listCounts, 2, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 340, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __pyx_t_3;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_6 = __pyx_t_2;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_9 = __pyx_t_1;
      __Pyx_INCREF(__pyx_t_9);
      __pyx_t_10.__pyx_n = 3;
      __pyx_t_10.src = __pyx_t_4;
      __pyx_t_10.trg = __pyx_t_6;
      __pyx_t_10.cooc = __pyx_t_9;
      ((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self->__pyx_base.counts->__pyx_vtab)->setCounts(__pyx_v_self->__pyx_base.counts, 0, &__pyx_t_10); 
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":339
 *             self.features = getTravatarFeatures(fields[2])
 *             listCounts = getCounts(fields[3])
 *             if len(listCounts) == 3:             # <<<<<<<<<<<<<<
 *                 self.counts.setCounts(cooc = listCounts[0], src = listCounts[1], trg = listCounts[2])
 *             #self.aligns = fields[4].strip().split()
 */
    }

    /* "lpu/smt/trans_models/records.pyx":342
 *                 self.counts.setCounts(cooc = listCounts[0], src = listCounts[1], trg = listCounts[2])
 *             #self.aligns = fields[4].strip().split()
 *             self.aligns = getAlignSet( fields[4] )             # <<<<<<<<<<<<<<
 *             #self.data = pd.DataFrame()
 *             #self.data = pd.Series()
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_getAlignSet); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 342, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(__pyx_v_fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 342, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_fields, 4, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 342, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_9) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_2};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_2};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 342, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_9); __pyx_t_9 = NULL;
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PySet_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "set", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 342, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.aligns);
    __Pyx_DECREF(__pyx_v_self->__pyx_base.aligns);
    __pyx_v_self->__pyx_base.aligns = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":331
 *         cdef list fields
 *         cdef list listCounts
 *         if line:             # <<<<<<<<<<<<<<
 *             fields = line.strip().split(delim)
 *             self.src = fields[0].strip()
 */
  }

  /* "lpu/smt/trans_models/records.pyx":328
 * #    cpdef loadLine(self, str line, str delim = '|||'):
 *     #def loadLine(self, line, delim = '|||'):
 *     cpdef loadLine(self, str line, str delim = '|||'):             # <<<<<<<<<<<<<<
 *         cdef list fields
 *         cdef list listCounts
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.loadLine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_listCounts);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_7loadLine(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_7loadLine(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_line = 0;
  PyObject *__pyx_v_delim = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("loadLine (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_line,&__pyx_n_s_delim,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject*)__pyx_kp_s__9);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_line)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_delim);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "loadLine") < 0)) __PYX_ERR(0, 328, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_line = ((PyObject*)values[0]);
    __pyx_v_delim = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("loadLine", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 328, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.loadLine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_line), (&PyString_Type), 1, "line", 1))) __PYX_ERR(0, 328, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_delim), (&PyString_Type), 1, "delim", 1))) __PYX_ERR(0, 328, __pyx_L1_error)
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_6loadLine(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self), __pyx_v_line, __pyx_v_delim);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_6loadLine(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_line, PyObject *__pyx_v_delim) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine __pyx_t_2;
  __Pyx_RefNannySetupContext("loadLine", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.delim = __pyx_v_delim;
  __pyx_t_1 = __pyx_vtabptr_3lpu_3smt_12trans_models_7records_TravatarRecord->loadLine(__pyx_v_self, __pyx_v_line, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.loadLine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":346
 *             #self.data = pd.Series()
 * 
 *     cpdef syncTags(self):             # <<<<<<<<<<<<<<
 *         self.sync = False
 *         src_symbols = self.src.split(' ')
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_9syncTags(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_syncTags(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_src_symbols = NULL;
  PyObject *__pyx_v_trgSymbols = NULL;
  PyObject *__pyx_v_srcTag = NULL;
  PyObject *__pyx_v_srcNonTerminals = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_index = NULL;
  PyObject *__pyx_v_tag = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  __Pyx_RefNannySetupContext("syncTags", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_syncTags); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 346, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_9syncTags)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 346, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 346, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":347
 * 
 *     cpdef syncTags(self):
 *         self.sync = False             # <<<<<<<<<<<<<<
 *         src_symbols = self.src.split(' ')
 *         trgSymbols = self.trg.split(' ')
 */
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_sync, Py_False) < 0) __PYX_ERR(0, 347, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":348
 *     cpdef syncTags(self):
 *         self.sync = False
 *         src_symbols = self.src.split(' ')             # <<<<<<<<<<<<<<
 *         trgSymbols = self.trg.split(' ')
 * #        if len(src_symbols) >= 4 and src_symbols[0][0] != '"':
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.src, __pyx_n_s_split); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_src_symbols = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":349
 *         self.sync = False
 *         src_symbols = self.src.split(' ')
 *         trgSymbols = self.trg.split(' ')             # <<<<<<<<<<<<<<
 * #        if len(src_symbols) >= 4 and src_symbols[0][0] != '"':
 *         if isTree(src_symbols):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base.trg, __pyx_n_s_split); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_trgSymbols = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":351
 *         trgSymbols = self.trg.split(' ')
 * #        if len(src_symbols) >= 4 and src_symbols[0][0] != '"':
 *         if isTree(src_symbols):             # <<<<<<<<<<<<<<
 *             # src is tree (s-expression)
 *             if len(trgSymbols) >= 2 and trgSymbols[-2] == "@":
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_isTree); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_src_symbols); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_src_symbols};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_src_symbols};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_src_symbols);
      __Pyx_GIVEREF(__pyx_v_src_symbols);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_src_symbols);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":353
 *         if isTree(src_symbols):
 *             # src is tree (s-expression)
 *             if len(trgSymbols) >= 2 and trgSymbols[-2] == "@":             # <<<<<<<<<<<<<<
 *                 # trg is tagged
 *                 pass
 */
    __pyx_t_6 = PyObject_Length(__pyx_v_trgSymbols); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 353, __pyx_L1_error)
    __pyx_t_7 = ((__pyx_t_6 >= 2) != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_5 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_trgSymbols, -2L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_kp_s__20, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 353, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __pyx_t_7;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_5) {
      goto __pyx_L4;
    }

    /* "lpu/smt/trans_models/records.pyx":358
 *             else:
 *                 # trg is not tagged
 *                 srcTag = src_symbols[0]             # <<<<<<<<<<<<<<
 *                 srcNonTerminals = []
 *                 for s in src_symbols:
 */
    /*else*/ {
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_src_symbols, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_srcTag = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":359
 *                 # trg is not tagged
 *                 srcTag = src_symbols[0]
 *                 srcNonTerminals = []             # <<<<<<<<<<<<<<
 *                 for s in src_symbols:
 *                     if s[0:1] == 'x' and s[1:2].isdigit():
 */
      __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 359, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_srcNonTerminals = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":360
 *                 srcTag = src_symbols[0]
 *                 srcNonTerminals = []
 *                 for s in src_symbols:             # <<<<<<<<<<<<<<
 *                     if s[0:1] == 'x' and s[1:2].isdigit():
 *                         srcNonTerminals.append(s)
 */
      if (likely(PyList_CheckExact(__pyx_v_src_symbols)) || PyTuple_CheckExact(__pyx_v_src_symbols)) {
        __pyx_t_1 = __pyx_v_src_symbols; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
        __pyx_t_8 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_src_symbols); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_8 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 360, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_8)) {
          if (likely(PyList_CheckExact(__pyx_t_1))) {
            if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 360, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          } else {
            if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 360, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          }
        } else {
          __pyx_t_2 = __pyx_t_8(__pyx_t_1);
          if (unlikely(!__pyx_t_2)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 360, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "lpu/smt/trans_models/records.pyx":361
 *                 srcNonTerminals = []
 *                 for s in src_symbols:
 *                     if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 *                         srcNonTerminals.append(s)
 * #                        if len(s) > 3:
 */
        __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_s, 0, 1, NULL, NULL, &__pyx_slice__21, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 361, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 361, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (__pyx_t_7) {
        } else {
          __pyx_t_5 = __pyx_t_7;
          goto __pyx_L10_bool_binop_done;
        }
        __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_s, 1, 2, NULL, NULL, &__pyx_slice__22, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 361, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_isdigit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 361, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        if (__pyx_t_4) {
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 361, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else {
          __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 361, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 361, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_5 = __pyx_t_7;
        __pyx_L10_bool_binop_done:;
        if (__pyx_t_5) {

          /* "lpu/smt/trans_models/records.pyx":362
 *                 for s in src_symbols:
 *                     if s[0:1] == 'x' and s[1:2].isdigit():
 *                         srcNonTerminals.append(s)             # <<<<<<<<<<<<<<
 * #                        if len(s) > 3:
 * #                            srcNonTerminals.append(s[3:])
 */
          __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_srcNonTerminals, __pyx_v_s); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 362, __pyx_L1_error)

          /* "lpu/smt/trans_models/records.pyx":361
 *                 srcNonTerminals = []
 *                 for s in src_symbols:
 *                     if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 *                         srcNonTerminals.append(s)
 * #                        if len(s) > 3:
 */
        }

        /* "lpu/smt/trans_models/records.pyx":360
 *                 srcTag = src_symbols[0]
 *                 srcNonTerminals = []
 *                 for s in src_symbols:             # <<<<<<<<<<<<<<
 *                     if s[0:1] == 'x' and s[1:2].isdigit():
 *                         srcNonTerminals.append(s)
 */
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":367
 * #                        else:
 * #                            srcNonTerminals.append('X')
 *                 for i, s in enumerate(trgSymbols):             # <<<<<<<<<<<<<<
 * #                    print("TRG SYMBOLS: %s" % s)
 *                     if s[0:1] == 'x' and s[1:2].isdigit():
 */
      __Pyx_INCREF(__pyx_int_0);
      __pyx_t_1 = __pyx_int_0;
      if (likely(PyList_CheckExact(__pyx_v_trgSymbols)) || PyTuple_CheckExact(__pyx_v_trgSymbols)) {
        __pyx_t_2 = __pyx_v_trgSymbols; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
        __pyx_t_8 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_trgSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 367, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_8 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 367, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_8)) {
          if (likely(PyList_CheckExact(__pyx_t_2))) {
            if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_3); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 367, __pyx_L1_error)
            #else
            __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 367, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          } else {
            if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_3); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 367, __pyx_L1_error)
            #else
            __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 367, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          }
        } else {
          __pyx_t_3 = __pyx_t_8(__pyx_t_2);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 367, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
        __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 367, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1);
        __pyx_t_1 = __pyx_t_3;
        __pyx_t_3 = 0;

        /* "lpu/smt/trans_models/records.pyx":369
 *                 for i, s in enumerate(trgSymbols):
 * #                    print("TRG SYMBOLS: %s" % s)
 *                     if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 * #                        print("NONTERMINAL!")
 *                         if len(s) > 3:
 */
        __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_s, 0, 1, NULL, NULL, &__pyx_slice__23, 1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_3, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 369, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_7) {
        } else {
          __pyx_t_5 = __pyx_t_7;
          goto __pyx_L15_bool_binop_done;
        }
        __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_s, 1, 2, NULL, NULL, &__pyx_slice__24, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 369, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_isdigit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 369, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        if (__pyx_t_4) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else {
          __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 369, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_t_7;
        __pyx_L15_bool_binop_done:;
        if (__pyx_t_5) {

          /* "lpu/smt/trans_models/records.pyx":371
 *                     if s[0:1] == 'x' and s[1:2].isdigit():
 * #                        print("NONTERMINAL!")
 *                         if len(s) > 3:             # <<<<<<<<<<<<<<
 *                             # symbol is tagged
 *                             pass
 */
          __pyx_t_11 = PyObject_Length(__pyx_v_s); if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 371, __pyx_L1_error)
          __pyx_t_5 = ((__pyx_t_11 > 3) != 0);
          if (__pyx_t_5) {
            goto __pyx_L17;
          }

          /* "lpu/smt/trans_models/records.pyx":376
 *                         else:
 *                             # symbol is not tagged
 *                             index = int(s[1])             # <<<<<<<<<<<<<<
 *                             if self.sync:
 *                                 trgSymbols[i] = srcNonTerminals[index]
 */
          /*else*/ {
            __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 376, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_10 = __Pyx_PyNumber_Int(__pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 376, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_10);
            __pyx_t_10 = 0;

            /* "lpu/smt/trans_models/records.pyx":377
 *                             # symbol is not tagged
 *                             index = int(s[1])
 *                             if self.sync:             # <<<<<<<<<<<<<<
 *                                 trgSymbols[i] = srcNonTerminals[index]
 *                             else:
 */
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_sync); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 377, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 377, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (__pyx_t_5) {

              /* "lpu/smt/trans_models/records.pyx":378
 *                             index = int(s[1])
 *                             if self.sync:
 *                                 trgSymbols[i] = srcNonTerminals[index]             # <<<<<<<<<<<<<<
 *                             else:
 *                                 trgSymbols[i] = s + ":X"
 */
              __pyx_t_10 = PyObject_GetItem(__pyx_v_srcNonTerminals, __pyx_v_index); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 378, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (unlikely(PyObject_SetItem(__pyx_v_trgSymbols, __pyx_v_i, __pyx_t_10) < 0)) __PYX_ERR(0, 378, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

              /* "lpu/smt/trans_models/records.pyx":377
 *                             # symbol is not tagged
 *                             index = int(s[1])
 *                             if self.sync:             # <<<<<<<<<<<<<<
 *                                 trgSymbols[i] = srcNonTerminals[index]
 *                             else:
 */
              goto __pyx_L18;
            }

            /* "lpu/smt/trans_models/records.pyx":380
 *                                 trgSymbols[i] = srcNonTerminals[index]
 *                             else:
 *                                 trgSymbols[i] = s + ":X"             # <<<<<<<<<<<<<<
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *                 trgSymbols.append("@")
 */
            /*else*/ {
              __pyx_t_10 = PyNumber_Add(__pyx_v_s, __pyx_kp_s_X); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 380, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (unlikely(PyObject_SetItem(__pyx_v_trgSymbols, __pyx_v_i, __pyx_t_10) < 0)) __PYX_ERR(0, 380, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
            __pyx_L18:;
          }
          __pyx_L17:;

          /* "lpu/smt/trans_models/records.pyx":369
 *                 for i, s in enumerate(trgSymbols):
 * #                    print("TRG SYMBOLS: %s" % s)
 *                     if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 * #                        print("NONTERMINAL!")
 *                         if len(s) > 3:
 */
        }

        /* "lpu/smt/trans_models/records.pyx":367
 * #                        else:
 * #                            srcNonTerminals.append('X')
 *                 for i, s in enumerate(trgSymbols):             # <<<<<<<<<<<<<<
 * #                    print("TRG SYMBOLS: %s" % s)
 *                     if s[0:1] == 'x' and s[1:2].isdigit():
 */
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":382
 *                                 trgSymbols[i] = s + ":X"
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *                 trgSymbols.append("@")             # <<<<<<<<<<<<<<
 *                 if self.sync:
 *                     trgSymbols.append(srcTag)
 */
      __pyx_t_9 = __Pyx_PyObject_Append(__pyx_v_trgSymbols, __pyx_kp_s__20); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 382, __pyx_L1_error)

      /* "lpu/smt/trans_models/records.pyx":383
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *                 trgSymbols.append("@")
 *                 if self.sync:             # <<<<<<<<<<<<<<
 *                     trgSymbols.append(srcTag)
 *                 else:
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_sync); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_5) {

        /* "lpu/smt/trans_models/records.pyx":384
 *                 trgSymbols.append("@")
 *                 if self.sync:
 *                     trgSymbols.append(srcTag)             # <<<<<<<<<<<<<<
 *                 else:
 *                     trgSymbols.append('X')
 */
        __pyx_t_9 = __Pyx_PyObject_Append(__pyx_v_trgSymbols, __pyx_v_srcTag); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 384, __pyx_L1_error)

        /* "lpu/smt/trans_models/records.pyx":383
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *                 trgSymbols.append("@")
 *                 if self.sync:             # <<<<<<<<<<<<<<
 *                     trgSymbols.append(srcTag)
 *                 else:
 */
        goto __pyx_L19;
      }

      /* "lpu/smt/trans_models/records.pyx":386
 *                     trgSymbols.append(srcTag)
 *                 else:
 *                     trgSymbols.append('X')             # <<<<<<<<<<<<<<
 *                 self.trg = intern(str.join(' ', trgSymbols))
 *         if len(src_symbols) == 3:
 */
      /*else*/ {
        __pyx_t_9 = __Pyx_PyObject_Append(__pyx_v_trgSymbols, __pyx_n_s_X_2); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 386, __pyx_L1_error)
      }
      __pyx_L19:;

      /* "lpu/smt/trans_models/records.pyx":387
 *                 else:
 *                     trgSymbols.append('X')
 *                 self.trg = intern(str.join(' ', trgSymbols))             # <<<<<<<<<<<<<<
 *         if len(src_symbols) == 3:
 *             tag = src_symbols[2]
 */
      __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s__4, __pyx_v_trgSymbols); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_Intern(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 387, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 387, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_v_self->__pyx_base.trg);
      __Pyx_DECREF(__pyx_v_self->__pyx_base.trg);
      __pyx_v_self->__pyx_base.trg = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L4:;

    /* "lpu/smt/trans_models/records.pyx":351
 *         trgSymbols = self.trg.split(' ')
 * #        if len(src_symbols) >= 4 and src_symbols[0][0] != '"':
 *         if isTree(src_symbols):             # <<<<<<<<<<<<<<
 *             # src is tree (s-expression)
 *             if len(trgSymbols) >= 2 and trgSymbols[-2] == "@":
 */
  }

  /* "lpu/smt/trans_models/records.pyx":388
 *                     trgSymbols.append('X')
 *                 self.trg = intern(str.join(' ', trgSymbols))
 *         if len(src_symbols) == 3:             # <<<<<<<<<<<<<<
 *             tag = src_symbols[2]
 *             if src_symbols[0] == "x0:"+tag:
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_src_symbols); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 388, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_6 == 3) != 0);
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":389
 *                 self.trg = intern(str.join(' ', trgSymbols))
 *         if len(src_symbols) == 3:
 *             tag = src_symbols[2]             # <<<<<<<<<<<<<<
 *             if src_symbols[0] == "x0:"+tag:
 *                 # src is unary cycle
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_src_symbols, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_tag = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "lpu/smt/trans_models/records.pyx":390
 *         if len(src_symbols) == 3:
 *             tag = src_symbols[2]
 *             if src_symbols[0] == "x0:"+tag:             # <<<<<<<<<<<<<<
 *                 # src is unary cycle
 *                 #self.src = ""
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_src_symbols, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Add(__pyx_kp_s_x0, __pyx_v_tag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (__pyx_t_5) {

      /* "lpu/smt/trans_models/records.pyx":393
 *                 # src is unary cycle
 *                 #self.src = ""
 *                 self.src = ()             # <<<<<<<<<<<<<<
 * 
 * #    cpdef toDict(self):
 */
      if (!(likely(PyString_CheckExact(__pyx_empty_tuple))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_empty_tuple)->tp_name), 0))) __PYX_ERR(0, 393, __pyx_L1_error)
      __pyx_t_10 = __pyx_empty_tuple;
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_10);
      __Pyx_GOTREF(__pyx_v_self->__pyx_base.src);
      __Pyx_DECREF(__pyx_v_self->__pyx_base.src);
      __pyx_v_self->__pyx_base.src = ((PyObject*)__pyx_t_10);
      __pyx_t_10 = 0;

      /* "lpu/smt/trans_models/records.pyx":390
 *         if len(src_symbols) == 3:
 *             tag = src_symbols[2]
 *             if src_symbols[0] == "x0:"+tag:             # <<<<<<<<<<<<<<
 *                 # src is unary cycle
 *                 #self.src = ""
 */
    }

    /* "lpu/smt/trans_models/records.pyx":388
 *                     trgSymbols.append('X')
 *                 self.trg = intern(str.join(' ', trgSymbols))
 *         if len(src_symbols) == 3:             # <<<<<<<<<<<<<<
 *             tag = src_symbols[2]
 *             if src_symbols[0] == "x0:"+tag:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":346
 *             #self.data = pd.Series()
 * 
 *     cpdef syncTags(self):             # <<<<<<<<<<<<<<
 *         self.sync = False
 *         src_symbols = self.src.split(' ')
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.syncTags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_src_symbols);
  __Pyx_XDECREF(__pyx_v_trgSymbols);
  __Pyx_XDECREF(__pyx_v_srcTag);
  __Pyx_XDECREF(__pyx_v_srcNonTerminals);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_9syncTags(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_9syncTags(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("syncTags (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_8syncTags(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_8syncTags(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("syncTags", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_syncTags(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.syncTags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":404
 * #        return d
 * 
 *     cpdef to_str(self, s = ' ||| '):             # <<<<<<<<<<<<<<
 *       strFeatures = getStrTravatarFeatures(self.features)
 *       self.counts.round()
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_11to_str(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str *__pyx_optional_args) {
  PyObject *__pyx_v_s = ((PyObject *)__pyx_kp_s__10);
  PyObject *__pyx_v_strFeatures = NULL;
  PyObject *__pyx_v_strCounts = NULL;
  PyObject *__pyx_v_strAligns = NULL;
  PyObject *__pyx_v_buf = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("to_str", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_s = __pyx_optional_args->s;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_to_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_11to_str)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_s); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 404, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_s);
          __Pyx_GIVEREF(__pyx_v_s);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_s);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "lpu/smt/trans_models/records.pyx":405
 * 
 *     cpdef to_str(self, s = ' ||| '):
 *       strFeatures = getStrTravatarFeatures(self.features)             # <<<<<<<<<<<<<<
 *       self.counts.round()
 *       strCounts = ""
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.features;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_3lpu_3smt_12trans_models_7records_getStrTravatarFeatures(((PyObject*)__pyx_t_1), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_strFeatures = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":406
 *     cpdef to_str(self, s = ' ||| '):
 *       strFeatures = getStrTravatarFeatures(self.features)
 *       self.counts.round()             # <<<<<<<<<<<<<<
 *       strCounts = ""
 *       if self.counts.cooc > 0:
 */
  ((struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence *)__pyx_v_self->__pyx_base.counts->__pyx_vtab)->round(__pyx_v_self->__pyx_base.counts, NULL);

  /* "lpu/smt/trans_models/records.pyx":407
 *       strFeatures = getStrTravatarFeatures(self.features)
 *       self.counts.round()
 *       strCounts = ""             # <<<<<<<<<<<<<<
 *       if self.counts.cooc > 0:
 *           #self.counts.cooc = round(self.counts.cooc, 6)
 */
  __Pyx_INCREF(__pyx_kp_s__3);
  __pyx_v_strCounts = __pyx_kp_s__3;

  /* "lpu/smt/trans_models/records.pyx":408
 *       self.counts.round()
 *       strCounts = ""
 *       if self.counts.cooc > 0:             # <<<<<<<<<<<<<<
 *           #self.counts.cooc = round(self.counts.cooc, 6)
 *           #strCounts = "%s %s %s" % (self.counts.cooc, self.counts.src, self.counts.trg)
 */
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.counts->cooc > 0.0) != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":411
 *           #self.counts.cooc = round(self.counts.cooc, 6)
 *           #strCounts = "%s %s %s" % (self.counts.cooc, self.counts.src, self.counts.trg)
 *           strCounts = "%s %s %s" % (round(self.counts.cooc,6), round(self.counts.src,6), round(self.counts.trg,6))             # <<<<<<<<<<<<<<
 *   #    strAligns = str.join(' ', self.aligns)
 *       strAligns = str.join(' ', sorted(self.aligns))
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.counts->cooc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_int_6);
    __Pyx_GIVEREF(__pyx_int_6);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_6);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.counts->src); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_int_6);
    __Pyx_GIVEREF(__pyx_int_6);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_6);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.counts->trg); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
    __Pyx_INCREF(__pyx_int_6);
    __Pyx_GIVEREF(__pyx_int_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_6);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
    __pyx_t_2 = 0;
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_s_s_s, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_strCounts, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "lpu/smt/trans_models/records.pyx":408
 *       self.counts.round()
 *       strCounts = ""
 *       if self.counts.cooc > 0:             # <<<<<<<<<<<<<<
 *           #self.counts.cooc = round(self.counts.cooc, 6)
 *           #strCounts = "%s %s %s" % (self.counts.cooc, self.counts.src, self.counts.trg)
 */
  }

  /* "lpu/smt/trans_models/records.pyx":413
 *           strCounts = "%s %s %s" % (round(self.counts.cooc,6), round(self.counts.src,6), round(self.counts.trg,6))
 *   #    strAligns = str.join(' ', self.aligns)
 *       strAligns = str.join(' ', sorted(self.aligns))             # <<<<<<<<<<<<<<
 *       #buf = str.join(s, [self.src, self.trg, strFeatures, strCounts, strAligns]) + "\n"
 *       buf = str.join(s, [self.src, self.trg, strFeatures, strCounts, strAligns])
 */
  __pyx_t_5 = __pyx_v_self->__pyx_base.aligns;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_1 = PySequence_List(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_3 = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_7 = PyList_Sort(__pyx_t_3); if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 413, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s__4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_strAligns = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":415
 *       strAligns = str.join(' ', sorted(self.aligns))
 *       #buf = str.join(s, [self.src, self.trg, strFeatures, strCounts, strAligns]) + "\n"
 *       buf = str.join(s, [self.src, self.trg, strFeatures, strCounts, strAligns])             # <<<<<<<<<<<<<<
 *       return buf
 * 
 */
  if (unlikely(__pyx_v_s == Py_None)) {
    PyErr_Format(PyExc_TypeError, "descriptor '%s' requires a '%s' object but received a 'NoneType'", "join", "str");
    __PYX_ERR(0, 415, __pyx_L1_error)
  }
  if (!(likely(PyString_Check(__pyx_v_s))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_s)->tp_name), 0))) __PYX_ERR(0, 415, __pyx_L1_error)
  __pyx_t_1 = PyList_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.src);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.src);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->__pyx_base.src);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.trg);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.trg);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->__pyx_base.trg);
  __Pyx_INCREF(__pyx_v_strFeatures);
  __Pyx_GIVEREF(__pyx_v_strFeatures);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_v_strFeatures);
  __Pyx_INCREF(__pyx_v_strCounts);
  __Pyx_GIVEREF(__pyx_v_strCounts);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_v_strCounts);
  __Pyx_INCREF(__pyx_v_strAligns);
  __Pyx_GIVEREF(__pyx_v_strAligns);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_v_strAligns);
  __pyx_t_3 = __Pyx_PyString_Join(((PyObject*)__pyx_v_s), __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_buf = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "lpu/smt/trans_models/records.pyx":416
 *       #buf = str.join(s, [self.src, self.trg, strFeatures, strCounts, strAligns]) + "\n"
 *       buf = str.join(s, [self.src, self.trg, strFeatures, strCounts, strAligns])
 *       return buf             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_buf);
  __pyx_r = __pyx_v_buf;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":404
 * #        return d
 * 
 *     cpdef to_str(self, s = ' ||| '):             # <<<<<<<<<<<<<<
 *       strFeatures = getStrTravatarFeatures(self.features)
 *       self.counts.round()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_strFeatures);
  __Pyx_XDECREF(__pyx_v_strCounts);
  __Pyx_XDECREF(__pyx_v_strAligns);
  __Pyx_XDECREF(__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_11to_str(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_11to_str(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_s = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("to_str (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_s,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_kp_s__10);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_s);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "to_str") < 0)) __PYX_ERR(0, 404, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_s = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("to_str", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 404, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_10to_str(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self), __pyx_v_s);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_10to_str(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str __pyx_t_2;
  __Pyx_RefNannySetupContext("to_str", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.s = __pyx_v_s;
  __pyx_t_1 = __pyx_vtabptr_3lpu_3smt_12trans_models_7records_TravatarRecord->to_str(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.to_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":419
 * 
 *     @staticmethod
 *     def fixOrderStrings(str src, str trg):             # <<<<<<<<<<<<<<
 *         cdef list srcElements = []
 *         cdef list trgElements = []
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_13fixOrderStrings(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_14TravatarRecord_13fixOrderStrings = {"fixOrderStrings", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_13fixOrderStrings, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_13fixOrderStrings(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_src = 0;
  PyObject *__pyx_v_trg = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fixOrderStrings (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_src,&__pyx_n_s_trg,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_src)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_trg)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("fixOrderStrings", 1, 2, 2, 1); __PYX_ERR(0, 419, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "fixOrderStrings") < 0)) __PYX_ERR(0, 419, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_src = ((PyObject*)values[0]);
    __pyx_v_trg = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("fixOrderStrings", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 419, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.fixOrderStrings", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src), (&PyString_Type), 1, "src", 1))) __PYX_ERR(0, 419, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_trg), (&PyString_Type), 1, "trg", 1))) __PYX_ERR(0, 419, __pyx_L1_error)
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_12fixOrderStrings(__pyx_v_src, __pyx_v_trg);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_12fixOrderStrings(PyObject *__pyx_v_src, PyObject *__pyx_v_trg) {
  PyObject *__pyx_v_srcElements = 0;
  PyObject *__pyx_v_trgElements = 0;
  PyObject *__pyx_v_replaceDict = 0;
  PyObject *__pyx_v_newSymbol = 0;
  PyObject *__pyx_v_changed = 0;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("fixOrderStrings", 0);

  /* "lpu/smt/trans_models/records.pyx":420
 *     @staticmethod
 *     def fixOrderStrings(str src, str trg):
 *         cdef list srcElements = []             # <<<<<<<<<<<<<<
 *         cdef list trgElements = []
 *         cdef dict replaceDict = {}
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_srcElements = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":421
 *     def fixOrderStrings(str src, str trg):
 *         cdef list srcElements = []
 *         cdef list trgElements = []             # <<<<<<<<<<<<<<
 *         cdef dict replaceDict = {}
 *         cdef str newSymbol
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_trgElements = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":422
 *         cdef list srcElements = []
 *         cdef list trgElements = []
 *         cdef dict replaceDict = {}             # <<<<<<<<<<<<<<
 *         cdef str newSymbol
 *         cdef object changed = False
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_replaceDict = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":424
 *         cdef dict replaceDict = {}
 *         cdef str newSymbol
 *         cdef object changed = False             # <<<<<<<<<<<<<<
 *         for s in src.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():
 */
  __Pyx_INCREF(Py_False);
  __pyx_v_changed = Py_False;

  /* "lpu/smt/trans_models/records.pyx":425
 *         cdef str newSymbol
 *         cdef object changed = False
 *         for s in src.split(' '):             # <<<<<<<<<<<<<<
 *             if s[0] == 'x' and s[1].isdigit():
 *                 replaceDict[s[1]] = str(len(replaceDict))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_src, __pyx_n_s_split); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 425, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 425, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 425, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 425, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "lpu/smt/trans_models/records.pyx":426
 *         cdef object changed = False
 *         for s in src.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():             # <<<<<<<<<<<<<<
 *                 replaceDict[s[1]] = str(len(replaceDict))
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_s, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_isdigit); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    if (__pyx_t_7) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __pyx_t_6;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_5) {

      /* "lpu/smt/trans_models/records.pyx":427
 *         for s in src.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():
 *                 replaceDict[s[1]] = str(len(replaceDict))             # <<<<<<<<<<<<<<
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 *                 srcElements.append(newSymbol)
 */
      __pyx_t_9 = PyDict_Size(__pyx_v_replaceDict); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 427, __pyx_L1_error)
      __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely(PyDict_SetItem(__pyx_v_replaceDict, __pyx_t_8, __pyx_t_2) < 0)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "lpu/smt/trans_models/records.pyx":428
 *             if s[0] == 'x' and s[1].isdigit():
 *                 replaceDict[s[1]] = str(len(replaceDict))
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]             # <<<<<<<<<<<<<<
 *                 srcElements.append(newSymbol)
 *                 if s[1] != newSymbol[1]:
 */
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 428, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_v_replaceDict, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 428, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyNumber_Add(__pyx_n_s_x, __pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 428, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetSlice(__pyx_v_s, 2, 0, NULL, NULL, &__pyx_slice__26, 1, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 428, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = PyNumber_Add(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 428, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (!(likely(PyString_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(0, 428, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_newSymbol, ((PyObject*)__pyx_t_7));
      __pyx_t_7 = 0;

      /* "lpu/smt/trans_models/records.pyx":429
 *                 replaceDict[s[1]] = str(len(replaceDict))
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 *                 srcElements.append(newSymbol)             # <<<<<<<<<<<<<<
 *                 if s[1] != newSymbol[1]:
 *                     changed = True
 */
      __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_srcElements, __pyx_v_newSymbol); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 429, __pyx_L1_error)

      /* "lpu/smt/trans_models/records.pyx":430
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 *                 srcElements.append(newSymbol)
 *                 if s[1] != newSymbol[1]:             # <<<<<<<<<<<<<<
 *                     changed = True
 *             else:
 */
      __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_GetItemInt(__pyx_v_newSymbol, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_2 = PyObject_RichCompare(__pyx_t_7, __pyx_t_8, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 430, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 430, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_5) {

        /* "lpu/smt/trans_models/records.pyx":431
 *                 srcElements.append(newSymbol)
 *                 if s[1] != newSymbol[1]:
 *                     changed = True             # <<<<<<<<<<<<<<
 *             else:
 *                 # as-is
 */
        __Pyx_INCREF(Py_True);
        __Pyx_DECREF_SET(__pyx_v_changed, Py_True);

        /* "lpu/smt/trans_models/records.pyx":430
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 *                 srcElements.append(newSymbol)
 *                 if s[1] != newSymbol[1]:             # <<<<<<<<<<<<<<
 *                     changed = True
 *             else:
 */
      }

      /* "lpu/smt/trans_models/records.pyx":426
 *         cdef object changed = False
 *         for s in src.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():             # <<<<<<<<<<<<<<
 *                 replaceDict[s[1]] = str(len(replaceDict))
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 */
      goto __pyx_L5;
    }

    /* "lpu/smt/trans_models/records.pyx":434
 *             else:
 *                 # as-is
 *                 srcElements.append(s)             # <<<<<<<<<<<<<<
 *         if not changed:
 *             return (src, trg)
 */
    /*else*/ {
      __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_srcElements, __pyx_v_s); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 434, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "lpu/smt/trans_models/records.pyx":425
 *         cdef str newSymbol
 *         cdef object changed = False
 *         for s in src.split(' '):             # <<<<<<<<<<<<<<
 *             if s[0] == 'x' and s[1].isdigit():
 *                 replaceDict[s[1]] = str(len(replaceDict))
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":435
 *                 # as-is
 *                 srcElements.append(s)
 *         if not changed:             # <<<<<<<<<<<<<<
 *             return (src, trg)
 *         for s in trg.split(' '):
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_changed); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 435, __pyx_L1_error)
  __pyx_t_6 = ((!__pyx_t_5) != 0);
  if (__pyx_t_6) {

    /* "lpu/smt/trans_models/records.pyx":436
 *                 srcElements.append(s)
 *         if not changed:
 *             return (src, trg)             # <<<<<<<<<<<<<<
 *         for s in trg.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_src);
    __Pyx_GIVEREF(__pyx_v_src);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_src);
    __Pyx_INCREF(__pyx_v_trg);
    __Pyx_GIVEREF(__pyx_v_trg);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_trg);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "lpu/smt/trans_models/records.pyx":435
 *                 # as-is
 *                 srcElements.append(s)
 *         if not changed:             # <<<<<<<<<<<<<<
 *             return (src, trg)
 *         for s in trg.split(' '):
 */
  }

  /* "lpu/smt/trans_models/records.pyx":437
 *         if not changed:
 *             return (src, trg)
 *         for s in trg.split(' '):             # <<<<<<<<<<<<<<
 *             if s[0] == 'x' and s[1].isdigit():
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_trg, __pyx_n_s_split); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 437, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 437, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 437, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 437, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "lpu/smt/trans_models/records.pyx":438
 *             return (src, trg)
 *         for s in trg.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():             # <<<<<<<<<<<<<<
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 *                 trgElements.append(newSymbol)
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_s, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L13_bool_binop_done;
    }
    __pyx_t_8 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_isdigit); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (__pyx_t_8) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __pyx_t_5;
    __pyx_L13_bool_binop_done:;
    if (__pyx_t_6) {

      /* "lpu/smt/trans_models/records.pyx":439
 *         for s in trg.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]             # <<<<<<<<<<<<<<
 *                 trgElements.append(newSymbol)
 *             else:
 */
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_replaceDict, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyNumber_Add(__pyx_n_s_x, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetSlice(__pyx_v_s, 2, 0, NULL, NULL, &__pyx_slice__28, 1, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyNumber_Add(__pyx_t_2, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (!(likely(PyString_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_8)->tp_name), 0))) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_newSymbol, ((PyObject*)__pyx_t_8));
      __pyx_t_8 = 0;

      /* "lpu/smt/trans_models/records.pyx":440
 *             if s[0] == 'x' and s[1].isdigit():
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 *                 trgElements.append(newSymbol)             # <<<<<<<<<<<<<<
 *             else:
 *                 # as-is
 */
      __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_trgElements, __pyx_v_newSymbol); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 440, __pyx_L1_error)

      /* "lpu/smt/trans_models/records.pyx":438
 *             return (src, trg)
 *         for s in trg.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():             # <<<<<<<<<<<<<<
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 *                 trgElements.append(newSymbol)
 */
      goto __pyx_L12;
    }

    /* "lpu/smt/trans_models/records.pyx":443
 *             else:
 *                 # as-is
 *                 trgElements.append(s)             # <<<<<<<<<<<<<<
 *         return (str.join(' ',srcElements), str.join(' ',trgElements))
 * 
 */
    /*else*/ {
      __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_trgElements, __pyx_v_s); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 443, __pyx_L1_error)
    }
    __pyx_L12:;

    /* "lpu/smt/trans_models/records.pyx":437
 *         if not changed:
 *             return (src, trg)
 *         for s in trg.split(' '):             # <<<<<<<<<<<<<<
 *             if s[0] == 'x' and s[1].isdigit():
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":444
 *                 # as-is
 *                 trgElements.append(s)
 *         return (str.join(' ',srcElements), str.join(' ',trgElements))             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s__4, __pyx_v_srcElements); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyString_Join(__pyx_kp_s__4, __pyx_v_trgElements); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_8);
  __pyx_t_1 = 0;
  __pyx_t_8 = 0;
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":419
 * 
 *     @staticmethod
 *     def fixOrderStrings(str src, str trg):             # <<<<<<<<<<<<<<
 *         cdef list srcElements = []
 *         cdef list trgElements = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.fixOrderStrings", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_srcElements);
  __Pyx_XDECREF(__pyx_v_trgElements);
  __Pyx_XDECREF(__pyx_v_replaceDict);
  __Pyx_XDECREF(__pyx_v_newSymbol);
  __Pyx_XDECREF(__pyx_v_changed);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":447
 * 
 *     @staticmethod
 *     def getSymbols(rule, hiero=False):             # <<<<<<<<<<<<<<
 *         symbols = []
 *         for s in rule.split(' '):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_15getSymbols(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_14TravatarRecord_15getSymbols = {"getSymbols", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_15getSymbols, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_15getSymbols(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_rule = 0;
  PyObject *__pyx_v_hiero = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getSymbols (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_rule,&__pyx_n_s_hiero,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_rule)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_hiero);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getSymbols") < 0)) __PYX_ERR(0, 447, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_rule = values[0];
    __pyx_v_hiero = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getSymbols", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 447, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.getSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_14getSymbols(__pyx_v_rule, __pyx_v_hiero);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_14getSymbols(PyObject *__pyx_v_rule, PyObject *__pyx_v_hiero) {
  PyObject *__pyx_v_symbols = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("getSymbols", 0);

  /* "lpu/smt/trans_models/records.pyx":448
 *     @staticmethod
 *     def getSymbols(rule, hiero=False):
 *         symbols = []             # <<<<<<<<<<<<<<
 *         for s in rule.split(' '):
 *           if len(s) < 2:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_symbols = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":449
 *     def getSymbols(rule, hiero=False):
 *         symbols = []
 *         for s in rule.split(' '):             # <<<<<<<<<<<<<<
 *           if len(s) < 2:
 *             if s == "@":
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_n_s_split); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 449, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 449, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 449, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 449, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "lpu/smt/trans_models/records.pyx":450
 *         symbols = []
 *         for s in rule.split(' '):
 *           if len(s) < 2:             # <<<<<<<<<<<<<<
 *             if s == "@":
 *               break
 */
    __pyx_t_5 = PyObject_Length(__pyx_v_s); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 450, __pyx_L1_error)
    __pyx_t_6 = ((__pyx_t_5 < 2) != 0);
    if (__pyx_t_6) {

      /* "lpu/smt/trans_models/records.pyx":451
 *         for s in rule.split(' '):
 *           if len(s) < 2:
 *             if s == "@":             # <<<<<<<<<<<<<<
 *               break
 *           elif s[0] == '"' and s[-1] == '"':
 */
      __pyx_t_6 = (__Pyx_PyString_Equals(__pyx_v_s, __pyx_kp_s__20, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 451, __pyx_L1_error)
      if (__pyx_t_6) {

        /* "lpu/smt/trans_models/records.pyx":452
 *           if len(s) < 2:
 *             if s == "@":
 *               break             # <<<<<<<<<<<<<<
 *           elif s[0] == '"' and s[-1] == '"':
 *             symbols.append(s[1:-1])
 */
        goto __pyx_L4_break;

        /* "lpu/smt/trans_models/records.pyx":451
 *         for s in rule.split(' '):
 *           if len(s) < 2:
 *             if s == "@":             # <<<<<<<<<<<<<<
 *               break
 *           elif s[0] == '"' and s[-1] == '"':
 */
      }

      /* "lpu/smt/trans_models/records.pyx":450
 *         symbols = []
 *         for s in rule.split(' '):
 *           if len(s) < 2:             # <<<<<<<<<<<<<<
 *             if s == "@":
 *               break
 */
      goto __pyx_L5;
    }

    /* "lpu/smt/trans_models/records.pyx":453
 *             if s == "@":
 *               break
 *           elif s[0] == '"' and s[-1] == '"':             # <<<<<<<<<<<<<<
 *             symbols.append(s[1:-1])
 *           elif s[0] == 'x' and s[1].isdigit():
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_s, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_kp_s__30, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_s, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_kp_s__30, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_6) {

      /* "lpu/smt/trans_models/records.pyx":454
 *               break
 *           elif s[0] == '"' and s[-1] == '"':
 *             symbols.append(s[1:-1])             # <<<<<<<<<<<<<<
 *           elif s[0] == 'x' and s[1].isdigit():
 *             if len(s) > 3 and not hiero:
 */
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_s, 1, -1L, NULL, NULL, &__pyx_slice__31, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 454, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_symbols, __pyx_t_2); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 454, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "lpu/smt/trans_models/records.pyx":453
 *             if s == "@":
 *               break
 *           elif s[0] == '"' and s[-1] == '"':             # <<<<<<<<<<<<<<
 *             symbols.append(s[1:-1])
 *           elif s[0] == 'x' and s[1].isdigit():
 */
      goto __pyx_L5;
    }

    /* "lpu/smt/trans_models/records.pyx":455
 *           elif s[0] == '"' and s[-1] == '"':
 *             symbols.append(s[1:-1])
 *           elif s[0] == 'x' and s[1].isdigit():             # <<<<<<<<<<<<<<
 *             if len(s) > 3 and not hiero:
 *               symbols.append('[%s]' % s[3:])
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_s, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_9 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_isdigit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    if (__pyx_t_9) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 455, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 455, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __pyx_t_7;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_6) {

      /* "lpu/smt/trans_models/records.pyx":456
 *             symbols.append(s[1:-1])
 *           elif s[0] == 'x' and s[1].isdigit():
 *             if len(s) > 3 and not hiero:             # <<<<<<<<<<<<<<
 *               symbols.append('[%s]' % s[3:])
 *             else:
 */
      __pyx_t_5 = PyObject_Length(__pyx_v_s); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 456, __pyx_L1_error)
      __pyx_t_7 = ((__pyx_t_5 > 3) != 0);
      if (__pyx_t_7) {
      } else {
        __pyx_t_6 = __pyx_t_7;
        goto __pyx_L12_bool_binop_done;
      }
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_hiero); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 456, __pyx_L1_error)
      __pyx_t_11 = ((!__pyx_t_7) != 0);
      __pyx_t_6 = __pyx_t_11;
      __pyx_L12_bool_binop_done:;
      if (__pyx_t_6) {

        /* "lpu/smt/trans_models/records.pyx":457
 *           elif s[0] == 'x' and s[1].isdigit():
 *             if len(s) > 3 and not hiero:
 *               symbols.append('[%s]' % s[3:])             # <<<<<<<<<<<<<<
 *             else:
 *               symbols.append('[X]')
 */
        __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_s, 3, 0, NULL, NULL, &__pyx_slice__32, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 457, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_10 = __Pyx_PyString_Format(__pyx_kp_s_s_2, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 457, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_symbols, __pyx_t_10); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 457, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "lpu/smt/trans_models/records.pyx":456
 *             symbols.append(s[1:-1])
 *           elif s[0] == 'x' and s[1].isdigit():
 *             if len(s) > 3 and not hiero:             # <<<<<<<<<<<<<<
 *               symbols.append('[%s]' % s[3:])
 *             else:
 */
        goto __pyx_L11;
      }

      /* "lpu/smt/trans_models/records.pyx":459
 *               symbols.append('[%s]' % s[3:])
 *             else:
 *               symbols.append('[X]')             # <<<<<<<<<<<<<<
 *         return symbols
 * 
 */
      /*else*/ {
        __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_symbols, __pyx_kp_s_X_3); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 459, __pyx_L1_error)
      }
      __pyx_L11:;

      /* "lpu/smt/trans_models/records.pyx":455
 *           elif s[0] == '"' and s[-1] == '"':
 *             symbols.append(s[1:-1])
 *           elif s[0] == 'x' and s[1].isdigit():             # <<<<<<<<<<<<<<
 *             if len(s) > 3 and not hiero:
 *               symbols.append('[%s]' % s[3:])
 */
    }
    __pyx_L5:;

    /* "lpu/smt/trans_models/records.pyx":449
 *     def getSymbols(rule, hiero=False):
 *         symbols = []
 *         for s in rule.split(' '):             # <<<<<<<<<<<<<<
 *           if len(s) < 2:
 *             if s == "@":
 */
  }
  __pyx_L4_break:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":460
 *             else:
 *               symbols.append('[X]')
 *         return symbols             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_symbols);
  __pyx_r = __pyx_v_symbols;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":447
 * 
 *     @staticmethod
 *     def getSymbols(rule, hiero=False):             # <<<<<<<<<<<<<<
 *         symbols = []
 *         for s in rule.split(' '):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.getSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_symbols);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":463
 * 
 *     @staticmethod
 *     def getTerms(symbols):             # <<<<<<<<<<<<<<
 *       terms = []
 *       if type(symbols) == str:
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_17getTerms(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_14TravatarRecord_17getTerms = {"getTerms", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_17getTerms, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_17getTerms(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_symbols = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getTerms (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_symbols,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_symbols)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getTerms") < 0)) __PYX_ERR(0, 463, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_symbols = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getTerms", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 463, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.getTerms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_16getTerms(__pyx_v_symbols);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_16getTerms(PyObject *__pyx_v_symbols) {
  PyObject *__pyx_v_terms = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("getTerms", 0);
  __Pyx_INCREF(__pyx_v_symbols);

  /* "lpu/smt/trans_models/records.pyx":464
 *     @staticmethod
 *     def getTerms(symbols):
 *       terms = []             # <<<<<<<<<<<<<<
 *       if type(symbols) == str:
 *           symbols = symbols.split(' ')
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_terms = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":465
 *     def getTerms(symbols):
 *       terms = []
 *       if type(symbols) == str:             # <<<<<<<<<<<<<<
 *           symbols = symbols.split(' ')
 *       for s in symbols:
 */
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_symbols)), ((PyObject *)(&PyString_Type)), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "lpu/smt/trans_models/records.pyx":466
 *       terms = []
 *       if type(symbols) == str:
 *           symbols = symbols.split(' ')             # <<<<<<<<<<<<<<
 *       for s in symbols:
 *         if len(s) < 2:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_symbols, __pyx_n_s_split); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 466, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 466, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_symbols, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "lpu/smt/trans_models/records.pyx":465
 *     def getTerms(symbols):
 *       terms = []
 *       if type(symbols) == str:             # <<<<<<<<<<<<<<
 *           symbols = symbols.split(' ')
 *       for s in symbols:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":467
 *       if type(symbols) == str:
 *           symbols = symbols.split(' ')
 *       for s in symbols:             # <<<<<<<<<<<<<<
 *         if len(s) < 2:
 *           if s == "@":
 */
  if (likely(PyList_CheckExact(__pyx_v_symbols)) || PyTuple_CheckExact(__pyx_v_symbols)) {
    __pyx_t_3 = __pyx_v_symbols; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_symbols); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 467, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 467, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 467, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 467, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":468
 *           symbols = symbols.split(' ')
 *       for s in symbols:
 *         if len(s) < 2:             # <<<<<<<<<<<<<<
 *           if s == "@":
 *             break
 */
    __pyx_t_6 = PyObject_Length(__pyx_v_s); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 468, __pyx_L1_error)
    __pyx_t_2 = ((__pyx_t_6 < 2) != 0);
    if (__pyx_t_2) {

      /* "lpu/smt/trans_models/records.pyx":469
 *       for s in symbols:
 *         if len(s) < 2:
 *           if s == "@":             # <<<<<<<<<<<<<<
 *             break
 *         elif s[0] == '"' and s[-1] == '"':
 */
      __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_s, __pyx_kp_s__20, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 469, __pyx_L1_error)
      if (__pyx_t_2) {

        /* "lpu/smt/trans_models/records.pyx":470
 *         if len(s) < 2:
 *           if s == "@":
 *             break             # <<<<<<<<<<<<<<
 *         elif s[0] == '"' and s[-1] == '"':
 *           terms.append(s[1:-1])
 */
        goto __pyx_L5_break;

        /* "lpu/smt/trans_models/records.pyx":469
 *       for s in symbols:
 *         if len(s) < 2:
 *           if s == "@":             # <<<<<<<<<<<<<<
 *             break
 *         elif s[0] == '"' and s[-1] == '"':
 */
      }

      /* "lpu/smt/trans_models/records.pyx":468
 *           symbols = symbols.split(' ')
 *       for s in symbols:
 *         if len(s) < 2:             # <<<<<<<<<<<<<<
 *           if s == "@":
 *             break
 */
      goto __pyx_L6;
    }

    /* "lpu/smt/trans_models/records.pyx":471
 *           if s == "@":
 *             break
 *         elif s[0] == '"' and s[-1] == '"':             # <<<<<<<<<<<<<<
 *           terms.append(s[1:-1])
 *       return terms
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_s, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_kp_s__30, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 471, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_7) {
    } else {
      __pyx_t_2 = __pyx_t_7;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_s, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_kp_s__30, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 471, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_t_7;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_2) {

      /* "lpu/smt/trans_models/records.pyx":472
 *             break
 *         elif s[0] == '"' and s[-1] == '"':
 *           terms.append(s[1:-1])             # <<<<<<<<<<<<<<
 *       return terms
 * 
 */
      __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_s, 1, -1L, NULL, NULL, &__pyx_slice__34, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_terms, __pyx_t_1); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":471
 *           if s == "@":
 *             break
 *         elif s[0] == '"' and s[-1] == '"':             # <<<<<<<<<<<<<<
 *           terms.append(s[1:-1])
 *       return terms
 */
    }
    __pyx_L6:;

    /* "lpu/smt/trans_models/records.pyx":467
 *       if type(symbols) == str:
 *           symbols = symbols.split(' ')
 *       for s in symbols:             # <<<<<<<<<<<<<<
 *         if len(s) < 2:
 *           if s == "@":
 */
  }
  __pyx_L5_break:;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "lpu/smt/trans_models/records.pyx":473
 *         elif s[0] == '"' and s[-1] == '"':
 *           terms.append(s[1:-1])
 *       return terms             # <<<<<<<<<<<<<<
 * 
 * def getFlattenSymbols(symbols, grammar = 'scfg'):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_terms);
  __pyx_r = __pyx_v_terms;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":463
 * 
 *     @staticmethod
 *     def getTerms(symbols):             # <<<<<<<<<<<<<<
 *       terms = []
 *       if type(symbols) == str:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.getTerms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_terms);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_symbols);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_18__reduce_cython__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_20__setstate_cython__(((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14TravatarRecord_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.TravatarRecord.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":475
 *       return terms
 * 
 * def getFlattenSymbols(symbols, grammar = 'scfg'):             # <<<<<<<<<<<<<<
 *     if isTree(symbols):
 *         # strSymbols is tree (S-expression)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_15getFlattenSymbols(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_15getFlattenSymbols = {"getFlattenSymbols", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_15getFlattenSymbols, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_15getFlattenSymbols(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_symbols = 0;
  PyObject *__pyx_v_grammar = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getFlattenSymbols (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_symbols,&__pyx_n_s_grammar,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_n_s_scfg);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_symbols)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_grammar);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getFlattenSymbols") < 0)) __PYX_ERR(0, 475, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_symbols = values[0];
    __pyx_v_grammar = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getFlattenSymbols", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 475, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getFlattenSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_14getFlattenSymbols(__pyx_self, __pyx_v_symbols, __pyx_v_grammar);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_14getFlattenSymbols(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_symbols, PyObject *__pyx_v_grammar) {
  PyObject *__pyx_v_newSymbols = NULL;
  PyObject *__pyx_v_tag = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_fields = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("getFlattenSymbols", 0);
  __Pyx_INCREF(__pyx_v_symbols);

  /* "lpu/smt/trans_models/records.pyx":476
 * 
 * def getFlattenSymbols(symbols, grammar = 'scfg'):
 *     if isTree(symbols):             # <<<<<<<<<<<<<<
 *         # strSymbols is tree (S-expression)
 *         newSymbols = []
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_isTree); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_symbols); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_symbols};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_symbols};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_symbols);
      __Pyx_GIVEREF(__pyx_v_symbols);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_symbols);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":478
 *     if isTree(symbols):
 *         # strSymbols is tree (S-expression)
 *         newSymbols = []             # <<<<<<<<<<<<<<
 *         tag = symbols[0]
 *         for s in symbols:
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 478, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_newSymbols = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":479
 *         # strSymbols is tree (S-expression)
 *         newSymbols = []
 *         tag = symbols[0]             # <<<<<<<<<<<<<<
 *         for s in symbols:
 *             if s[0:1] == '"':
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_symbols, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_tag = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":480
 *         newSymbols = []
 *         tag = symbols[0]
 *         for s in symbols:             # <<<<<<<<<<<<<<
 *             if s[0:1] == '"':
 *                 newSymbols.append(s)
 */
    if (likely(PyList_CheckExact(__pyx_v_symbols)) || PyTuple_CheckExact(__pyx_v_symbols)) {
      __pyx_t_1 = __pyx_v_symbols; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_symbols); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 480, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 480, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 480, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 480, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 480, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_7(__pyx_t_1);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 480, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "lpu/smt/trans_models/records.pyx":481
 *         tag = symbols[0]
 *         for s in symbols:
 *             if s[0:1] == '"':             # <<<<<<<<<<<<<<
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":
 */
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_s, 0, 1, NULL, NULL, &__pyx_slice__37, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_kp_s__30, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_5) {

        /* "lpu/smt/trans_models/records.pyx":482
 *         for s in symbols:
 *             if s[0:1] == '"':
 *                 newSymbols.append(s)             # <<<<<<<<<<<<<<
 *             elif s[0:1] == "x":
 *                 if grammar == 'scfg':
 */
        __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_newSymbols, __pyx_v_s); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 482, __pyx_L1_error)

        /* "lpu/smt/trans_models/records.pyx":481
 *         tag = symbols[0]
 *         for s in symbols:
 *             if s[0:1] == '"':             # <<<<<<<<<<<<<<
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":
 */
        goto __pyx_L6;
      }

      /* "lpu/smt/trans_models/records.pyx":483
 *             if s[0:1] == '"':
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":             # <<<<<<<<<<<<<<
 *                 if grammar == 'scfg':
 *                     newSymbols.append(s)
 */
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_s, 0, 1, NULL, NULL, &__pyx_slice__38, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 483, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 483, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_5) {

        /* "lpu/smt/trans_models/records.pyx":484
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":
 *                 if grammar == 'scfg':             # <<<<<<<<<<<<<<
 *                     newSymbols.append(s)
 *                 else:
 */
        __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_grammar, __pyx_n_s_scfg, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 484, __pyx_L1_error)
        if (__pyx_t_5) {

          /* "lpu/smt/trans_models/records.pyx":485
 *             elif s[0:1] == "x":
 *                 if grammar == 'scfg':
 *                     newSymbols.append(s)             # <<<<<<<<<<<<<<
 *                 else:
 *                     fields = s.split(':')
 */
          __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_newSymbols, __pyx_v_s); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 485, __pyx_L1_error)

          /* "lpu/smt/trans_models/records.pyx":484
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":
 *                 if grammar == 'scfg':             # <<<<<<<<<<<<<<
 *                     newSymbols.append(s)
 *                 else:
 */
          goto __pyx_L7;
        }

        /* "lpu/smt/trans_models/records.pyx":487
 *                     newSymbols.append(s)
 *                 else:
 *                     fields = s.split(':')             # <<<<<<<<<<<<<<
 *                     newSymbols.append(fields[0]+':X')
 *         newSymbols.append("@")
 */
        /*else*/ {
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_n_s_split); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__40, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 487, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF_SET(__pyx_v_fields, __pyx_t_4);
          __pyx_t_4 = 0;

          /* "lpu/smt/trans_models/records.pyx":488
 *                 else:
 *                     fields = s.split(':')
 *                     newSymbols.append(fields[0]+':X')             # <<<<<<<<<<<<<<
 *         newSymbols.append("@")
 *         if grammar == 'hiero':
 */
          __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_fields, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 488, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_2 = PyNumber_Add(__pyx_t_4, __pyx_kp_s_X); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 488, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_newSymbols, __pyx_t_2); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 488, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }
        __pyx_L7:;

        /* "lpu/smt/trans_models/records.pyx":483
 *             if s[0:1] == '"':
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":             # <<<<<<<<<<<<<<
 *                 if grammar == 'scfg':
 *                     newSymbols.append(s)
 */
      }
      __pyx_L6:;

      /* "lpu/smt/trans_models/records.pyx":480
 *         newSymbols = []
 *         tag = symbols[0]
 *         for s in symbols:             # <<<<<<<<<<<<<<
 *             if s[0:1] == '"':
 *                 newSymbols.append(s)
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":489
 *                     fields = s.split(':')
 *                     newSymbols.append(fields[0]+':X')
 *         newSymbols.append("@")             # <<<<<<<<<<<<<<
 *         if grammar == 'hiero':
 *             newSymbols.append('X')
 */
    __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_newSymbols, __pyx_kp_s__20); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 489, __pyx_L1_error)

    /* "lpu/smt/trans_models/records.pyx":490
 *                     newSymbols.append(fields[0]+':X')
 *         newSymbols.append("@")
 *         if grammar == 'hiero':             # <<<<<<<<<<<<<<
 *             newSymbols.append('X')
 *         else:
 */
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_grammar, __pyx_n_s_hiero, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 490, __pyx_L1_error)
    if (__pyx_t_5) {

      /* "lpu/smt/trans_models/records.pyx":491
 *         newSymbols.append("@")
 *         if grammar == 'hiero':
 *             newSymbols.append('X')             # <<<<<<<<<<<<<<
 *         else:
 *             newSymbols.append(tag)
 */
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_newSymbols, __pyx_n_s_X_2); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 491, __pyx_L1_error)

      /* "lpu/smt/trans_models/records.pyx":490
 *                     newSymbols.append(fields[0]+':X')
 *         newSymbols.append("@")
 *         if grammar == 'hiero':             # <<<<<<<<<<<<<<
 *             newSymbols.append('X')
 *         else:
 */
      goto __pyx_L8;
    }

    /* "lpu/smt/trans_models/records.pyx":493
 *             newSymbols.append('X')
 *         else:
 *             newSymbols.append(tag)             # <<<<<<<<<<<<<<
 *         symbols = newSymbols
 *     return symbols
 */
    /*else*/ {
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_newSymbols, __pyx_v_tag); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 493, __pyx_L1_error)
    }
    __pyx_L8:;

    /* "lpu/smt/trans_models/records.pyx":494
 *         else:
 *             newSymbols.append(tag)
 *         symbols = newSymbols             # <<<<<<<<<<<<<<
 *     return symbols
 * 
 */
    __Pyx_INCREF(__pyx_v_newSymbols);
    __Pyx_DECREF_SET(__pyx_v_symbols, __pyx_v_newSymbols);

    /* "lpu/smt/trans_models/records.pyx":476
 * 
 * def getFlattenSymbols(symbols, grammar = 'scfg'):
 *     if isTree(symbols):             # <<<<<<<<<<<<<<
 *         # strSymbols is tree (S-expression)
 *         newSymbols = []
 */
  }

  /* "lpu/smt/trans_models/records.pyx":495
 *             newSymbols.append(tag)
 *         symbols = newSymbols
 *     return symbols             # <<<<<<<<<<<<<<
 * 
 * cpdef str getStrTravatarFeatures(dict dict_features):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_symbols);
  __pyx_r = __pyx_v_symbols;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":475
 *       return terms
 * 
 * def getFlattenSymbols(symbols, grammar = 'scfg'):             # <<<<<<<<<<<<<<
 *     if isTree(symbols):
 *         # strSymbols is tree (S-expression)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getFlattenSymbols", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_newSymbols);
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_symbols);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":497
 *     return symbols
 * 
 * cpdef str getStrTravatarFeatures(dict dict_features):             # <<<<<<<<<<<<<<
 *     '''convert features dict into string in format of 'key=val' list'''
 *     cdef list featureList = []
 */

static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_17getStrTravatarFeatures(PyObject *__pyx_self, PyObject *__pyx_v_dict_features); /*proto*/
static PyObject *__pyx_f_3lpu_3smt_12trans_models_7records_getStrTravatarFeatures(PyObject *__pyx_v_dict_features, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_featureList = 0;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_val = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  __Pyx_RefNannySetupContext("getStrTravatarFeatures", 0);

  /* "lpu/smt/trans_models/records.pyx":499
 * cpdef str getStrTravatarFeatures(dict dict_features):
 *     '''convert features dict into string in format of 'key=val' list'''
 *     cdef list featureList = []             # <<<<<<<<<<<<<<
 *     cdef str key
 *     cdef object val
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_featureList = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":502
 *     cdef str key
 *     cdef object val
 *     for key, val in dict_features.items():             # <<<<<<<<<<<<<<
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:
 *             try:
 */
  if (unlikely(__pyx_v_dict_features == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%s'", "items");
    __PYX_ERR(0, 502, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_Items(__pyx_v_dict_features); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 502, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 502, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 502, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 502, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 502, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 502, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 502, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 502, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 502, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 502, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    if (!(likely(PyString_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "lpu/smt/trans_models/records.pyx":503
 *     cdef object val
 *     for key, val in dict_features.items():
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:             # <<<<<<<<<<<<<<
 *             try:
 *                 #val = math.log(val)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_key, __pyx_n_s_find); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__41, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_6, __pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_key, __pyx_n_s_find); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__42, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_6, __pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __pyx_t_10;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_9) {

      /* "lpu/smt/trans_models/records.pyx":504
 *     for key, val in dict_features.items():
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:
 *             try:             # <<<<<<<<<<<<<<
 *                 #val = math.log(val)
 *                 val = round(math.log(val), 6)
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_13);
        /*try:*/ {

          /* "lpu/smt/trans_models/records.pyx":506
 *             try:
 *                 #val = math.log(val)
 *                 val = round(math.log(val), 6)             # <<<<<<<<<<<<<<
 *             except:
 *                 logging.warn( (key,val) )
 */
          __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_math); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 506, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_log); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 506, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_6 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_6)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_6);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          if (!__pyx_t_6) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_1);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_val};
              __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L10_error)
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_val};
              __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L10_error)
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            {
              __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 506, __pyx_L10_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
              __Pyx_INCREF(__pyx_v_val);
              __Pyx_GIVEREF(__pyx_v_val);
              PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_val);
              __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L10_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 506, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
          __Pyx_INCREF(__pyx_int_6);
          __Pyx_GIVEREF(__pyx_int_6);
          PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_6);
          __pyx_t_1 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF_SET(__pyx_v_val, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "lpu/smt/trans_models/records.pyx":504
 *     for key, val in dict_features.items():
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:
 *             try:             # <<<<<<<<<<<<<<
 *                 #val = math.log(val)
 *                 val = round(math.log(val), 6)
 */
        }
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L17_try_end;
        __pyx_L10_error:;
        __Pyx_PyThreadState_assign
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "lpu/smt/trans_models/records.pyx":507
 *                 #val = math.log(val)
 *                 val = round(math.log(val), 6)
 *             except:             # <<<<<<<<<<<<<<
 *                 logging.warn( (key,val) )
 *         featureList.append( "%s=%s" % (key, val) )
 */
        /*except:*/ {
          __Pyx_AddTraceback("lpu.smt.trans_models.records.getStrTravatarFeatures", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_5, &__pyx_t_7) < 0) __PYX_ERR(0, 507, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GOTREF(__pyx_t_7);

          /* "lpu/smt/trans_models/records.pyx":508
 *                 val = round(math.log(val), 6)
 *             except:
 *                 logging.warn( (key,val) )             # <<<<<<<<<<<<<<
 *         featureList.append( "%s=%s" % (key, val) )
 *     return str.join(' ', sorted(featureList))
 */
          __pyx_t_14 = __Pyx_GetModuleGlobalName(__pyx_n_s_logging); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 508, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_warn); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 508, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 508, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_INCREF(__pyx_v_key);
          __Pyx_GIVEREF(__pyx_v_key);
          PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_v_key);
          __Pyx_INCREF(__pyx_v_val);
          __Pyx_GIVEREF(__pyx_v_val);
          PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_v_val);
          __pyx_t_16 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_15))) {
            __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_15);
            if (likely(__pyx_t_16)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
              __Pyx_INCREF(__pyx_t_16);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_15, function);
            }
          }
          if (!__pyx_t_16) {
            __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_15, __pyx_t_14); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 508, __pyx_L12_except_error)
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_GOTREF(__pyx_t_6);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_15)) {
              PyObject *__pyx_temp[2] = {__pyx_t_16, __pyx_t_14};
              __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 508, __pyx_L12_except_error)
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
              PyObject *__pyx_temp[2] = {__pyx_t_16, __pyx_t_14};
              __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 508, __pyx_L12_except_error)
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            } else
            #endif
            {
              __pyx_t_17 = PyTuple_New(1+1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 508, __pyx_L12_except_error)
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_t_16); __pyx_t_16 = NULL;
              __Pyx_GIVEREF(__pyx_t_14);
              PyTuple_SET_ITEM(__pyx_t_17, 0+1, __pyx_t_14);
              __pyx_t_14 = 0;
              __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_17, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 508, __pyx_L12_except_error)
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          goto __pyx_L11_exception_handled;
        }
        __pyx_L12_except_error:;

        /* "lpu/smt/trans_models/records.pyx":504
 *     for key, val in dict_features.items():
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:
 *             try:             # <<<<<<<<<<<<<<
 *                 #val = math.log(val)
 *                 val = round(math.log(val), 6)
 */
        __Pyx_PyThreadState_assign
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
        goto __pyx_L1_error;
        __pyx_L11_exception_handled:;
        __Pyx_PyThreadState_assign
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
        __pyx_L17_try_end:;
      }

      /* "lpu/smt/trans_models/records.pyx":503
 *     cdef object val
 *     for key, val in dict_features.items():
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:             # <<<<<<<<<<<<<<
 *             try:
 *                 #val = math.log(val)
 */
    }

    /* "lpu/smt/trans_models/records.pyx":509
 *             except:
 *                 logging.warn( (key,val) )
 *         featureList.append( "%s=%s" % (key, val) )             # <<<<<<<<<<<<<<
 *     return str.join(' ', sorted(featureList))
 * 
 */
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_key);
    __Pyx_INCREF(__pyx_v_val);
    __Pyx_GIVEREF(__pyx_v_val);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_val);
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_s_s, __pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_18 = __Pyx_PyList_Append(__pyx_v_featureList, __pyx_t_5); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 509, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "lpu/smt/trans_models/records.pyx":502
 *     cdef str key
 *     cdef object val
 *     for key, val in dict_features.items():             # <<<<<<<<<<<<<<
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:
 *             try:
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":510
 *                 logging.warn( (key,val) )
 *         featureList.append( "%s=%s" % (key, val) )
 *     return str.join(' ', sorted(featureList))             # <<<<<<<<<<<<<<
 * 
 * def getTravatarFeatures(field):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = PySequence_List(__pyx_v_featureList); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_18 = PyList_Sort(__pyx_t_2); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 510, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyString_Join(__pyx_kp_s__4, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(0, 510, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":497
 *     return symbols
 * 
 * cpdef str getStrTravatarFeatures(dict dict_features):             # <<<<<<<<<<<<<<
 *     '''convert features dict into string in format of 'key=val' list'''
 *     cdef list featureList = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getStrTravatarFeatures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_featureList);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_17getStrTravatarFeatures(PyObject *__pyx_self, PyObject *__pyx_v_dict_features); /*proto*/
static char __pyx_doc_3lpu_3smt_12trans_models_7records_16getStrTravatarFeatures[] = "convert features dict into string in format of 'key=val' list";
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_17getStrTravatarFeatures(PyObject *__pyx_self, PyObject *__pyx_v_dict_features) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getStrTravatarFeatures (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dict_features), (&PyDict_Type), 1, "dict_features", 1))) __PYX_ERR(0, 497, __pyx_L1_error)
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_16getStrTravatarFeatures(__pyx_self, ((PyObject*)__pyx_v_dict_features));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_16getStrTravatarFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dict_features) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getStrTravatarFeatures", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_3lpu_3smt_12trans_models_7records_getStrTravatarFeatures(__pyx_v_dict_features, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getStrTravatarFeatures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":512
 *     return str.join(' ', sorted(featureList))
 * 
 * def getTravatarFeatures(field):             # <<<<<<<<<<<<<<
 *   features = {}
 *   for strKeyVal in field.split():
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_19getTravatarFeatures(PyObject *__pyx_self, PyObject *__pyx_v_field); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_19getTravatarFeatures = {"getTravatarFeatures", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_19getTravatarFeatures, METH_O, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_19getTravatarFeatures(PyObject *__pyx_self, PyObject *__pyx_v_field) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getTravatarFeatures (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_18getTravatarFeatures(__pyx_self, ((PyObject *)__pyx_v_field));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_18getTravatarFeatures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_field) {
  PyObject *__pyx_v_features = NULL;
  PyObject *__pyx_v_strKeyVal = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("getTravatarFeatures", 0);

  /* "lpu/smt/trans_models/records.pyx":513
 * 
 * def getTravatarFeatures(field):
 *   features = {}             # <<<<<<<<<<<<<<
 *   for strKeyVal in field.split():
 *     #debug.log(strKeyVal)
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_features = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":514
 * def getTravatarFeatures(field):
 *   features = {}
 *   for strKeyVal in field.split():             # <<<<<<<<<<<<<<
 *     #debug.log(strKeyVal)
 *     (key, val) = strKeyVal.split('=')
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_field, __pyx_n_s_split); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 514, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 514, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 514, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 514, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_strKeyVal, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "lpu/smt/trans_models/records.pyx":516
 *   for strKeyVal in field.split():
 *     #debug.log(strKeyVal)
 *     (key, val) = strKeyVal.split('=')             # <<<<<<<<<<<<<<
 *     val = getNumber(val)
 * #    if key in ['egfl', 'egfp', 'fgel', 'fgep']:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_strKeyVal, __pyx_n_s_split); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__44, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
      PyObject* sequence = __pyx_t_3;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 516, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 516, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 516, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_1 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_1)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_1);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 516, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 516, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_1);
    __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "lpu/smt/trans_models/records.pyx":517
 *     #debug.log(strKeyVal)
 *     (key, val) = strKeyVal.split('=')
 *     val = getNumber(val)             # <<<<<<<<<<<<<<
 * #    if key in ['egfl', 'egfp', 'fgel', 'fgep']:
 * #      val = math.e ** val
 */
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_getNumber); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 517, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_v_val};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_v_val};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 517, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1); __pyx_t_1 = NULL;
        __Pyx_INCREF(__pyx_v_val);
        __Pyx_GIVEREF(__pyx_v_val);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_val);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_val, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "lpu/smt/trans_models/records.pyx":522
 * #    if key[-1] not in ['p', 'w']:
 * #      val = math.e ** val
 *     if len(key) >= 4 :             # <<<<<<<<<<<<<<
 *       val = math.e ** val
 *     features[key] = val
 */
    __pyx_t_9 = PyObject_Length(__pyx_v_key); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 522, __pyx_L1_error)
    __pyx_t_10 = ((__pyx_t_9 >= 4) != 0);
    if (__pyx_t_10) {

      /* "lpu/smt/trans_models/records.pyx":523
 * #      val = math.e ** val
 *     if len(key) >= 4 :
 *       val = math.e ** val             # <<<<<<<<<<<<<<
 *     features[key] = val
 *   return features
 */
      __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_math); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_e); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyNumber_Power(__pyx_t_6, __pyx_v_val, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_val, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "lpu/smt/trans_models/records.pyx":522
 * #    if key[-1] not in ['p', 'w']:
 * #      val = math.e ** val
 *     if len(key) >= 4 :             # <<<<<<<<<<<<<<
 *       val = math.e ** val
 *     features[key] = val
 */
    }

    /* "lpu/smt/trans_models/records.pyx":524
 *     if len(key) >= 4 :
 *       val = math.e ** val
 *     features[key] = val             # <<<<<<<<<<<<<<
 *   return features
 * 
 */
    if (unlikely(PyDict_SetItem(__pyx_v_features, __pyx_v_key, __pyx_v_val) < 0)) __PYX_ERR(0, 524, __pyx_L1_error)

    /* "lpu/smt/trans_models/records.pyx":514
 * def getTravatarFeatures(field):
 *   features = {}
 *   for strKeyVal in field.split():             # <<<<<<<<<<<<<<
 *     #debug.log(strKeyVal)
 *     (key, val) = strKeyVal.split('=')
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":525
 *       val = math.e ** val
 *     features[key] = val
 *   return features             # <<<<<<<<<<<<<<
 * 
 * def isTree(symbols):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_features);
  __pyx_r = __pyx_v_features;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":512
 *     return str.join(' ', sorted(featureList))
 * 
 * def getTravatarFeatures(field):             # <<<<<<<<<<<<<<
 *   features = {}
 *   for strKeyVal in field.split():
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.getTravatarFeatures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_features);
  __Pyx_XDECREF(__pyx_v_strKeyVal);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":527
 *   return features
 * 
 * def isTree(symbols):             # <<<<<<<<<<<<<<
 * #    if len(symbols) >= 4 and symbols[0][0] != '"':
 *     if len(symbols) >= 4 and symbols[1] == '(':
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_21isTree(PyObject *__pyx_self, PyObject *__pyx_v_symbols); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_21isTree = {"isTree", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_21isTree, METH_O, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_21isTree(PyObject *__pyx_self, PyObject *__pyx_v_symbols) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isTree (wrapper)", 0);
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_20isTree(__pyx_self, ((PyObject *)__pyx_v_symbols));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_20isTree(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_symbols) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("isTree", 0);

  /* "lpu/smt/trans_models/records.pyx":529
 * def isTree(symbols):
 * #    if len(symbols) >= 4 and symbols[0][0] != '"':
 *     if len(symbols) >= 4 and symbols[1] == '(':             # <<<<<<<<<<<<<<
 *         return True
 *     else:
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_symbols); if (unlikely(__pyx_t_2 == -1)) __PYX_ERR(0, 529, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 >= 4) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_symbols, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_t_4, __pyx_kp_s__45, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "lpu/smt/trans_models/records.pyx":530
 * #    if len(symbols) >= 4 and symbols[0][0] != '"':
 *     if len(symbols) >= 4 and symbols[1] == '(':
 *         return True             # <<<<<<<<<<<<<<
 *     else:
 *         return False
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "lpu/smt/trans_models/records.pyx":529
 * def isTree(symbols):
 * #    if len(symbols) >= 4 and symbols[0][0] != '"':
 *     if len(symbols) >= 4 and symbols[1] == '(':             # <<<<<<<<<<<<<<
 *         return True
 *     else:
 */
  }

  /* "lpu/smt/trans_models/records.pyx":532
 *         return True
 *     else:
 *         return False             # <<<<<<<<<<<<<<
 * 
 * def syncTags(src_symbols, trgSymbols, sync):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "lpu/smt/trans_models/records.pyx":527
 *   return features
 * 
 * def isTree(symbols):             # <<<<<<<<<<<<<<
 * #    if len(symbols) >= 4 and symbols[0][0] != '"':
 *     if len(symbols) >= 4 and symbols[1] == '(':
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.isTree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "lpu/smt/trans_models/records.pyx":534
 *         return False
 * 
 * def syncTags(src_symbols, trgSymbols, sync):             # <<<<<<<<<<<<<<
 *     if isTree(src_symbols):
 *         # src is tree (s-expression)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_23syncTags(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_3lpu_3smt_12trans_models_7records_23syncTags = {"syncTags", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_23syncTags, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3lpu_3smt_12trans_models_7records_23syncTags(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_src_symbols = 0;
  PyObject *__pyx_v_trgSymbols = 0;
  PyObject *__pyx_v_sync = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("syncTags (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_src_symbols,&__pyx_n_s_trgSymbols,&__pyx_n_s_sync,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_src_symbols)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_trgSymbols)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("syncTags", 1, 3, 3, 1); __PYX_ERR(0, 534, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sync)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("syncTags", 1, 3, 3, 2); __PYX_ERR(0, 534, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "syncTags") < 0)) __PYX_ERR(0, 534, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_src_symbols = values[0];
    __pyx_v_trgSymbols = values[1];
    __pyx_v_sync = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("syncTags", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 534, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("lpu.smt.trans_models.records.syncTags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3lpu_3smt_12trans_models_7records_22syncTags(__pyx_self, __pyx_v_src_symbols, __pyx_v_trgSymbols, __pyx_v_sync);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3lpu_3smt_12trans_models_7records_22syncTags(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_src_symbols, PyObject *__pyx_v_trgSymbols, PyObject *__pyx_v_sync) {
  PyObject *__pyx_v_srcTag = NULL;
  PyObject *__pyx_v_srcNonTerminals = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_index = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  __Pyx_RefNannySetupContext("syncTags", 0);

  /* "lpu/smt/trans_models/records.pyx":535
 * 
 * def syncTags(src_symbols, trgSymbols, sync):
 *     if isTree(src_symbols):             # <<<<<<<<<<<<<<
 *         # src is tree (s-expression)
 *         if len(trgSymbols) >= 2 and trgSymbols[-2] == "@":
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_isTree); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_src_symbols); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_src_symbols};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_src_symbols};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 535, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_src_symbols);
      __Pyx_GIVEREF(__pyx_v_src_symbols);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_src_symbols);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "lpu/smt/trans_models/records.pyx":537
 *     if isTree(src_symbols):
 *         # src is tree (s-expression)
 *         if len(trgSymbols) >= 2 and trgSymbols[-2] == "@":             # <<<<<<<<<<<<<<
 *             # trg is tagged
 *             pass
 */
    __pyx_t_6 = PyObject_Length(__pyx_v_trgSymbols); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 537, __pyx_L1_error)
    __pyx_t_7 = ((__pyx_t_6 >= 2) != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_5 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_trgSymbols, -2L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_kp_s__20, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 537, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __pyx_t_7;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_5) {
      goto __pyx_L4;
    }

    /* "lpu/smt/trans_models/records.pyx":542
 *         else:
 *             # trg is not tagged
 *             srcTag = src_symbols[0]             # <<<<<<<<<<<<<<
 *             srcNonTerminals = []
 *             for s in src_symbols:
 */
    /*else*/ {
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_src_symbols, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_srcTag = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":543
 *             # trg is not tagged
 *             srcTag = src_symbols[0]
 *             srcNonTerminals = []             # <<<<<<<<<<<<<<
 *             for s in src_symbols:
 *                 if s[0:1] == 'x' and s[1:2].isdigit():
 */
      __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_srcNonTerminals = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":544
 *             srcTag = src_symbols[0]
 *             srcNonTerminals = []
 *             for s in src_symbols:             # <<<<<<<<<<<<<<
 *                 if s[0:1] == 'x' and s[1:2].isdigit():
 *                     srcNonTerminals.append(s)
 */
      if (likely(PyList_CheckExact(__pyx_v_src_symbols)) || PyTuple_CheckExact(__pyx_v_src_symbols)) {
        __pyx_t_1 = __pyx_v_src_symbols; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
        __pyx_t_8 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_src_symbols); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 544, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_8 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 544, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_8)) {
          if (likely(PyList_CheckExact(__pyx_t_1))) {
            if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 544, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          } else {
            if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 544, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          }
        } else {
          __pyx_t_2 = __pyx_t_8(__pyx_t_1);
          if (unlikely(!__pyx_t_2)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 544, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "lpu/smt/trans_models/records.pyx":545
 *             srcNonTerminals = []
 *             for s in src_symbols:
 *                 if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 *                     srcNonTerminals.append(s)
 * #                        if len(s) > 3:
 */
        __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_s, 0, 1, NULL, NULL, &__pyx_slice__46, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (__pyx_t_7) {
        } else {
          __pyx_t_5 = __pyx_t_7;
          goto __pyx_L10_bool_binop_done;
        }
        __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_s, 1, 2, NULL, NULL, &__pyx_slice__47, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_isdigit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        if (__pyx_t_4) {
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else {
          __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_5 = __pyx_t_7;
        __pyx_L10_bool_binop_done:;
        if (__pyx_t_5) {

          /* "lpu/smt/trans_models/records.pyx":546
 *             for s in src_symbols:
 *                 if s[0:1] == 'x' and s[1:2].isdigit():
 *                     srcNonTerminals.append(s)             # <<<<<<<<<<<<<<
 * #                        if len(s) > 3:
 * #                            srcNonTerminals.append(s[3:])
 */
          __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_srcNonTerminals, __pyx_v_s); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 546, __pyx_L1_error)

          /* "lpu/smt/trans_models/records.pyx":545
 *             srcNonTerminals = []
 *             for s in src_symbols:
 *                 if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 *                     srcNonTerminals.append(s)
 * #                        if len(s) > 3:
 */
        }

        /* "lpu/smt/trans_models/records.pyx":544
 *             srcTag = src_symbols[0]
 *             srcNonTerminals = []
 *             for s in src_symbols:             # <<<<<<<<<<<<<<
 *                 if s[0:1] == 'x' and s[1:2].isdigit():
 *                     srcNonTerminals.append(s)
 */
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":551
 * #                        else:
 * #                            srcNonTerminals.append('X')
 *             for i, s in enumerate(trgSymbols):             # <<<<<<<<<<<<<<
 * #                    print("TRG SYMBOLS: %s" % s)
 *                 if s[0:1] == 'x' and s[1:2].isdigit():
 */
      __Pyx_INCREF(__pyx_int_0);
      __pyx_t_1 = __pyx_int_0;
      if (likely(PyList_CheckExact(__pyx_v_trgSymbols)) || PyTuple_CheckExact(__pyx_v_trgSymbols)) {
        __pyx_t_2 = __pyx_v_trgSymbols; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
        __pyx_t_8 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_trgSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_8 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 551, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_8)) {
          if (likely(PyList_CheckExact(__pyx_t_2))) {
            if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_3); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
            #else
            __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          } else {
            if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_3); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
            #else
            __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          }
        } else {
          __pyx_t_3 = __pyx_t_8(__pyx_t_2);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 551, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_XDECREF_SET(__pyx_v_s, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
        __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1);
        __pyx_t_1 = __pyx_t_3;
        __pyx_t_3 = 0;

        /* "lpu/smt/trans_models/records.pyx":553
 *             for i, s in enumerate(trgSymbols):
 * #                    print("TRG SYMBOLS: %s" % s)
 *                 if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 * #                        print("NONTERMINAL!")
 *                     if len(s) > 3:
 */
        __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_s, 0, 1, NULL, NULL, &__pyx_slice__48, 1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_t_3, __pyx_n_s_x, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_7) {
        } else {
          __pyx_t_5 = __pyx_t_7;
          goto __pyx_L15_bool_binop_done;
        }
        __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_s, 1, 2, NULL, NULL, &__pyx_slice__49, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_isdigit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        if (__pyx_t_4) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 553, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else {
          __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 553, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_t_7;
        __pyx_L15_bool_binop_done:;
        if (__pyx_t_5) {

          /* "lpu/smt/trans_models/records.pyx":555
 *                 if s[0:1] == 'x' and s[1:2].isdigit():
 * #                        print("NONTERMINAL!")
 *                     if len(s) > 3:             # <<<<<<<<<<<<<<
 *                         # symbol is tagged
 *                         pass
 */
          __pyx_t_11 = PyObject_Length(__pyx_v_s); if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 555, __pyx_L1_error)
          __pyx_t_5 = ((__pyx_t_11 > 3) != 0);
          if (__pyx_t_5) {
            goto __pyx_L17;
          }

          /* "lpu/smt/trans_models/records.pyx":560
 *                     else:
 *                         # symbol is not tagged
 *                         index = int(s[1])             # <<<<<<<<<<<<<<
 *                         if sync == 'scfg':
 *                             trgSymbols[i] = srcNonTerminals[index]
 */
          /*else*/ {
            __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_s, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 560, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_10 = __Pyx_PyNumber_Int(__pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 560, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_10);
            __pyx_t_10 = 0;

            /* "lpu/smt/trans_models/records.pyx":561
 *                         # symbol is not tagged
 *                         index = int(s[1])
 *                         if sync == 'scfg':             # <<<<<<<<<<<<<<
 *                             trgSymbols[i] = srcNonTerminals[index]
 *                         elif sync == 'hiero':
 */
            __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_sync, __pyx_n_s_scfg, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 561, __pyx_L1_error)
            if (__pyx_t_5) {

              /* "lpu/smt/trans_models/records.pyx":562
 *                         index = int(s[1])
 *                         if sync == 'scfg':
 *                             trgSymbols[i] = srcNonTerminals[index]             # <<<<<<<<<<<<<<
 *                         elif sync == 'hiero':
 *                             trgSymbols[i] = s + ":X"
 */
              __pyx_t_10 = PyObject_GetItem(__pyx_v_srcNonTerminals, __pyx_v_index); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 562, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (unlikely(PyObject_SetItem(__pyx_v_trgSymbols, __pyx_v_i, __pyx_t_10) < 0)) __PYX_ERR(0, 562, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

              /* "lpu/smt/trans_models/records.pyx":561
 *                         # symbol is not tagged
 *                         index = int(s[1])
 *                         if sync == 'scfg':             # <<<<<<<<<<<<<<
 *                             trgSymbols[i] = srcNonTerminals[index]
 *                         elif sync == 'hiero':
 */
              goto __pyx_L18;
            }

            /* "lpu/smt/trans_models/records.pyx":563
 *                         if sync == 'scfg':
 *                             trgSymbols[i] = srcNonTerminals[index]
 *                         elif sync == 'hiero':             # <<<<<<<<<<<<<<
 *                             trgSymbols[i] = s + ":X"
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 */
            __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_sync, __pyx_n_s_hiero, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 563, __pyx_L1_error)
            if (__pyx_t_5) {

              /* "lpu/smt/trans_models/records.pyx":564
 *                             trgSymbols[i] = srcNonTerminals[index]
 *                         elif sync == 'hiero':
 *                             trgSymbols[i] = s + ":X"             # <<<<<<<<<<<<<<
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *             trgSymbols.append("@")
 */
              __pyx_t_10 = PyNumber_Add(__pyx_v_s, __pyx_kp_s_X); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 564, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (unlikely(PyObject_SetItem(__pyx_v_trgSymbols, __pyx_v_i, __pyx_t_10) < 0)) __PYX_ERR(0, 564, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

              /* "lpu/smt/trans_models/records.pyx":563
 *                         if sync == 'scfg':
 *                             trgSymbols[i] = srcNonTerminals[index]
 *                         elif sync == 'hiero':             # <<<<<<<<<<<<<<
 *                             trgSymbols[i] = s + ":X"
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 */
            }
            __pyx_L18:;
          }
          __pyx_L17:;

          /* "lpu/smt/trans_models/records.pyx":553
 *             for i, s in enumerate(trgSymbols):
 * #                    print("TRG SYMBOLS: %s" % s)
 *                 if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 * #                        print("NONTERMINAL!")
 *                     if len(s) > 3:
 */
        }

        /* "lpu/smt/trans_models/records.pyx":551
 * #                        else:
 * #                            srcNonTerminals.append('X')
 *             for i, s in enumerate(trgSymbols):             # <<<<<<<<<<<<<<
 * #                    print("TRG SYMBOLS: %s" % s)
 *                 if s[0:1] == 'x' and s[1:2].isdigit():
 */
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "lpu/smt/trans_models/records.pyx":566
 *                             trgSymbols[i] = s + ":X"
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *             trgSymbols.append("@")             # <<<<<<<<<<<<<<
 *             if sync == 'scfg':
 *                 trgSymbols.append(srcTag)
 */
      __pyx_t_9 = __Pyx_PyObject_Append(__pyx_v_trgSymbols, __pyx_kp_s__20); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 566, __pyx_L1_error)

      /* "lpu/smt/trans_models/records.pyx":567
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *             trgSymbols.append("@")
 *             if sync == 'scfg':             # <<<<<<<<<<<<<<
 *                 trgSymbols.append(srcTag)
 *             elif sync == 'hiero':
 */
      __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_sync, __pyx_n_s_scfg, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 567, __pyx_L1_error)
      if (__pyx_t_5) {

        /* "lpu/smt/trans_models/records.pyx":568
 *             trgSymbols.append("@")
 *             if sync == 'scfg':
 *                 trgSymbols.append(srcTag)             # <<<<<<<<<<<<<<
 *             elif sync == 'hiero':
 *                 trgSymbols.append('X')
 */
        __pyx_t_9 = __Pyx_PyObject_Append(__pyx_v_trgSymbols, __pyx_v_srcTag); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 568, __pyx_L1_error)

        /* "lpu/smt/trans_models/records.pyx":567
 * #                self.trg = intern(self.trg + " @ " + srcTag)
 *             trgSymbols.append("@")
 *             if sync == 'scfg':             # <<<<<<<<<<<<<<
 *                 trgSymbols.append(srcTag)
 *             elif sync == 'hiero':
 */
        goto __pyx_L19;
      }

      /* "lpu/smt/trans_models/records.pyx":569
 *             if sync == 'scfg':
 *                 trgSymbols.append(srcTag)
 *             elif sync == 'hiero':             # <<<<<<<<<<<<<<
 *                 trgSymbols.append('X')
 * #            self.trg = intern(str.join(' ', trgSymbols))
 */
      __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_sync, __pyx_n_s_hiero, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 569, __pyx_L1_error)
      if (__pyx_t_5) {

        /* "lpu/smt/trans_models/records.pyx":570
 *                 trgSymbols.append(srcTag)
 *             elif sync == 'hiero':
 *                 trgSymbols.append('X')             # <<<<<<<<<<<<<<
 * #            self.trg = intern(str.join(' ', trgSymbols))
 * #    if len(src_symbols) == 3:
 */
        __pyx_t_9 = __Pyx_PyObject_Append(__pyx_v_trgSymbols, __pyx_n_s_X_2); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 570, __pyx_L1_error)

        /* "lpu/smt/trans_models/records.pyx":569
 *             if sync == 'scfg':
 *                 trgSymbols.append(srcTag)
 *             elif sync == 'hiero':             # <<<<<<<<<<<<<<
 *                 trgSymbols.append('X')
 * #            self.trg = intern(str.join(' ', trgSymbols))
 */
      }
      __pyx_L19:;
    }
    __pyx_L4:;

    /* "lpu/smt/trans_models/records.pyx":535
 * 
 * def syncTags(src_symbols, trgSymbols, sync):
 *     if isTree(src_symbols):             # <<<<<<<<<<<<<<
 *         # src is tree (s-expression)
 *         if len(trgSymbols) >= 2 and trgSymbols[-2] == "@":
 */
  }

  /* "lpu/smt/trans_models/records.pyx":578
 * #            self.src = ""
 * #    print(trgSymbols)
 *     return src_symbols, trgSymbols             # <<<<<<<<<<<<<<
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_src_symbols);
  __Pyx_GIVEREF(__pyx_v_src_symbols);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_src_symbols);
  __Pyx_INCREF(__pyx_v_trgSymbols);
  __Pyx_GIVEREF(__pyx_v_trgSymbols);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_trgSymbols);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "lpu/smt/trans_models/records.pyx":534
 *         return False
 * 
 * def syncTags(src_symbols, trgSymbols, sync):             # <<<<<<<<<<<<<<
 *     if isTree(src_symbols):
 *         # src is tree (s-expression)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("lpu.smt.trans_models.records.syncTags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_srcTag);
  __Pyx_XDECREF(__pyx_v_srcNonTerminals);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_CoOccurrence __pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence;

static PyObject *__pyx_tp_new_3lpu_3smt_12trans_models_7records_CoOccurrence(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)o);
  p->__pyx_vtab = __pyx_vtabptr_3lpu_3smt_12trans_models_7records_CoOccurrence;
  if (unlikely(__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_3lpu_3smt_12trans_models_7records_CoOccurrence(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_egfp(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4egfp_1__get__(o);
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_fgep(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4fgep_1__get__(o);
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_src(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_src(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3src_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_trg(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_trg(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3trg_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_cooc(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_cooc(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_4cooc_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_3lpu_3smt_12trans_models_7records_CoOccurrence[] = {
  {"calcEGFP", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_3calcEGFP, METH_NOARGS, 0},
  {"calcFGEP", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_5calcFGEP, METH_NOARGS, 0},
  {"getReversed", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_7getReversed, METH_NOARGS, 0},
  {"setCounts", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_9setCounts, METH_VARARGS|METH_KEYWORDS, 0},
  {"to_str", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_11to_str, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_15__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_17__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_3lpu_3smt_12trans_models_7records_CoOccurrence[] = {
  {(char *)"egfp", __pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_egfp, 0, (char *)0, 0},
  {(char *)"fgep", __pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_fgep, 0, (char *)0, 0},
  {(char *)"src", __pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_src, __pyx_setprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_src, (char *)0, 0},
  {(char *)"trg", __pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_trg, __pyx_setprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_trg, (char *)0, 0},
  {(char *)"cooc", __pyx_getprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_cooc, __pyx_setprop_3lpu_3smt_12trans_models_7records_12CoOccurrence_cooc, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_3lpu_3smt_12trans_models_7records_CoOccurrence = {
  PyVarObject_HEAD_INIT(0, 0)
  "lpu.smt.trans_models.records.CoOccurrence", /*tp_name*/
  sizeof(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_3lpu_3smt_12trans_models_7records_CoOccurrence, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_3lpu_3smt_12trans_models_7records_12CoOccurrence_13__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_3lpu_3smt_12trans_models_7records_CoOccurrence, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_3lpu_3smt_12trans_models_7records_CoOccurrence, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_3lpu_3smt_12trans_models_7records_CoOccurrence, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record __pyx_vtable_3lpu_3smt_12trans_models_7records_Record;

static PyObject *__pyx_tp_new_3lpu_3smt_12trans_models_7records_Record(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)o);
  p->__pyx_vtab = __pyx_vtabptr_3lpu_3smt_12trans_models_7records_Record;
  p->src = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->trg = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->features = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->counts = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)Py_None); Py_INCREF(Py_None);
  p->aligns = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->data = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_3lpu_3smt_12trans_models_7records_Record(PyObject *o) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *p = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->src);
  Py_CLEAR(p->trg);
  Py_CLEAR(p->features);
  Py_CLEAR(p->counts);
  Py_CLEAR(p->aligns);
  Py_CLEAR(p->data);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_3lpu_3smt_12trans_models_7records_Record(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *p = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)o;
  if (p->features) {
    e = (*v)(p->features, a); if (e) return e;
  }
  if (p->counts) {
    e = (*v)(((PyObject *)p->counts), a); if (e) return e;
  }
  if (p->aligns) {
    e = (*v)(p->aligns, a); if (e) return e;
  }
  if (p->data) {
    e = (*v)(p->data, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_3lpu_3smt_12trans_models_7records_Record(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *p = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *)o;
  tmp = ((PyObject*)p->features);
  p->features = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->counts);
  p->counts = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->aligns);
  p->aligns = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->data);
  p->data = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_alignMap(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8alignMap_1__get__(o);
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_alignMapRev(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_11alignMapRev_1__get__(o);
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_src(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_src(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_3__set__(o, v);
  }
  else {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3src_5__del__(o);
  }
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_trg(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_trg(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_3__set__(o, v);
  }
  else {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3trg_5__del__(o);
  }
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_features(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_features(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_3__set__(o, v);
  }
  else {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_8features_5__del__(o);
  }
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_counts(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_counts(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_3__set__(o, v);
  }
  else {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6counts_5__del__(o);
  }
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_aligns(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_aligns(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_3__set__(o, v);
  }
  else {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_6aligns_5__del__(o);
  }
}

static PyObject *__pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_1__get__(o);
}

static int __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_data(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_3__set__(o, v);
  }
  else {
    return __pyx_pw_3lpu_3smt_12trans_models_7records_6Record_4data_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_3lpu_3smt_12trans_models_7records_Record[] = {
  {"fixOrder", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_3fixOrder, METH_NOARGS, 0},
  {"getSrcSymbols", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_5getSrcSymbols, METH_NOARGS, 0},
  {"getSrcTerms", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_7getSrcTerms, METH_NOARGS, 0},
  {"getTrgSymbols", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_9getTrgSymbols, METH_NOARGS, 0},
  {"getTrgTerms", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_11getTrgTerms, METH_NOARGS, 0},
  {"getAlignMap", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_13getAlignMap, METH_NOARGS, 0},
  {"getAlignMapRev", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_15getAlignMapRev, METH_NOARGS, 0},
  {"getReversed", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_17getReversed, METH_NOARGS, 0},
  {"getTerms", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_19getTerms, METH_VARARGS|METH_KEYWORDS, 0},
  {"getSymbols", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_21getSymbols, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_23__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_6Record_25__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_3lpu_3smt_12trans_models_7records_Record[] = {
  {(char *)"alignMap", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_alignMap, 0, (char *)0, 0},
  {(char *)"alignMapRev", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_alignMapRev, 0, (char *)0, 0},
  {(char *)"src", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_src, __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_src, (char *)0, 0},
  {(char *)"trg", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_trg, __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_trg, (char *)0, 0},
  {(char *)"features", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_features, __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_features, (char *)0, 0},
  {(char *)"counts", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_counts, __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_counts, (char *)0, 0},
  {(char *)"aligns", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_aligns, __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_aligns, (char *)0, 0},
  {(char *)"data", __pyx_getprop_3lpu_3smt_12trans_models_7records_6Record_data, __pyx_setprop_3lpu_3smt_12trans_models_7records_6Record_data, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_3lpu_3smt_12trans_models_7records_Record = {
  PyVarObject_HEAD_INIT(0, 0)
  "lpu.smt.trans_models.records.Record", /*tp_name*/
  sizeof(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_3lpu_3smt_12trans_models_7records_Record, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_3lpu_3smt_12trans_models_7records_Record, /*tp_traverse*/
  __pyx_tp_clear_3lpu_3smt_12trans_models_7records_Record, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_3lpu_3smt_12trans_models_7records_Record, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_3lpu_3smt_12trans_models_7records_Record, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_3lpu_3smt_12trans_models_7records_Record, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_TravatarRecord __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord;

static PyObject *__pyx_tp_new_3lpu_3smt_12trans_models_7records_TravatarRecord(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *p;
  PyObject *o = __pyx_tp_new_3lpu_3smt_12trans_models_7records_Record(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_3lpu_3smt_12trans_models_7records_Record*)__pyx_vtabptr_3lpu_3smt_12trans_models_7records_TravatarRecord;
  p->delim = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_3lpu_3smt_12trans_models_7records_TravatarRecord(PyObject *o) {
  struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *p = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->delim);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_3lpu_3smt_12trans_models_7records_Record(o);
}

static PyMethodDef __pyx_methods_3lpu_3smt_12trans_models_7records_TravatarRecord[] = {
  {"fixOrder", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_3fixOrder, METH_NOARGS, 0},
  {"getReversed", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_5getReversed, METH_NOARGS, 0},
  {"loadLine", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_7loadLine, METH_VARARGS|METH_KEYWORDS, 0},
  {"syncTags", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_9syncTags, METH_NOARGS, 0},
  {"to_str", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_11to_str, METH_VARARGS|METH_KEYWORDS, 0},
  {"fixOrderStrings", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_13fixOrderStrings, METH_VARARGS|METH_KEYWORDS, 0},
  {"getSymbols", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_15getSymbols, METH_VARARGS|METH_KEYWORDS, 0},
  {"getTerms", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_17getTerms, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_19__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_14TravatarRecord_21__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord = {
  PyVarObject_HEAD_INIT(0, 0)
  "lpu.smt.trans_models.records.TravatarRecord", /*tp_name*/
  sizeof(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_3lpu_3smt_12trans_models_7records_TravatarRecord, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_3lpu_3smt_12trans_models_7records_Record, /*tp_traverse*/
  __pyx_tp_clear_3lpu_3smt_12trans_models_7records_Record, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_3lpu_3smt_12trans_models_7records_TravatarRecord, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_3lpu_3smt_12trans_models_7records_TravatarRecord, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"getStrTravatarFeatures", (PyCFunction)__pyx_pw_3lpu_3smt_12trans_models_7records_17getStrTravatarFeatures, METH_O, __pyx_doc_3lpu_3smt_12trans_models_7records_16getStrTravatarFeatures},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "records",
    __pyx_k_classes_handling_rule_table_reco, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_MATCH, __pyx_k_MATCH, sizeof(__pyx_k_MATCH), 0, 0, 1, 1},
  {&__pyx_n_s_MosesRecord, __pyx_k_MosesRecord, sizeof(__pyx_k_MosesRecord), 0, 0, 1, 1},
  {&__pyx_n_s_MosesRecord___init, __pyx_k_MosesRecord___init, sizeof(__pyx_k_MosesRecord___init), 0, 0, 1, 1},
  {&__pyx_n_s_MosesRecord_loadLine, __pyx_k_MosesRecord_loadLine, sizeof(__pyx_k_MosesRecord_loadLine), 0, 0, 1, 1},
  {&__pyx_n_s_MosesRecord_to_str, __pyx_k_MosesRecord_to_str, sizeof(__pyx_k_MosesRecord_to_str), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_X, __pyx_k_X, sizeof(__pyx_k_X), 0, 0, 1, 0},
  {&__pyx_n_s_X_2, __pyx_k_X_2, sizeof(__pyx_k_X_2), 0, 0, 1, 1},
  {&__pyx_kp_s_X_3, __pyx_k_X_3, sizeof(__pyx_k_X_3), 0, 0, 1, 0},
  {&__pyx_kp_s__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 0, 1, 0},
  {&__pyx_kp_s__11, __pyx_k__11, sizeof(__pyx_k__11), 0, 0, 1, 0},
  {&__pyx_kp_s__20, __pyx_k__20, sizeof(__pyx_k__20), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_kp_s__30, __pyx_k__30, sizeof(__pyx_k__30), 0, 0, 1, 0},
  {&__pyx_kp_s__39, __pyx_k__39, sizeof(__pyx_k__39), 0, 0, 1, 0},
  {&__pyx_kp_s__4, __pyx_k__4, sizeof(__pyx_k__4), 0, 0, 1, 0},
  {&__pyx_kp_s__43, __pyx_k__43, sizeof(__pyx_k__43), 0, 0, 1, 0},
  {&__pyx_kp_s__45, __pyx_k__45, sizeof(__pyx_k__45), 0, 0, 1, 0},
  {&__pyx_kp_s__9, __pyx_k__9, sizeof(__pyx_k__9), 0, 0, 1, 0},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_align, __pyx_k_align, sizeof(__pyx_k_align), 0, 0, 1, 1},
  {&__pyx_n_s_alignMap, __pyx_k_alignMap, sizeof(__pyx_k_alignMap), 0, 0, 1, 1},
  {&__pyx_n_s_aligns, __pyx_k_aligns, sizeof(__pyx_k_aligns), 0, 0, 1, 1},
  {&__pyx_n_s_anyNum, __pyx_k_anyNum, sizeof(__pyx_k_anyNum), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_buf, __pyx_k_buf, sizeof(__pyx_k_buf), 0, 0, 1, 1},
  {&__pyx_n_s_calcEGFP, __pyx_k_calcEGFP, sizeof(__pyx_k_calcEGFP), 0, 0, 1, 1},
  {&__pyx_n_s_calcFGEP, __pyx_k_calcFGEP, sizeof(__pyx_k_calcFGEP), 0, 0, 1, 1},
  {&__pyx_n_s_changed, __pyx_k_changed, sizeof(__pyx_k_changed), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_co, __pyx_k_co, sizeof(__pyx_k_co), 0, 0, 1, 1},
  {&__pyx_n_s_compat, __pyx_k_compat, sizeof(__pyx_k_compat), 0, 0, 1, 1},
  {&__pyx_n_s_cooc, __pyx_k_cooc, sizeof(__pyx_k_cooc), 0, 0, 1, 1},
  {&__pyx_n_s_counts, __pyx_k_counts, sizeof(__pyx_k_counts), 0, 0, 1, 1},
  {&__pyx_kp_s_d_d, __pyx_k_d_d, sizeof(__pyx_k_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_delim, __pyx_k_delim, sizeof(__pyx_k_delim), 0, 0, 1, 1},
  {&__pyx_n_s_dicFeatures, __pyx_k_dicFeatures, sizeof(__pyx_k_dicFeatures), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_e, __pyx_k_e, sizeof(__pyx_k_e), 0, 0, 1, 1},
  {&__pyx_n_s_egf, __pyx_k_egf, sizeof(__pyx_k_egf), 0, 0, 1, 1},
  {&__pyx_n_s_egfl, __pyx_k_egfl, sizeof(__pyx_k_egfl), 0, 0, 1, 1},
  {&__pyx_n_s_egfp, __pyx_k_egfp, sizeof(__pyx_k_egfp), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_features, __pyx_k_features, sizeof(__pyx_k_features), 0, 0, 1, 1},
  {&__pyx_n_s_fge, __pyx_k_fge, sizeof(__pyx_k_fge), 0, 0, 1, 1},
  {&__pyx_n_s_fgel, __pyx_k_fgel, sizeof(__pyx_k_fgel), 0, 0, 1, 1},
  {&__pyx_n_s_fgep, __pyx_k_fgep, sizeof(__pyx_k_fgep), 0, 0, 1, 1},
  {&__pyx_n_s_field, __pyx_k_field, sizeof(__pyx_k_field), 0, 0, 1, 1},
  {&__pyx_n_s_fields, __pyx_k_fields, sizeof(__pyx_k_fields), 0, 0, 1, 1},
  {&__pyx_n_s_find, __pyx_k_find, sizeof(__pyx_k_find), 0, 0, 1, 1},
  {&__pyx_n_s_fixOrder, __pyx_k_fixOrder, sizeof(__pyx_k_fixOrder), 0, 0, 1, 1},
  {&__pyx_n_s_fixOrderStrings, __pyx_k_fixOrderStrings, sizeof(__pyx_k_fixOrderStrings), 0, 0, 1, 1},
  {&__pyx_n_s_ftree, __pyx_k_ftree, sizeof(__pyx_k_ftree), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getAlignMap, __pyx_k_getAlignMap, sizeof(__pyx_k_getAlignMap), 0, 0, 1, 1},
  {&__pyx_n_s_getAlignMapRev, __pyx_k_getAlignMapRev, sizeof(__pyx_k_getAlignMapRev), 0, 0, 1, 1},
  {&__pyx_n_s_getAlignSet, __pyx_k_getAlignSet, sizeof(__pyx_k_getAlignSet), 0, 0, 1, 1},
  {&__pyx_n_s_getCounts, __pyx_k_getCounts, sizeof(__pyx_k_getCounts), 0, 0, 1, 1},
  {&__pyx_n_s_getFlattenSymbols, __pyx_k_getFlattenSymbols, sizeof(__pyx_k_getFlattenSymbols), 0, 0, 1, 1},
  {&__pyx_n_s_getMosesFeatures, __pyx_k_getMosesFeatures, sizeof(__pyx_k_getMosesFeatures), 0, 0, 1, 1},
  {&__pyx_n_s_getNumber, __pyx_k_getNumber, sizeof(__pyx_k_getNumber), 0, 0, 1, 1},
  {&__pyx_n_s_getRevAlignSet, __pyx_k_getRevAlignSet, sizeof(__pyx_k_getRevAlignSet), 0, 0, 1, 1},
  {&__pyx_n_s_getReversed, __pyx_k_getReversed, sizeof(__pyx_k_getReversed), 0, 0, 1, 1},
  {&__pyx_n_s_getSrcSymbols, __pyx_k_getSrcSymbols, sizeof(__pyx_k_getSrcSymbols), 0, 0, 1, 1},
  {&__pyx_n_s_getSrcTerms, __pyx_k_getSrcTerms, sizeof(__pyx_k_getSrcTerms), 0, 0, 1, 1},
  {&__pyx_n_s_getStrMosesFeatures, __pyx_k_getStrMosesFeatures, sizeof(__pyx_k_getStrMosesFeatures), 0, 0, 1, 1},
  {&__pyx_n_s_getSymbols, __pyx_k_getSymbols, sizeof(__pyx_k_getSymbols), 0, 0, 1, 1},
  {&__pyx_n_s_getTerms, __pyx_k_getTerms, sizeof(__pyx_k_getTerms), 0, 0, 1, 1},
  {&__pyx_n_s_getTravatarFeatures, __pyx_k_getTravatarFeatures, sizeof(__pyx_k_getTravatarFeatures), 0, 0, 1, 1},
  {&__pyx_n_s_getTrgSymbols, __pyx_k_getTrgSymbols, sizeof(__pyx_k_getTrgSymbols), 0, 0, 1, 1},
  {&__pyx_n_s_getTrgTerms, __pyx_k_getTrgTerms, sizeof(__pyx_k_getTrgTerms), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_grammar, __pyx_k_grammar, sizeof(__pyx_k_grammar), 0, 0, 1, 1},
  {&__pyx_n_s_hiero, __pyx_k_hiero, sizeof(__pyx_k_hiero), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_isTree, __pyx_k_isTree, sizeof(__pyx_k_isTree), 0, 0, 1, 1},
  {&__pyx_n_s_isdigit, __pyx_k_isdigit, sizeof(__pyx_k_isdigit), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_line, __pyx_k_line, sizeof(__pyx_k_line), 0, 0, 1, 1},
  {&__pyx_n_s_listCounts, __pyx_k_listCounts, sizeof(__pyx_k_listCounts), 0, 0, 1, 1},
  {&__pyx_n_s_loadLine, __pyx_k_loadLine, sizeof(__pyx_k_loadLine), 0, 0, 1, 1},
  {&__pyx_n_s_log, __pyx_k_log, sizeof(__pyx_k_log), 0, 0, 1, 1},
  {&__pyx_n_s_logging, __pyx_k_logging, sizeof(__pyx_k_logging), 0, 0, 1, 1},
  {&__pyx_n_s_lpu_common, __pyx_k_lpu_common, sizeof(__pyx_k_lpu_common), 0, 0, 1, 1},
  {&__pyx_n_s_lpu_smt_trans_models_records, __pyx_k_lpu_smt_trans_models_records, sizeof(__pyx_k_lpu_smt_trans_models_records), 0, 0, 1, 1},
  {&__pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_k_lpu_smt_trans_models_records_pyx, sizeof(__pyx_k_lpu_smt_trans_models_records_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_map, __pyx_k_map, sizeof(__pyx_k_map), 0, 0, 1, 1},
  {&__pyx_n_s_margin, __pyx_k_margin, sizeof(__pyx_k_margin), 0, 0, 1, 1},
  {&__pyx_n_s_math, __pyx_k_math, sizeof(__pyx_k_math), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_newSymbol, __pyx_k_newSymbol, sizeof(__pyx_k_newSymbol), 0, 0, 1, 1},
  {&__pyx_n_s_newSymbols, __pyx_k_newSymbols, sizeof(__pyx_k_newSymbols), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_numFloat, __pyx_k_numFloat, sizeof(__pyx_k_numFloat), 0, 0, 1, 1},
  {&__pyx_n_s_numInt, __pyx_k_numInt, sizeof(__pyx_k_numInt), 0, 0, 1, 1},
  {&__pyx_n_s_numbers, __pyx_k_numbers, sizeof(__pyx_k_numbers), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_phrase, __pyx_k_phrase, sizeof(__pyx_k_phrase), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_property, __pyx_k_property, sizeof(__pyx_k_property), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_replaceDict, __pyx_k_replaceDict, sizeof(__pyx_k_replaceDict), 0, 0, 1, 1},
  {&__pyx_n_s_revAlignSet, __pyx_k_revAlignSet, sizeof(__pyx_k_revAlignSet), 0, 0, 1, 1},
  {&__pyx_n_s_reverse, __pyx_k_reverse, sizeof(__pyx_k_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_s_rule, __pyx_k_rule, sizeof(__pyx_k_rule), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_kp_s_s_2, __pyx_k_s_2, sizeof(__pyx_k_s_2), 0, 0, 1, 0},
  {&__pyx_kp_s_s_s, __pyx_k_s_s, sizeof(__pyx_k_s_s), 0, 0, 1, 0},
  {&__pyx_kp_s_s_s_s, __pyx_k_s_s_s, sizeof(__pyx_k_s_s_s), 0, 0, 1, 0},
  {&__pyx_kp_s_s_s_src_s_trg_s_cooc_s, __pyx_k_s_s_src_s_trg_s_cooc_s, sizeof(__pyx_k_s_s_src_s_trg_s_cooc_s), 0, 0, 1, 0},
  {&__pyx_n_s_scfg, __pyx_k_scfg, sizeof(__pyx_k_scfg), 0, 0, 1, 1},
  {&__pyx_n_s_scores, __pyx_k_scores, sizeof(__pyx_k_scores), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_setCounts, __pyx_k_setCounts, sizeof(__pyx_k_setCounts), 0, 0, 1, 1},
  {&__pyx_n_s_setdefault, __pyx_k_setdefault, sizeof(__pyx_k_setdefault), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_src, __pyx_k_src, sizeof(__pyx_k_src), 0, 0, 1, 1},
  {&__pyx_n_s_srcElements, __pyx_k_srcElements, sizeof(__pyx_k_srcElements), 0, 0, 1, 1},
  {&__pyx_n_s_srcNonTerminals, __pyx_k_srcNonTerminals, sizeof(__pyx_k_srcNonTerminals), 0, 0, 1, 1},
  {&__pyx_n_s_srcTag, __pyx_k_srcTag, sizeof(__pyx_k_srcTag), 0, 0, 1, 1},
  {&__pyx_n_s_srcTerms, __pyx_k_srcTerms, sizeof(__pyx_k_srcTerms), 0, 0, 1, 1},
  {&__pyx_n_s_src_symbols, __pyx_k_src_symbols, sizeof(__pyx_k_src_symbols), 0, 0, 1, 1},
  {&__pyx_n_s_staticmethod, __pyx_k_staticmethod, sizeof(__pyx_k_staticmethod), 0, 0, 1, 1},
  {&__pyx_n_s_strAligns, __pyx_k_strAligns, sizeof(__pyx_k_strAligns), 0, 0, 1, 1},
  {&__pyx_n_s_strCounts, __pyx_k_strCounts, sizeof(__pyx_k_strCounts), 0, 0, 1, 1},
  {&__pyx_n_s_strFeatures, __pyx_k_strFeatures, sizeof(__pyx_k_strFeatures), 0, 0, 1, 1},
  {&__pyx_n_s_strField, __pyx_k_strField, sizeof(__pyx_k_strField), 0, 0, 1, 1},
  {&__pyx_n_s_strKeyVal, __pyx_k_strKeyVal, sizeof(__pyx_k_strKeyVal), 0, 0, 1, 1},
  {&__pyx_n_s_string, __pyx_k_string, sizeof(__pyx_k_string), 0, 0, 1, 1},
  {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
  {&__pyx_n_s_symbols, __pyx_k_symbols, sizeof(__pyx_k_symbols), 0, 0, 1, 1},
  {&__pyx_n_s_sync, __pyx_k_sync, sizeof(__pyx_k_sync), 0, 0, 1, 1},
  {&__pyx_n_s_syncTags, __pyx_k_syncTags, sizeof(__pyx_k_syncTags), 0, 0, 1, 1},
  {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
  {&__pyx_n_s_tag, __pyx_k_tag, sizeof(__pyx_k_tag), 0, 0, 1, 1},
  {&__pyx_n_s_terms, __pyx_k_terms, sizeof(__pyx_k_terms), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_toNumber, __pyx_k_toNumber, sizeof(__pyx_k_toNumber), 0, 0, 1, 1},
  {&__pyx_n_s_to_str, __pyx_k_to_str, sizeof(__pyx_k_to_str), 0, 0, 1, 1},
  {&__pyx_n_s_tree, __pyx_k_tree, sizeof(__pyx_k_tree), 0, 0, 1, 1},
  {&__pyx_n_s_trg, __pyx_k_trg, sizeof(__pyx_k_trg), 0, 0, 1, 1},
  {&__pyx_n_s_trgElements, __pyx_k_trgElements, sizeof(__pyx_k_trgElements), 0, 0, 1, 1},
  {&__pyx_n_s_trgSymbols, __pyx_k_trgSymbols, sizeof(__pyx_k_trgSymbols), 0, 0, 1, 1},
  {&__pyx_n_s_trgTerms, __pyx_k_trgTerms, sizeof(__pyx_k_trgTerms), 0, 0, 1, 1},
  {&__pyx_n_s_val, __pyx_k_val, sizeof(__pyx_k_val), 0, 0, 1, 1},
  {&__pyx_n_s_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 0, 1, 1},
  {&__pyx_n_s_warn, __pyx_k_warn, sizeof(__pyx_k_warn), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_kp_s_x0, __pyx_k_x0, sizeof(__pyx_k_x0), 0, 0, 1, 0},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_property = __Pyx_GetBuiltinName(__pyx_n_s_property); if (!__pyx_builtin_property) __PYX_ERR(0, 91, __pyx_L1_error)
  __pyx_builtin_staticmethod = __Pyx_GetBuiltinName(__pyx_n_s_staticmethod); if (!__pyx_builtin_staticmethod) __PYX_ERR(0, 143, __pyx_L1_error)
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_map = __Pyx_GetBuiltinName(__pyx_n_s_map); if (!__pyx_builtin_map) __PYX_ERR(0, 202, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 367, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "lpu/smt/trans_models/records.pyx":146
 *     def getTerms(phrase):
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')             # <<<<<<<<<<<<<<
 *         return phrase
 * 
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "lpu/smt/trans_models/records.pyx":152
 *     def getSymbols(phrase):
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')             # <<<<<<<<<<<<<<
 *         return phrase
 * 
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "lpu/smt/trans_models/records.pyx":202
 *     alignMap = {}
 *     for align in aligns:
 *         (s, t) = map(int, align.split('-'))             # <<<<<<<<<<<<<<
 *         if reverse:
 *             alignMap.setdefault(t, []).append(s)
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s__11); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "lpu/smt/trans_models/records.pyx":236
 * #    debug.log(aligns)
 *     for a in aligns:
 *       (s, t) = map(int, a.split('-'))             # <<<<<<<<<<<<<<
 * #      revAlignList.append( "%d-%d" % (t, s) )
 *       revAlignSet.add( "%d-%d" % (t, s) )
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s__11); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "lpu/smt/trans_models/records.pyx":255
 *     '''convert back the feature dictionary to score string separated by space'''
 *     scores = []
 *     scores.append( dicFeatures.get('fgep', 0) )             # <<<<<<<<<<<<<<
 *     scores.append( dicFeatures.get('fgel', 0) )
 *     scores.append( dicFeatures.get('egfp', 0) )
 */
  __pyx_tuple__14 = PyTuple_Pack(2, __pyx_n_s_fgep, __pyx_int_0); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "lpu/smt/trans_models/records.pyx":256
 *     scores = []
 *     scores.append( dicFeatures.get('fgep', 0) )
 *     scores.append( dicFeatures.get('fgel', 0) )             # <<<<<<<<<<<<<<
 *     scores.append( dicFeatures.get('egfp', 0) )
 *     scores.append( dicFeatures.get('egfl', 0) )
 */
  __pyx_tuple__15 = PyTuple_Pack(2, __pyx_n_s_fgel, __pyx_int_0); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "lpu/smt/trans_models/records.pyx":257
 *     scores.append( dicFeatures.get('fgep', 0) )
 *     scores.append( dicFeatures.get('fgel', 0) )
 *     scores.append( dicFeatures.get('egfp', 0) )             # <<<<<<<<<<<<<<
 *     scores.append( dicFeatures.get('egfl', 0) )
 *     return str.join(' ', map(str,scores))
 */
  __pyx_tuple__16 = PyTuple_Pack(2, __pyx_n_s_egfp, __pyx_int_0); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "lpu/smt/trans_models/records.pyx":258
 *     scores.append( dicFeatures.get('fgel', 0) )
 *     scores.append( dicFeatures.get('egfp', 0) )
 *     scores.append( dicFeatures.get('egfl', 0) )             # <<<<<<<<<<<<<<
 *     return str.join(' ', map(str,scores))
 * 
 */
  __pyx_tuple__17 = PyTuple_Pack(2, __pyx_n_s_egfl, __pyx_int_0); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "lpu/smt/trans_models/records.pyx":348
 *     cpdef syncTags(self):
 *         self.sync = False
 *         src_symbols = self.src.split(' ')             # <<<<<<<<<<<<<<
 *         trgSymbols = self.trg.split(' ')
 * #        if len(src_symbols) >= 4 and src_symbols[0][0] != '"':
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "lpu/smt/trans_models/records.pyx":349
 *         self.sync = False
 *         src_symbols = self.src.split(' ')
 *         trgSymbols = self.trg.split(' ')             # <<<<<<<<<<<<<<
 * #        if len(src_symbols) >= 4 and src_symbols[0][0] != '"':
 *         if isTree(src_symbols):
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "lpu/smt/trans_models/records.pyx":361
 *                 srcNonTerminals = []
 *                 for s in src_symbols:
 *                     if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 *                         srcNonTerminals.append(s)
 * #                        if len(s) > 3:
 */
  __pyx_slice__21 = PySlice_New(__pyx_int_0, __pyx_int_1, Py_None); if (unlikely(!__pyx_slice__21)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__21);
  __Pyx_GIVEREF(__pyx_slice__21);
  __pyx_slice__22 = PySlice_New(__pyx_int_1, __pyx_int_2, Py_None); if (unlikely(!__pyx_slice__22)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__22);
  __Pyx_GIVEREF(__pyx_slice__22);

  /* "lpu/smt/trans_models/records.pyx":369
 *                 for i, s in enumerate(trgSymbols):
 * #                    print("TRG SYMBOLS: %s" % s)
 *                     if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 * #                        print("NONTERMINAL!")
 *                         if len(s) > 3:
 */
  __pyx_slice__23 = PySlice_New(__pyx_int_0, __pyx_int_1, Py_None); if (unlikely(!__pyx_slice__23)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__23);
  __Pyx_GIVEREF(__pyx_slice__23);
  __pyx_slice__24 = PySlice_New(__pyx_int_1, __pyx_int_2, Py_None); if (unlikely(!__pyx_slice__24)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__24);
  __Pyx_GIVEREF(__pyx_slice__24);

  /* "lpu/smt/trans_models/records.pyx":425
 *         cdef str newSymbol
 *         cdef object changed = False
 *         for s in src.split(' '):             # <<<<<<<<<<<<<<
 *             if s[0] == 'x' and s[1].isdigit():
 *                 replaceDict[s[1]] = str(len(replaceDict))
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "lpu/smt/trans_models/records.pyx":428
 *             if s[0] == 'x' and s[1].isdigit():
 *                 replaceDict[s[1]] = str(len(replaceDict))
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]             # <<<<<<<<<<<<<<
 *                 srcElements.append(newSymbol)
 *                 if s[1] != newSymbol[1]:
 */
  __pyx_slice__26 = PySlice_New(__pyx_int_2, Py_None, Py_None); if (unlikely(!__pyx_slice__26)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__26);
  __Pyx_GIVEREF(__pyx_slice__26);

  /* "lpu/smt/trans_models/records.pyx":437
 *         if not changed:
 *             return (src, trg)
 *         for s in trg.split(' '):             # <<<<<<<<<<<<<<
 *             if s[0] == 'x' and s[1].isdigit():
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "lpu/smt/trans_models/records.pyx":439
 *         for s in trg.split(' '):
 *             if s[0] == 'x' and s[1].isdigit():
 *                 newSymbol = 'x'+replaceDict[s[1]]+s[2:]             # <<<<<<<<<<<<<<
 *                 trgElements.append(newSymbol)
 *             else:
 */
  __pyx_slice__28 = PySlice_New(__pyx_int_2, Py_None, Py_None); if (unlikely(!__pyx_slice__28)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__28);
  __Pyx_GIVEREF(__pyx_slice__28);

  /* "lpu/smt/trans_models/records.pyx":449
 *     def getSymbols(rule, hiero=False):
 *         symbols = []
 *         for s in rule.split(' '):             # <<<<<<<<<<<<<<
 *           if len(s) < 2:
 *             if s == "@":
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "lpu/smt/trans_models/records.pyx":454
 *               break
 *           elif s[0] == '"' and s[-1] == '"':
 *             symbols.append(s[1:-1])             # <<<<<<<<<<<<<<
 *           elif s[0] == 'x' and s[1].isdigit():
 *             if len(s) > 3 and not hiero:
 */
  __pyx_slice__31 = PySlice_New(__pyx_int_1, __pyx_int_neg_1, Py_None); if (unlikely(!__pyx_slice__31)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__31);
  __Pyx_GIVEREF(__pyx_slice__31);

  /* "lpu/smt/trans_models/records.pyx":457
 *           elif s[0] == 'x' and s[1].isdigit():
 *             if len(s) > 3 and not hiero:
 *               symbols.append('[%s]' % s[3:])             # <<<<<<<<<<<<<<
 *             else:
 *               symbols.append('[X]')
 */
  __pyx_slice__32 = PySlice_New(__pyx_int_3, Py_None, Py_None); if (unlikely(!__pyx_slice__32)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__32);
  __Pyx_GIVEREF(__pyx_slice__32);

  /* "lpu/smt/trans_models/records.pyx":466
 *       terms = []
 *       if type(symbols) == str:
 *           symbols = symbols.split(' ')             # <<<<<<<<<<<<<<
 *       for s in symbols:
 *         if len(s) < 2:
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_kp_s__4); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "lpu/smt/trans_models/records.pyx":472
 *             break
 *         elif s[0] == '"' and s[-1] == '"':
 *           terms.append(s[1:-1])             # <<<<<<<<<<<<<<
 *       return terms
 * 
 */
  __pyx_slice__34 = PySlice_New(__pyx_int_1, __pyx_int_neg_1, Py_None); if (unlikely(!__pyx_slice__34)) __PYX_ERR(0, 472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__34);
  __Pyx_GIVEREF(__pyx_slice__34);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "lpu/smt/trans_models/records.pyx":481
 *         tag = symbols[0]
 *         for s in symbols:
 *             if s[0:1] == '"':             # <<<<<<<<<<<<<<
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":
 */
  __pyx_slice__37 = PySlice_New(__pyx_int_0, __pyx_int_1, Py_None); if (unlikely(!__pyx_slice__37)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__37);
  __Pyx_GIVEREF(__pyx_slice__37);

  /* "lpu/smt/trans_models/records.pyx":483
 *             if s[0:1] == '"':
 *                 newSymbols.append(s)
 *             elif s[0:1] == "x":             # <<<<<<<<<<<<<<
 *                 if grammar == 'scfg':
 *                     newSymbols.append(s)
 */
  __pyx_slice__38 = PySlice_New(__pyx_int_0, __pyx_int_1, Py_None); if (unlikely(!__pyx_slice__38)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__38);
  __Pyx_GIVEREF(__pyx_slice__38);

  /* "lpu/smt/trans_models/records.pyx":487
 *                     newSymbols.append(s)
 *                 else:
 *                     fields = s.split(':')             # <<<<<<<<<<<<<<
 *                     newSymbols.append(fields[0]+':X')
 *         newSymbols.append("@")
 */
  __pyx_tuple__40 = PyTuple_Pack(1, __pyx_kp_s__39); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "lpu/smt/trans_models/records.pyx":503
 *     cdef object val
 *     for key, val in dict_features.items():
 *         if key.find('egf') >= 0 or key.find('fge') >= 0:             # <<<<<<<<<<<<<<
 *             try:
 *                 #val = math.log(val)
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_n_s_egf); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_tuple__42 = PyTuple_Pack(1, __pyx_n_s_fge); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);

  /* "lpu/smt/trans_models/records.pyx":516
 *   for strKeyVal in field.split():
 *     #debug.log(strKeyVal)
 *     (key, val) = strKeyVal.split('=')             # <<<<<<<<<<<<<<
 *     val = getNumber(val)
 * #    if key in ['egfl', 'egfp', 'fgel', 'fgep']:
 */
  __pyx_tuple__44 = PyTuple_Pack(1, __pyx_kp_s__43); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);

  /* "lpu/smt/trans_models/records.pyx":545
 *             srcNonTerminals = []
 *             for s in src_symbols:
 *                 if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 *                     srcNonTerminals.append(s)
 * #                        if len(s) > 3:
 */
  __pyx_slice__46 = PySlice_New(__pyx_int_0, __pyx_int_1, Py_None); if (unlikely(!__pyx_slice__46)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__46);
  __Pyx_GIVEREF(__pyx_slice__46);
  __pyx_slice__47 = PySlice_New(__pyx_int_1, __pyx_int_2, Py_None); if (unlikely(!__pyx_slice__47)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__47);
  __Pyx_GIVEREF(__pyx_slice__47);

  /* "lpu/smt/trans_models/records.pyx":553
 *             for i, s in enumerate(trgSymbols):
 * #                    print("TRG SYMBOLS: %s" % s)
 *                 if s[0:1] == 'x' and s[1:2].isdigit():             # <<<<<<<<<<<<<<
 * #                        print("NONTERMINAL!")
 *                     if len(s) > 3:
 */
  __pyx_slice__48 = PySlice_New(__pyx_int_0, __pyx_int_1, Py_None); if (unlikely(!__pyx_slice__48)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__48);
  __Pyx_GIVEREF(__pyx_slice__48);
  __pyx_slice__49 = PySlice_New(__pyx_int_1, __pyx_int_2, Py_None); if (unlikely(!__pyx_slice__49)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__49);
  __Pyx_GIVEREF(__pyx_slice__49);

  /* "lpu/smt/trans_models/records.pyx":144
 * 
 *     @staticmethod
 *     def getTerms(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */
  __pyx_tuple__50 = PyTuple_Pack(1, __pyx_n_s_phrase); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getTerms, 144, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(0, 144, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":150
 * 
 *     @staticmethod
 *     def getSymbols(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */
  __pyx_tuple__52 = PyTuple_Pack(1, __pyx_n_s_phrase); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  __pyx_codeobj__53 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getSymbols, 150, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__53)) __PYX_ERR(0, 150, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":156
 * 
 * class MosesRecord(Record):
 *     def __init__(self, line = "", delim = '|||'):             # <<<<<<<<<<<<<<
 *         Record.__init__(self)
 *         self.delim = delim
 */
  __pyx_tuple__54 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_line, __pyx_n_s_delim); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);
  __pyx_codeobj__55 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__54, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_init, 156, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__55)) __PYX_ERR(0, 156, __pyx_L1_error)
  __pyx_tuple__56 = PyTuple_Pack(2, ((PyObject*)__pyx_kp_s__3), ((PyObject*)__pyx_kp_s__9)); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "lpu/smt/trans_models/records.pyx":161
 *         self.loadLine(line, delim)
 * 
 *     def loadLine(self, line, delim = '|||'):             # <<<<<<<<<<<<<<
 *         if line:
 *             fields = line.strip().split(delim)
 */
  __pyx_tuple__57 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_line, __pyx_n_s_delim, __pyx_n_s_fields, __pyx_n_s_listCounts); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);
  __pyx_codeobj__58 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__57, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_loadLine, 161, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__58)) __PYX_ERR(0, 161, __pyx_L1_error)
  __pyx_tuple__59 = PyTuple_Pack(1, ((PyObject*)__pyx_kp_s__9)); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);

  /* "lpu/smt/trans_models/records.pyx":188
 * #    trgTerms = property(getTrgTerms)
 * 
 *     def to_str(self, s = ' ||| '):             # <<<<<<<<<<<<<<
 *         strFeatures = getStrMosesFeatures(self.features)
 * #        strAligns = str.join(' ', self.aligns)
 */
  __pyx_tuple__60 = PyTuple_Pack(6, __pyx_n_s_self, __pyx_n_s_s, __pyx_n_s_strFeatures, __pyx_n_s_strAligns, __pyx_n_s_strCounts, __pyx_n_s_buf); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  __pyx_codeobj__61 = (PyObject*)__Pyx_PyCode_New(2, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__60, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_to_str, 188, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__61)) __PYX_ERR(0, 188, __pyx_L1_error)
  __pyx_tuple__62 = PyTuple_Pack(1, ((PyObject*)__pyx_kp_s__10)); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);

  /* "lpu/smt/trans_models/records.pyx":199
 *         return buf
 * 
 * def getAlignMap(aligns, reverse = False):             # <<<<<<<<<<<<<<
 *     alignMap = {}
 *     for align in aligns:
 */
  __pyx_tuple__63 = PyTuple_Pack(6, __pyx_n_s_aligns, __pyx_n_s_reverse, __pyx_n_s_alignMap, __pyx_n_s_align, __pyx_n_s_s, __pyx_n_s_t); if (unlikely(!__pyx_tuple__63)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);
  __pyx_codeobj__64 = (PyObject*)__Pyx_PyCode_New(2, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__63, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getAlignMap, 199, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__64)) __PYX_ERR(0, 199, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":209
 *     return alignMap
 * 
 * def getAlignSet(strField):             # <<<<<<<<<<<<<<
 *     return set(strField.strip().split())
 * 
 */
  __pyx_tuple__65 = PyTuple_Pack(1, __pyx_n_s_strField); if (unlikely(!__pyx_tuple__65)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);
  __pyx_codeobj__66 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__65, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getAlignSet, 209, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__66)) __PYX_ERR(0, 209, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":212
 *     return set(strField.strip().split())
 * 
 * def getCounts(field):             # <<<<<<<<<<<<<<
 * #    return map(getNumber, field.split())
 * #    return map(numbers.toNumber, field.split())
 */
  __pyx_tuple__67 = PyTuple_Pack(1, __pyx_n_s_field); if (unlikely(!__pyx_tuple__67)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);
  __pyx_codeobj__68 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__67, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getCounts, 212, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__68)) __PYX_ERR(0, 212, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":217
 *     return list(map(numbers.toNumber, field.split()))
 * 
 * def getNumber(anyNum, margin = 0):             # <<<<<<<<<<<<<<
 *     numFloat = float(anyNum)
 *     numInt = int(round(numFloat))
 */
  __pyx_tuple__69 = PyTuple_Pack(4, __pyx_n_s_anyNum, __pyx_n_s_margin, __pyx_n_s_numFloat, __pyx_n_s_numInt); if (unlikely(!__pyx_tuple__69)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);
  __pyx_codeobj__70 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__69, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getNumber, 217, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__70)) __PYX_ERR(0, 217, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":231
 * 
 * #def getRevAligns(aligns):
 * def getRevAlignSet(aligns):             # <<<<<<<<<<<<<<
 * #    revAlignList = []
 *     revAlignSet = set()
 */
  __pyx_tuple__71 = PyTuple_Pack(5, __pyx_n_s_aligns, __pyx_n_s_revAlignSet, __pyx_n_s_a, __pyx_n_s_s, __pyx_n_s_t); if (unlikely(!__pyx_tuple__71)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);
  __pyx_codeobj__72 = (PyObject*)__Pyx_PyCode_New(1, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__71, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getRevAlignSet, 231, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__72)) __PYX_ERR(0, 231, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":243
 * 
 * 
 * def getMosesFeatures(field):             # <<<<<<<<<<<<<<
 *     features = {}
 *     scores = map(getNumber, field.split())
 */
  __pyx_tuple__73 = PyTuple_Pack(3, __pyx_n_s_field, __pyx_n_s_features, __pyx_n_s_scores); if (unlikely(!__pyx_tuple__73)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);
  __pyx_codeobj__74 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__73, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getMosesFeatures, 243, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__74)) __PYX_ERR(0, 243, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":252
 *     return features
 * 
 * def getStrMosesFeatures(dicFeatures):             # <<<<<<<<<<<<<<
 *     '''convert back the feature dictionary to score string separated by space'''
 *     scores = []
 */
  __pyx_tuple__75 = PyTuple_Pack(2, __pyx_n_s_dicFeatures, __pyx_n_s_scores); if (unlikely(!__pyx_tuple__75)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);
  __pyx_codeobj__76 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__75, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getStrMosesFeatures, 252, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__76)) __PYX_ERR(0, 252, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":419
 * 
 *     @staticmethod
 *     def fixOrderStrings(str src, str trg):             # <<<<<<<<<<<<<<
 *         cdef list srcElements = []
 *         cdef list trgElements = []
 */
  __pyx_tuple__77 = PyTuple_Pack(8, __pyx_n_s_src, __pyx_n_s_trg, __pyx_n_s_srcElements, __pyx_n_s_trgElements, __pyx_n_s_replaceDict, __pyx_n_s_newSymbol, __pyx_n_s_changed, __pyx_n_s_s); if (unlikely(!__pyx_tuple__77)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);
  __pyx_codeobj__78 = (PyObject*)__Pyx_PyCode_New(2, 0, 8, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__77, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_fixOrderStrings, 419, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__78)) __PYX_ERR(0, 419, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":447
 * 
 *     @staticmethod
 *     def getSymbols(rule, hiero=False):             # <<<<<<<<<<<<<<
 *         symbols = []
 *         for s in rule.split(' '):
 */
  __pyx_tuple__79 = PyTuple_Pack(4, __pyx_n_s_rule, __pyx_n_s_hiero, __pyx_n_s_symbols, __pyx_n_s_s); if (unlikely(!__pyx_tuple__79)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__79);
  __Pyx_GIVEREF(__pyx_tuple__79);
  __pyx_codeobj__80 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__79, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getSymbols, 447, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__80)) __PYX_ERR(0, 447, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":463
 * 
 *     @staticmethod
 *     def getTerms(symbols):             # <<<<<<<<<<<<<<
 *       terms = []
 *       if type(symbols) == str:
 */
  __pyx_tuple__81 = PyTuple_Pack(3, __pyx_n_s_symbols, __pyx_n_s_terms, __pyx_n_s_s); if (unlikely(!__pyx_tuple__81)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);
  __pyx_codeobj__82 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__81, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getTerms, 463, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__82)) __PYX_ERR(0, 463, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":475
 *       return terms
 * 
 * def getFlattenSymbols(symbols, grammar = 'scfg'):             # <<<<<<<<<<<<<<
 *     if isTree(symbols):
 *         # strSymbols is tree (S-expression)
 */
  __pyx_tuple__83 = PyTuple_Pack(6, __pyx_n_s_symbols, __pyx_n_s_grammar, __pyx_n_s_newSymbols, __pyx_n_s_tag, __pyx_n_s_s, __pyx_n_s_fields); if (unlikely(!__pyx_tuple__83)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);
  __pyx_codeobj__84 = (PyObject*)__Pyx_PyCode_New(2, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__83, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getFlattenSymbols, 475, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__84)) __PYX_ERR(0, 475, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":512
 *     return str.join(' ', sorted(featureList))
 * 
 * def getTravatarFeatures(field):             # <<<<<<<<<<<<<<
 *   features = {}
 *   for strKeyVal in field.split():
 */
  __pyx_tuple__85 = PyTuple_Pack(5, __pyx_n_s_field, __pyx_n_s_features, __pyx_n_s_strKeyVal, __pyx_n_s_key, __pyx_n_s_val); if (unlikely(!__pyx_tuple__85)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);
  __pyx_codeobj__86 = (PyObject*)__Pyx_PyCode_New(1, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__85, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_getTravatarFeatures, 512, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__86)) __PYX_ERR(0, 512, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":527
 *   return features
 * 
 * def isTree(symbols):             # <<<<<<<<<<<<<<
 * #    if len(symbols) >= 4 and symbols[0][0] != '"':
 *     if len(symbols) >= 4 and symbols[1] == '(':
 */
  __pyx_tuple__87 = PyTuple_Pack(1, __pyx_n_s_symbols); if (unlikely(!__pyx_tuple__87)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);
  __pyx_codeobj__88 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__87, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_isTree, 527, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__88)) __PYX_ERR(0, 527, __pyx_L1_error)

  /* "lpu/smt/trans_models/records.pyx":534
 *         return False
 * 
 * def syncTags(src_symbols, trgSymbols, sync):             # <<<<<<<<<<<<<<
 *     if isTree(src_symbols):
 *         # src is tree (s-expression)
 */
  __pyx_tuple__89 = PyTuple_Pack(8, __pyx_n_s_src_symbols, __pyx_n_s_trgSymbols, __pyx_n_s_sync, __pyx_n_s_srcTag, __pyx_n_s_srcNonTerminals, __pyx_n_s_s, __pyx_n_s_i, __pyx_n_s_index); if (unlikely(!__pyx_tuple__89)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);
  __pyx_codeobj__90 = (PyObject*)__Pyx_PyCode_New(3, 0, 8, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__89, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_lpu_smt_trans_models_records_pyx, __pyx_n_s_syncTags, 534, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__90)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initrecords(void); /*proto*/
PyMODINIT_FUNC initrecords(void)
#else
PyMODINIT_FUNC PyInit_records(void); /*proto*/
PyMODINIT_FUNC PyInit_records(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_records(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("records", __pyx_methods, __pyx_k_classes_handling_rule_table_reco, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_lpu__smt__trans_models__records) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "lpu.smt.trans_models.records")) {
      if (unlikely(PyDict_SetItemString(modules, "lpu.smt.trans_models.records", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_3lpu_3smt_12trans_models_7records_CoOccurrence = &__pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence.calcEGFP = (float (*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcEGFP;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence.calcFGEP = (float (*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_calcFGEP;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence.getReversed = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_getReversed;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence.round = (void (*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_round *__pyx_optional_args))__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_round;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence.setCounts = (void (*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts *__pyx_optional_args))__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_setCounts;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_CoOccurrence.to_str = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_CoOccurrence *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str *__pyx_optional_args))__pyx_f_3lpu_3smt_12trans_models_7records_12CoOccurrence_to_str;
  if (PyType_Ready(&__pyx_type_3lpu_3smt_12trans_models_7records_CoOccurrence) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_type_3lpu_3smt_12trans_models_7records_CoOccurrence.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_3lpu_3smt_12trans_models_7records_CoOccurrence.tp_dict, __pyx_vtabptr_3lpu_3smt_12trans_models_7records_CoOccurrence) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "CoOccurrence", (PyObject *)&__pyx_type_3lpu_3smt_12trans_models_7records_CoOccurrence) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_3lpu_3smt_12trans_models_7records_CoOccurrence) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_ptype_3lpu_3smt_12trans_models_7records_CoOccurrence = &__pyx_type_3lpu_3smt_12trans_models_7records_CoOccurrence;
  __pyx_vtabptr_3lpu_3smt_12trans_models_7records_Record = &__pyx_vtable_3lpu_3smt_12trans_models_7records_Record;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_Record.fixOrder = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_6Record_fixOrder;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_Record.getAlignMap = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMap;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_Record.getAlignMapRev = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getAlignMapRev;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_Record.getReversed = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_6Record_getReversed;
  if (PyType_Ready(&__pyx_type_3lpu_3smt_12trans_models_7records_Record) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __pyx_type_3lpu_3smt_12trans_models_7records_Record.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_3lpu_3smt_12trans_models_7records_Record.tp_dict, __pyx_vtabptr_3lpu_3smt_12trans_models_7records_Record) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Record", (PyObject *)&__pyx_type_3lpu_3smt_12trans_models_7records_Record) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_3lpu_3smt_12trans_models_7records_Record) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __pyx_ptype_3lpu_3smt_12trans_models_7records_Record = &__pyx_type_3lpu_3smt_12trans_models_7records_Record;
  __pyx_vtabptr_3lpu_3smt_12trans_models_7records_TravatarRecord = &__pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord.__pyx_base = *__pyx_vtabptr_3lpu_3smt_12trans_models_7records_Record;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord.__pyx_base.fixOrder = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_fixOrder;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord.__pyx_base.getReversed = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_Record *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed__pyx_wrap_1;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord.getReversed = (struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_getReversed;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord.loadLine = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine *__pyx_optional_args))__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_loadLine;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord.syncTags = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, int __pyx_skip_dispatch))__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_syncTags;
  __pyx_vtable_3lpu_3smt_12trans_models_7records_TravatarRecord.to_str = (PyObject *(*)(struct __pyx_obj_3lpu_3smt_12trans_models_7records_TravatarRecord *, int __pyx_skip_dispatch, struct __pyx_opt_args_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str *__pyx_optional_args))__pyx_f_3lpu_3smt_12trans_models_7records_14TravatarRecord_to_str;
  __pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord.tp_base = __pyx_ptype_3lpu_3smt_12trans_models_7records_Record;
  if (PyType_Ready(&__pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  __pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord.tp_dict, __pyx_vtabptr_3lpu_3smt_12trans_models_7records_TravatarRecord) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "TravatarRecord", (PyObject *)&__pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  __pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord = &__pyx_type_3lpu_3smt_12trans_models_7records_TravatarRecord;
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "lpu/smt/trans_models/records.pyx":7
 * 
 * # Standard libraries
 * import math             # <<<<<<<<<<<<<<
 * 
 * # Local libraries
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_math, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_math, __pyx_t_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":10
 * 
 * # Local libraries
 * from lpu.common import compat             # <<<<<<<<<<<<<<
 * from lpu.common import logging
 * from lpu.common import numbers
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_compat);
  __Pyx_GIVEREF(__pyx_n_s_compat);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_compat);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_lpu_common, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_compat); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compat, __pyx_t_1) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":11
 * # Local libraries
 * from lpu.common import compat
 * from lpu.common import logging             # <<<<<<<<<<<<<<
 * from lpu.common import numbers
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_logging);
  __Pyx_GIVEREF(__pyx_n_s_logging);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_logging);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_lpu_common, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_2) < 0) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "lpu/smt/trans_models/records.pyx":12
 * from lpu.common import compat
 * from lpu.common import logging
 * from lpu.common import numbers             # <<<<<<<<<<<<<<
 * 
 * # Matching Method
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_numbers);
  __Pyx_GIVEREF(__pyx_n_s_numbers);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_numbers);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_lpu_common, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_numbers); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_numbers, __pyx_t_1) < 0) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":18
 * #   tag:     Tag Match (Phrase Structure Match)
 * #   string:  Symbol String Match (Superficial Match)
 * MATCH=['tree', 'tag', 'string', 'ftree']             # <<<<<<<<<<<<<<
 * 
 * cdef class CoOccurrence:
 */
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_tree);
  __Pyx_GIVEREF(__pyx_n_s_tree);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_tree);
  __Pyx_INCREF(__pyx_n_s_tag);
  __Pyx_GIVEREF(__pyx_n_s_tag);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_tag);
  __Pyx_INCREF(__pyx_n_s_string);
  __Pyx_GIVEREF(__pyx_n_s_string);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_string);
  __Pyx_INCREF(__pyx_n_s_ftree);
  __Pyx_GIVEREF(__pyx_n_s_ftree);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_ftree);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MATCH, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":91
 *     def getSrcSymbols(self):
 *         return self.getSymbols(self.src)
 *     src_symbols = property(getSrcSymbols)             # <<<<<<<<<<<<<<
 * 
 *     def getSrcTerms(self):
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record, __pyx_n_s_getSrcSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_src_symbols, __pyx_t_2) < 0) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":95
 *     def getSrcTerms(self):
 *         return self.getTerms(self.src)
 *     srcTerms = property(getSrcTerms)             # <<<<<<<<<<<<<<
 * 
 *     def getTrgSymbols(self):
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record, __pyx_n_s_getSrcTerms); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_srcTerms, __pyx_t_2) < 0) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":99
 *     def getTrgSymbols(self):
 *         return self.getSymbols(self.trg)
 *     trgSymbols = property(getTrgSymbols)             # <<<<<<<<<<<<<<
 * 
 *     def getTrgTerms(self):
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record, __pyx_n_s_getTrgSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_trgSymbols, __pyx_t_2) < 0) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":103
 *     def getTrgTerms(self):
 *         return self.getTerms(self.trg)
 *     trgTerms = property(getTrgTerms)             # <<<<<<<<<<<<<<
 * 
 *     cpdef getAlignMap(self):
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record, __pyx_n_s_getTrgTerms); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_trgTerms, __pyx_t_2) < 0) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":144
 * 
 *     @staticmethod
 *     def getTerms(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_6Record_19getTerms, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_getTerms, __pyx_t_2) < 0) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":143
 *         return recRev
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def getTerms(phrase):
 *         if isinstance(phrase, str):
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record, __pyx_n_s_getTerms); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_getTerms, __pyx_t_2) < 0) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":150
 * 
 *     @staticmethod
 *     def getSymbols(phrase):             # <<<<<<<<<<<<<<
 *         if isinstance(phrase, str):
 *             phrase = phrase.split(' ')
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_6Record_21getSymbols, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_getSymbols, __pyx_t_2) < 0) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":149
 *         return phrase
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def getSymbols(phrase):
 *         if isinstance(phrase, str):
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record, __pyx_n_s_getSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record->tp_dict, __pyx_n_s_getSymbols, __pyx_t_2) < 0) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_Record);

  /* "lpu/smt/trans_models/records.pyx":155
 *         return phrase
 * 
 * class MosesRecord(Record):             # <<<<<<<<<<<<<<
 *     def __init__(self, line = "", delim = '|||'):
 *         Record.__init__(self)
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_Record));
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_MosesRecord, __pyx_n_s_MosesRecord, (PyObject *) NULL, __pyx_n_s_lpu_smt_trans_models_records, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "lpu/smt/trans_models/records.pyx":156
 * 
 * class MosesRecord(Record):
 *     def __init__(self, line = "", delim = '|||'):             # <<<<<<<<<<<<<<
 *         Record.__init__(self)
 *         self.delim = delim
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_11MosesRecord_1__init__, 0, __pyx_n_s_MosesRecord___init, NULL, __pyx_n_s_lpu_smt_trans_models_records, __pyx_d, ((PyObject *)__pyx_codeobj__55)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__56);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "lpu/smt/trans_models/records.pyx":161
 *         self.loadLine(line, delim)
 * 
 *     def loadLine(self, line, delim = '|||'):             # <<<<<<<<<<<<<<
 *         if line:
 *             fields = line.strip().split(delim)
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_11MosesRecord_3loadLine, 0, __pyx_n_s_MosesRecord_loadLine, NULL, __pyx_n_s_lpu_smt_trans_models_records, __pyx_d, ((PyObject *)__pyx_codeobj__58)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__59);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_loadLine, __pyx_t_4) < 0) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "lpu/smt/trans_models/records.pyx":188
 * #    trgTerms = property(getTrgTerms)
 * 
 *     def to_str(self, s = ' ||| '):             # <<<<<<<<<<<<<<
 *         strFeatures = getStrMosesFeatures(self.features)
 * #        strAligns = str.join(' ', self.aligns)
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_11MosesRecord_5to_str, 0, __pyx_n_s_MosesRecord_to_str, NULL, __pyx_n_s_lpu_smt_trans_models_records, __pyx_d, ((PyObject *)__pyx_codeobj__61)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__62);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_to_str, __pyx_t_4) < 0) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "lpu/smt/trans_models/records.pyx":155
 *         return phrase
 * 
 * class MosesRecord(Record):             # <<<<<<<<<<<<<<
 *     def __init__(self, line = "", delim = '|||'):
 *         Record.__init__(self)
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_MosesRecord, __pyx_t_2, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MosesRecord, __pyx_t_4) < 0) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":199
 *         return buf
 * 
 * def getAlignMap(aligns, reverse = False):             # <<<<<<<<<<<<<<
 *     alignMap = {}
 *     for align in aligns:
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_1getAlignMap, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getAlignMap, __pyx_t_2) < 0) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":209
 *     return alignMap
 * 
 * def getAlignSet(strField):             # <<<<<<<<<<<<<<
 *     return set(strField.strip().split())
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_3getAlignSet, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getAlignSet, __pyx_t_2) < 0) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":212
 *     return set(strField.strip().split())
 * 
 * def getCounts(field):             # <<<<<<<<<<<<<<
 * #    return map(getNumber, field.split())
 * #    return map(numbers.toNumber, field.split())
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_5getCounts, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getCounts, __pyx_t_2) < 0) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":217
 *     return list(map(numbers.toNumber, field.split()))
 * 
 * def getNumber(anyNum, margin = 0):             # <<<<<<<<<<<<<<
 *     numFloat = float(anyNum)
 *     numInt = int(round(numFloat))
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_7getNumber, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getNumber, __pyx_t_2) < 0) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":231
 * 
 * #def getRevAligns(aligns):
 * def getRevAlignSet(aligns):             # <<<<<<<<<<<<<<
 * #    revAlignList = []
 *     revAlignSet = set()
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_9getRevAlignSet, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getRevAlignSet, __pyx_t_2) < 0) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":243
 * 
 * 
 * def getMosesFeatures(field):             # <<<<<<<<<<<<<<
 *     features = {}
 *     scores = map(getNumber, field.split())
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_11getMosesFeatures, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getMosesFeatures, __pyx_t_2) < 0) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":252
 *     return features
 * 
 * def getStrMosesFeatures(dicFeatures):             # <<<<<<<<<<<<<<
 *     '''convert back the feature dictionary to score string separated by space'''
 *     scores = []
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_13getStrMosesFeatures, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getStrMosesFeatures, __pyx_t_2) < 0) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":419
 * 
 *     @staticmethod
 *     def fixOrderStrings(str src, str trg):             # <<<<<<<<<<<<<<
 *         cdef list srcElements = []
 *         cdef list trgElements = []
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_14TravatarRecord_13fixOrderStrings, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord->tp_dict, __pyx_n_s_fixOrderStrings, __pyx_t_2) < 0) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord);

  /* "lpu/smt/trans_models/records.pyx":418
 *       return buf
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def fixOrderStrings(str src, str trg):
 *         cdef list srcElements = []
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord, __pyx_n_s_fixOrderStrings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord->tp_dict, __pyx_n_s_fixOrderStrings, __pyx_t_2) < 0) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord);

  /* "lpu/smt/trans_models/records.pyx":447
 * 
 *     @staticmethod
 *     def getSymbols(rule, hiero=False):             # <<<<<<<<<<<<<<
 *         symbols = []
 *         for s in rule.split(' '):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_14TravatarRecord_15getSymbols, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord->tp_dict, __pyx_n_s_getSymbols, __pyx_t_2) < 0) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord);

  /* "lpu/smt/trans_models/records.pyx":446
 *         return (str.join(' ',srcElements), str.join(' ',trgElements))
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def getSymbols(rule, hiero=False):
 *         symbols = []
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord, __pyx_n_s_getSymbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord->tp_dict, __pyx_n_s_getSymbols, __pyx_t_2) < 0) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord);

  /* "lpu/smt/trans_models/records.pyx":463
 * 
 *     @staticmethod
 *     def getTerms(symbols):             # <<<<<<<<<<<<<<
 *       terms = []
 *       if type(symbols) == str:
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_14TravatarRecord_17getTerms, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord->tp_dict, __pyx_n_s_getTerms, __pyx_t_2) < 0) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord);

  /* "lpu/smt/trans_models/records.pyx":462
 *         return symbols
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def getTerms(symbols):
 *       terms = []
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord, __pyx_n_s_getTerms); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord->tp_dict, __pyx_n_s_getTerms, __pyx_t_2) < 0) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3lpu_3smt_12trans_models_7records_TravatarRecord);

  /* "lpu/smt/trans_models/records.pyx":475
 *       return terms
 * 
 * def getFlattenSymbols(symbols, grammar = 'scfg'):             # <<<<<<<<<<<<<<
 *     if isTree(symbols):
 *         # strSymbols is tree (S-expression)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_15getFlattenSymbols, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getFlattenSymbols, __pyx_t_2) < 0) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":512
 *     return str.join(' ', sorted(featureList))
 * 
 * def getTravatarFeatures(field):             # <<<<<<<<<<<<<<
 *   features = {}
 *   for strKeyVal in field.split():
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_19getTravatarFeatures, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getTravatarFeatures, __pyx_t_2) < 0) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":527
 *   return features
 * 
 * def isTree(symbols):             # <<<<<<<<<<<<<<
 * #    if len(symbols) >= 4 and symbols[0][0] != '"':
 *     if len(symbols) >= 4 and symbols[1] == '(':
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_21isTree, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_isTree, __pyx_t_2) < 0) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":534
 *         return False
 * 
 * def syncTags(src_symbols, trgSymbols, sync):             # <<<<<<<<<<<<<<
 *     if isTree(src_symbols):
 *         # src is tree (s-expression)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_3lpu_3smt_12trans_models_7records_23syncTags, NULL, __pyx_n_s_lpu_smt_trans_models_records); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_syncTags, __pyx_t_2) < 0) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "lpu/smt/trans_models/records.pyx":1
 * # distutils: language=c++             # <<<<<<<<<<<<<<
 * # -*- coding: utf-8 -*-
 * 
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init lpu.smt.trans_models.records", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init lpu.smt.trans_models.records");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = PyThreadState_GET();
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
  static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ExtTypeTest */
  static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* GetModuleGlobalName */
  static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* RaiseException */
    #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* KeywordStringCheck */
      static CYTHON_INLINE int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_CheckExact(key)) && unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* Intern */
      static PyObject* __Pyx_Intern(PyObject* s) {
    if (!(likely(PyString_CheckExact(s)))) {
        PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(s)->tp_name);
        return 0;
    }
    Py_INCREF(s);
    #if PY_MAJOR_VERSION >= 3
    PyUnicode_InternInPlace(&s);
    #else
    PyString_InternInPlace(&s);
    #endif
    return s;
}

/* GetItemInt */
      static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* StringJoin */
      #if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* RaiseTooManyValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyObjectCallMethod1 */
      static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            Py_DECREF(method); method = NULL;
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* append */
      static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* PyObjectCallMethod2 */
      static PyObject* __Pyx_PyObject_CallMethod2(PyObject* obj, PyObject* method_name, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) return NULL;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method)) && likely(PyMethod_GET_SELF(method))) {
        PyObject *self, *function;
        self = PyMethod_GET_SELF(method);
        function = PyMethod_GET_FUNCTION(method);
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(function)) {
            PyObject *args[3] = {self, arg1, arg2};
            result = __Pyx_PyFunction_FastCall(function, args, 3);
            goto done;
        }
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(function)) {
            PyObject *args[3] = {self, arg1, arg2};
            result = __Pyx_PyFunction_FastCall(function, args, 3);
            goto done;
        }
        #endif
        args = PyTuple_New(3);
        if (unlikely(!args)) goto done;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        Py_INCREF(function);
        Py_DECREF(method);
        method = function;
    } else
#endif
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(method)) {
        PyObject *args[2] = {arg1, arg2};
        result = __Pyx_PyFunction_FastCall(method, args, 2);
        goto done;
    } else
#endif
#if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(method)) {
        PyObject *args[2] = {arg1, arg2};
        result = __Pyx_PyCFunction_FastCall(method, args, 2);
        goto done;
    } else
#endif
    {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto done;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
    }
    result = __Pyx_PyObject_Call(method, args, NULL);
    Py_DECREF(args);
done:
    Py_DECREF(method);
    return result;
}

/* dict_setdefault */
      static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value,
                                                       CYTHON_UNUSED int is_safe_type) {
    PyObject* value;
#if PY_VERSION_HEX >= 0x030400A0
    if ((1)) {
        value = PyDict_SetDefault(d, key, default_value);
        if (unlikely(!value)) return NULL;
        Py_INCREF(value);
#else
    if (is_safe_type == 1 || (is_safe_type == -1 &&
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
            (PyUnicode_CheckExact(key) || PyString_CheckExact(key) || PyLong_CheckExact(key)))) {
        value = PyDict_GetItemWithError(d, key);
        if (unlikely(!value)) {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            if (unlikely(PyDict_SetItem(d, key, default_value) == -1))
                return NULL;
            value = default_value;
        }
        Py_INCREF(value);
#else
            (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key) || PyLong_CheckExact(key)))) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            if (unlikely(PyDict_SetItem(d, key, default_value) == -1))
                return NULL;
            value = default_value;
        }
        Py_INCREF(value);
#endif
#endif
    } else {
        value = __Pyx_PyObject_CallMethod2(d, __pyx_n_s_setdefault, key, default_value);
    }
    return value;
}

/* pyobject_as_double */
          static double __Pyx__PyObject_AsDouble(PyObject* obj) {
    PyObject* float_value;
#if !CYTHON_USE_TYPE_SLOTS
    float_value = PyNumber_Float(obj);  if (0) goto bad;
#else
    PyNumberMethods *nb = Py_TYPE(obj)->tp_as_number;
    if (likely(nb) && likely(nb->nb_float)) {
        float_value = nb->nb_float(obj);
        if (likely(float_value) && unlikely(!PyFloat_Check(float_value))) {
            PyErr_Format(PyExc_TypeError,
                "__float__ returned non-float (type %.200s)",
                Py_TYPE(float_value)->tp_name);
            Py_DECREF(float_value);
            goto bad;
        }
    } else if (PyUnicode_CheckExact(obj) || PyBytes_CheckExact(obj)) {
#if PY_MAJOR_VERSION >= 3
        float_value = PyFloat_FromString(obj);
#else
        float_value = PyFloat_FromString(obj, 0);
#endif
    } else {
        PyObject* args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        PyTuple_SET_ITEM(args, 0, obj);
        float_value = PyObject_Call((PyObject*)&PyFloat_Type, args, 0);
        PyTuple_SET_ITEM(args, 0, 0);
        Py_DECREF(args);
    }
#endif
    if (likely(float_value)) {
        double value = PyFloat_AS_DOUBLE(float_value);
        Py_DECREF(float_value);
        return value;
    }
bad:
    return (double)-1;
}

/* ArgTypeTest */
          static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

/* BytesEquals */
          static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
          static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* SliceObject */
          static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyIntBinop */
          #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* UnpackUnboundCMethod */
          static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(PyObject_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST);
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
          static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_items */
          static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_items, d);
    else
        return PyDict_Items(d);
}

/* SaveResetException */
          #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
          #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SetVTable */
            static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
            static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_COMPILING_IN_CPYTHON
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* Import */
            static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
            static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* GetNameInClass */
            static PyObject *__Pyx_GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result)
        result = __Pyx_GetModuleGlobalName(name);
    return result;
}

/* CalculateMetaclass */
            static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
            static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
            static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CyFunctionObject, func.m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0 = PySequence_ITEM(arg, 0);
                if (unlikely(!arg0)) return NULL;
                result = (*meth)(self, arg0);
                Py_DECREF(arg0);
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (__pyx_CyFunctionType == NULL) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (!m->defaults)
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* Py3ClassCreate */
                static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
                static int __Pyx_CLineForTraceback(int c_line) {
#ifdef CYTHON_CLINE_IN_TRACEBACK
    return ((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0;
#else
    PyObject *use_cline;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *ptype, *pvalue, *ptraceback;
      PyObject *use_cline_obj;
      PyErr_Fetch(&ptype, &pvalue, &ptraceback);
      use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        use_cline = NULL;
      }
      PyErr_Restore(ptype, pvalue, ptraceback);
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    return c_line;
#endif
}

/* CodeObjectCache */
                static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
                #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
                #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
                static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CheckBinaryVersion */
                static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
                static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
  #else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
  #endif
#else
  res = PyNumber_Int(x);
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
